Package: fuseMLR
Type: Package
Title: Fusing Machine Learning in R
Version: 0.0.4
Authors@R: c(person(given = c("Cesaire", "J."),
                      family = c("K.", "Fouodo"),
                      role = c("aut", "cre"),
                      email = "cesaire.joris.kuete.fouodo@emory.edu"),
             person(given = c("Marina"),
                      family = c("Bleskina"),
                      role = c("ctb"),
                      email = "marina.bleskina@uni-luebeck.de")
            )
Maintainer: Cesaire J. K. Fouodo <cesaire.joris.kuete.fouodo@emory.edu>
Description: Recent technological advances have enable the simultaneous collection
    of multi-omics data i.e., different types or modalities of molecular data, 
    presenting challenges for integrative prediction modeling due to the heterogeneous,
    high-dimensional nature and possible missing modalities of some individuals. 
    We introduce this package for late integrative prediction modeling, enabling 
    modality-specific variable selection and prediction modeling, followed by the 
    aggregation of the modality-specific predictions to train a final meta-model. 
    This package facilitates conducting late integration predictive modeling in a 
    systematic, structured, and reproducible way.
License: GPL-3
Encoding: UTF-8
Imports: R6, stats, digest
Suggests: testthat (>= 3.0.0), UpSetR (>= 1.4.0), caret, ranger,
        glmnet, Boruta, knitr, rmarkdown, pROC, checkmate
Config/testthat/edition: 3
Depends: R (>= 3.6.0)
RoxygenNote: 7.3.3
Collate: 'Data.R' 'HashTable.R' 'Lrner.R' 'Model.R' 'PredictData.R'
        'PredictLayer.R' 'PredictMetaLayer.R' 'Predicting.R' 'Target.R'
        'TestData.R' 'TestLayer.R' 'TestMetaLayer.R' 'Testing.R'
        'TrainData.R' 'TrainLayer.R' 'TrainMetaLayer.R' 'Training.R'
        'VarSel.R' 'bestLayerLearner.R' 'cobra.R' 'createCobraPred.R'
        'createDif.R' 'createLoss.R' 'createWeights.R' 'multi_omics.R'
        'predict.bestLayerLearner.R' 'predict.cobra.R'
        'weightedMeanLearner.R' 'predict.weightedMeanLearner.R'
        'testingFunctions.R' 'trainingFunctions.R'
VignetteBuilder: knitr, rmarkdown
BugReports: https://github.com/imbs-hl/fuseMLR/issues
NeedsCompilation: no
Packaged: 2026-05-19 22:00:08 UTC; fouodo
Author: Cesaire J. K. Fouodo [aut, cre],
  Marina Bleskina [ctb]
Repository: CRAN
Date/Publication: 2026-05-20 06:40:02 UTC
Built: R 4.5.2; ; 2026-05-20 07:38:35 UTC; unix

Package: fuseMLR
Type: Package
Title: Fusing Machine Learning in R
Version: 0.0.4
Authors@R: c(person(given = c("Cesaire", "J."),
                      family = c("K.", "Fouodo"),
                      role = c("aut", "cre"),
                      email = "cesaire.joris.kuete.fouodo@emory.edu"),
             person(given = c("Marina"),
                      family = c("Bleskina"),
                      role = c("ctb"),
                      email = "marina.bleskina@uni-luebeck.de")
            )
Maintainer: Cesaire J. K. Fouodo <cesaire.joris.kuete.fouodo@emory.edu>
Description: Recent technological advances have enable the simultaneous collection
    of multi-omics data i.e., different types or modalities of molecular data, 
    presenting challenges for integrative prediction modeling due to the heterogeneous,
    high-dimensional nature and possible missing modalities of some individuals. 
    We introduce this package for late integrative prediction modeling, enabling 
    modality-specific variable selection and prediction modeling, followed by the 
    aggregation of the modality-specific predictions to train a final meta-model. 
    This package facilitates conducting late integration predictive modeling in a 
    systematic, structured, and reproducible way.
License: GPL-3
Encoding: UTF-8
Imports: R6, stats, digest
Suggests: testthat (>= 3.0.0), UpSetR (>= 1.4.0), caret, ranger,
        glmnet, Boruta, knitr, rmarkdown, pROC, checkmate
Config/testthat/edition: 3
Depends: R (>= 3.6.0)
RoxygenNote: 7.3.3
Collate: 'Data.R' 'HashTable.R' 'Lrner.R' 'Model.R' 'PredictData.R'
        'PredictLayer.R' 'PredictMetaLayer.R' 'Predicting.R' 'Target.R'
        'TestData.R' 'TestLayer.R' 'TestMetaLayer.R' 'Testing.R'
        'TrainData.R' 'TrainLayer.R' 'TrainMetaLayer.R' 'Training.R'
        'VarSel.R' 'bestLayerLearner.R' 'cobra.R' 'createCobraPred.R'
        'createDif.R' 'createLoss.R' 'createWeights.R' 'multi_omics.R'
        'predict.bestLayerLearner.R' 'predict.cobra.R'
        'weightedMeanLearner.R' 'predict.weightedMeanLearner.R'
        'testingFunctions.R' 'trainingFunctions.R'
VignetteBuilder: knitr, rmarkdown
BugReports: https://github.com/imbs-hl/fuseMLR/issues
NeedsCompilation: no
Packaged: 2026-05-19 22:00:08 UTC; fouodo
Author: Cesaire J. K. Fouodo [aut, cre],
  Marina Bleskina [ctb]
Repository: CRAN
Date/Publication: 2026-05-20 06:40:02 UTC
Built: R 4.6.0; ; 2026-05-20 08:16:23 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(predict,Training)
S3method(predict,bestLayerLearner)
S3method(predict,cobra)
S3method(predict,weightedMeanLearner)
S3method(summary,Testing)
S3method(summary,Training)
export(Data)
export(HashTable)
export(Lrner)
export(Model)
export(PredictData)
export(PredictLayer)
export(PredictMetaLayer)
export(Predicting)
export(Target)
export(TestData)
export(TestLayer)
export(TestMetaLayer)
export(Testing)
export(TrainData)
export(TrainLayer)
export(TrainMetaLayer)
export(Training)
export(VarSel)
export(bestLayerLearner)
export(cobra)
export(createTestLayer)
export(createTesting)
export(createTrainLayer)
export(createTrainMetaLayer)
export(createTraining)
export(extractData)
export(extractModel)
export(fusemlr)
export(upsetplot)
export(varSelection)
export(weightedMeanLearner)
importFrom(R6,R6Class)
importFrom(digest,digest)
importFrom(stats,complete.cases)
importFrom(stats,weighted.mean)

<!-- badges: start -->

[![R-CMD-check](https://github.com/imbs-hl/fuseMLR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/imbs-hl/fuseMLR/actions/workflows/R-CMD-check.yaml)
[![codecov](https://codecov.io/github/imbs-hl/fuseMLR/graph/badge.svg?token=SZU7NGK8G8)](https://app.codecov.io/github/imbs-hl/fuseMLR/)
[![Lifecycle:
Stable](https://img.shields.io/badge/lifecycle-Stable-green.svg)](https://lifecycle.r-lib.org/articles/stages.html#Stable)
[![CRAN
Status](https://img.shields.io/badge/CRAN-fuseMLR-blue)](https://cran.r-project.org/package=fuseMLR)

<!-- badges: end -->

### fuseMLR

Cesaire J. K. Fouodo

### Introduction

Recent technological advances have enabled the simultaneous collection
of multi-omics data, i.e., different types or modalities of molecular
data across various organ tissues of patients. For integrative
predictive modeling, the analysis of such data is particularly
challenging. Ideally, data from the different modalities are measured in
the same individuals, allowing for early or intermediate integrative
techniques. However, they are often not applicable when patient data
only partially overlap, which requires either reducing the datasets or
imputing missing values. Additionally, the characteristics of each data
modality may necessitate specific statistical methods rather than
applying the same method across all modalities. Late integration
modeling approaches analyze each data modality separately to obtain
modality-specific predictions. These predictions are then integrated to
train aggregative models like Lasso, random forests, or compute the
weighted mean of modality-specific predictions. 

We introduce the R package fuseMLR for late integration prediction
modeling. This comprehensive package enables users to define a training
process with multiple data modalities and modality-specific machine
learning methods. The package is user-friendly, facilitates variable
selection and training of different models across modalities, and
automatically performs aggregation once modality-specific training is
completed. We simulated multi-omics data to illustrate the usage of our
new package for conducting late-stage multi-omics integrative modeling.

`fuseMLR` is an object-oriented package based on `R6` version 2.5.1.
Refer to the vignette (section *Usage* below) for a quick overview of
how to use the package.

The following figure illustrates the general architecture of `fuseMLR`:

<img src="README_files/figure-gfm/structure.png" width="70%" height="100%" />

The general architecture of `fuseMLR` includes the collection classes
`Training`, `TrainLayer`, and `TrainMetaLayer`. `TrainLayer` and
`TrainMetaLayer` are stored within a `Training` instance, while
`TrainData`, `Lrner`, and `VarSel` (for variable selection) are stored
within a `TrainLayer` or `MetaTrainLayer` instance. An `Training` object
can be used to automatically conduct layer-specific variable selection
and train layer-specfic learner and the meta-learner. Analogously, a
`Testing` can be set up and predicted after the training.

### Installation

Install the release version (v0.0.1) from CRAN with

``` r
install.packages("fuseMLR")
```

Install the development version from GitHub with

``` r
devtools::install_github("imbs-hl/fuseMLR")
```

### Usage

Refer to our vignette to understand [how fuseMLR
works](https://CRAN.R-project.org/package=fuseMLR).

© 2024 Institute of Medical Biometry and Statistics (IMBS). All rights
reserved.

### Reference

Fouodo, C. J. K., Bleskina, M. & Szymczak, S. fuseMLR: an R package for
integrative prediction modeling of multi-omics data. BMC Bioinformatics
26, 221 (2025). <https://doi.org/10.1186/s12859-025-06248-4>

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----libraries, warning = FALSE-----------------------------------------------
library(fuseMLR)

## ----data_exam, include=TRUE, eval=TRUE---------------------------------------
data("multi_omics")
# This list contains two lists of data: training and test.
# Each sublist contains three omics data as described above.
str(object = multi_omics, max.level = 2L)

## ----training, include=TRUE, eval=TRUE----------------------------------------
training <- createTraining(id = "training",
                           ind_col = "IDS",
                           target = "disease",
                           target_df = multi_omics$training$target,
                           verbose = TRUE)
print(training)

## ----geneexpr, include=TRUE, eval=TRUE----------------------------------------
# Create gene expression layer.
createTrainLayer(training = training,
                 train_layer_id = "geneexpr",
                 train_data = multi_omics$training$geneexpr,
                 varsel_package = "Boruta",
                 varsel_fct = "Boruta",
                 varsel_param = list(num.trees = 1000L,
                                     mtry = 3L,
                                     probability = TRUE,
                                     getImp = Boruta::getImpRfZ),
                 lrner_package = "ranger",
                 lrn_fct = "ranger",
                 param_train_list = list(probability = TRUE,
                                         mtry = 1L),
                 param_pred_list = list(),
                 na_action = "na.keep")

## ----proteinexpr, include=TRUE, eval=TRUE-------------------------------------
# Create gene protein abundance layer
createTrainLayer(training = training,
                 train_layer_id = "proteinexpr",
                 train_data = multi_omics$training$proteinexpr,
                 varsel_package = "Boruta",
                 varsel_fct = "Boruta",
                 varsel_param = list(num.trees = 1000L,
                                     mtry = 3L,
                                     probability = TRUE,
                                     getImp = Boruta::getImpRfZ),
                 lrner_package = "ranger",
                 lrn_fct = "ranger",
                 param_train_list = list(probability = TRUE,
                                         mtry = 1L),
                 param_pred_list = list(type = "response"),
                 na_action = "na.keep")

## ----methylation, include=TRUE, eval=TRUE-------------------------------------
# Create methylation layer
createTrainLayer(training = training,
                 train_layer_id = "methylation",
                 train_data = multi_omics$training$methylation,
                 varsel_package = "Boruta",
                 varsel_fct = "Boruta",
                 varsel_param = list(num.trees = 1000L,
                                     mtry = 3L,
                                     probability = TRUE,
                                     getImp = Boruta::getImpRfZ),
                 lrner_package = "ranger",
                 lrn_fct = "ranger",
                 param_train_list = list(probability = TRUE,
                                         mtry = 1L),
                 param_pred_list = list(),
                 na_action = "na.keep")

## ----training_meta_layers, include=TRUE, eval=TRUE, class.output="scroll-100"----
# Create meta layer with imputation of missing values.
createTrainMetaLayer(training = training,
                     meta_layer_id = "meta_layer",
                     lrner_package = NULL,
                     lrn_fct = "weightedMeanLearner",
                     param_train_list = list(),
                     param_pred_list = list(na_rm = TRUE),
                     na_action = "na.rm")
print(training)

## ----upsetplot, include=TRUE, eval=TRUE, fig.width=5, fig.align='center'------
upsetplot(object = training, order.by = "freq")

## ----varsel, include=TRUE, eval=TRUE, warning=FALSE---------------------------
# Variable selection
set.seed(5467)
var_sel_res <- varSelection(training = training)
print(var_sel_res)

## ----training_varsel_disp, include=TRUE, eval=TRUE, warning=FALSE-------------
print(training)

## ----lrner_train, include=TRUE, eval=TRUE, message=TRUE-----------------------
set.seed(5462)
fusemlr(training = training,
        use_var_sel = TRUE)

## ----display_lrner_trained, include=TRUE, eval=TRUE, message=TRUE-------------
print(training)

## ----training_summary, include=TRUE, eval=TRUE, message=TRUE------------------
summary(training)

## ----basic_lrnr, include=TRUE, eval=TRUE--------------------------------------
models_list <- extractModel(training = training)
str(object = models_list, max.level = 1L)

## ----basic_data, include=TRUE, eval=TRUE--------------------------------------
data_list <- extractData(object = training)
str(object = data_list, max.level = 1)

## ----testing, include=TRUE, eval=TRUE-----------------------------------------
# Create testing for predictions
testing <- createTesting(id = "testing",
                         ind_col = "IDS")

## ----testing_ge, include=TRUE, eval=TRUE--------------------------------------
# Create gene expression layer
createTestLayer(testing = testing,
                test_layer_id = "geneexpr",
                test_data = multi_omics$testing$geneexpr)

## ----testing_pr, include=TRUE, eval=TRUE--------------------------------------
# Create gene protein abundance layer
createTestLayer(testing = testing,
                test_layer_id = "proteinexpr",
                test_data = multi_omics$testing$proteinexpr)

## ----testing_me, include=TRUE, eval=TRUE--------------------------------------
# Create methylation layer
createTestLayer(testing = testing,
                test_layer_id = "methylation",
                test_data = multi_omics$testing$methylation)

## ----testing_summary, include=TRUE, eval=TRUE, message=TRUE-------------------
summary(testing)

## ----basic_test_data, include=TRUE, eval=TRUE---------------------------------
data_list <- extractData(object = testing)
str(object = data_list, max.level = 1)

## ----upsetplot_new, include=TRUE, eval=TRUE, fig.width=5, fig.align='center'----
upsetplot(object = testing, order.by = "freq")

## ----new_pred, include=TRUE, eval=TRUE----------------------------------------
predictions <- predict(object = training, testing = testing)
print(predictions)

## ----brier, include=TRUE, eval=TRUE, message = FALSE--------------------------
pred_values <- predictions$predicted_values
actual_pred <- merge(x = pred_values,
                     y = multi_omics$testing$target,
                     by = "IDS",
                     all.y = TRUE)
y <- as.numeric(actual_pred$disease == "1")

# On all patients
perf_bs <- sapply(X = actual_pred[ , 2L:5L], FUN = function (my_pred) {
  bs <- mean((y[complete.cases(my_pred)] - my_pred[complete.cases(my_pred)])^2)
  roc_obj <- pROC::roc(y[complete.cases(my_pred)], my_pred[complete.cases(my_pred)])
  auc <- pROC::auc(roc_obj)
  performances = rbind(bs, auc)
  return(performances)
})
rownames(perf_bs) <- c("BS", "AUC")
print(perf_bs)

## ----interface, include=TRUE, eval=FALSE--------------------------------------
# # Re-create the gene expression layer with support vector machine as learner.
# createTrainLayer(training = training,
#                  train_layer_id = "geneexpr",
#                  train_data = multi_omics$training$geneexpr,
#                  varsel_package = "Boruta",
#                  varsel_fct = "Boruta",
#                  varsel_param = list(num.trees = 1000L,
#                                      mtry = 3L,
#                                      probability = TRUE,
#                                      getImp = Boruta::getImpRfZ),
#                  lrner_package = "e1071",
#                  lrn_fct = "svm",
#                  param_train_list = list(type = 'C-classification',
#                                          kernel = 'radial',
#                                          probability = TRUE),
#                  param_pred_list = list(probability = TRUE),
#                  na_action = "na.rm",
#                  x_lrn = "x",
#                  y_lrn = "y",
#                  object = "object",
#                  data = "newdata", # Name discrepancy resolved.
#                  extract_pred_fct = function (pred) {
#                    pred <- attr(pred, "probabilities")
#                    return(pred[ , 1L])
#                  }
# )
# # Variable selection
# set.seed(5467)
# var_sel_res <- varSelection(training = training)
# set.seed(5462)
# training <- fusemlr(training = training,
#                     use_var_sel = TRUE)
# 
# print(training)

## ----wrap_lasso, include=TRUE, eval=FALSE-------------------------------------
# # We wrap the original functions
# mylasso <- function (x, y,
#                      nlambda = 25,
#                      nfolds = 5) {
#   # Perform cross-validation to find the optimal lambda
#   cv_lasso <- glmnet::cv.glmnet(x = as.matrix(x), y = y,
#                         family = "binomial",
#                         type.measure = "deviance",
#                         nfolds = nfolds)
#   best_lambda <- cv_lasso$lambda.min
#   lasso_best <- glmnet::glmnet(x = as.matrix(x), y = y,
#                        family = "binomial",
#                        alpha = 1,
#                        lambda = best_lambda
#   )
#   lasso_model <- list(model = lasso_best)
#   class(lasso_model) <- "mylasso"
#   return(lasso_model)
# }

## ----wrap_predict, include=TRUE, eval=FALSE-----------------------------------
# # We extend the generic predict function mylasso.
# predict.mylasso <- function (object, data) {
#   glmnet_pred <- predict(object = object$model,
#                          newx = as.matrix(data),
#                          type = "response",
#                          s = object$model$lambda)
#   return(as.vector(glmnet_pred))
# }
# 
# # Re-create the gene expression layer with support vector machine as learner.
# createTrainMetaLayer(training = training,
#                      meta_layer_id = "meta_layer",
#                      lrner_package = NULL,
#                      lrn_fct = "mylasso",
#                      param_train_list = list(nlambda = 100L),
#                      na_action = "na.impute")
# set.seed(5462)
# training <- fusemlr(training = training,
#                     use_var_sel = TRUE)
# print(training)

## ----add_wrap, include=TRUE, eval=FALSE---------------------------------------
# # Re-create the gene expression layer with support vector machine as learner.
# createTrainMetaLayer(training = training,
#                      meta_layer_id = "meta_layer",
#                      lrner_package = NULL,
#                      lrn_fct = "mylasso",
#                      param_train_list = list(nlambda = 100L),
#                      na_action = "na.impute")
# set.seed(5462)
# training <- fusemlr(training = training,
#                     use_var_sel = TRUE)
# print(training)

## ----implemented_learners, include=TRUE, echo=FALSE---------------------------
# Load knitr package
library(knitr)

# Create a data frame
data <- data.frame(
  Leaner = c("weightedMeanLearner", "bestLayerLearner", "cobra"),
  Description = c("The weighted mean meta learner. It uses modality-specific predictions to estimate the weights of the modality-specific models", "The best layer-specific model is used as meta model.", "cobra implements the COBRA (COmBined Regression Alternative), an aggregation method for combining predictions from multiple individual learners")
)

# Generate the table
kable(data, caption = "")

#' @title The weighted mean meta-learner
#'
#' @description
#' Modality-specific learner are assessed and weighted based on their predictions. This function is intended to be (internally) used as meta-learner in fuseMLR.
#'
#' @param x `data.frame` \cr
#' Modality-specific predictions. Each column of the `data.frame` content the predictions
#' a specific learner.
#' @param y `vector` \cr
#' True target values. If classification, either binary or two level factor variable.
#' @param weighted `boolean` \cr
#' If TRUE, a weighted sum is computed. As default, weights are estimated based on Brier Score
#' for classification setting and mean squared error for regression. Otherwise, use argument
#' `perf` below to specify the function to use estimate learner performance.
#' @param perf `function` \cr
#' Function to compute layer-specific performance of learners. If NULL, the Brier Score (classification) or a mean squared error (regression) is used by default as performance measure.
#' Otherwise, the performance function must accept two parameters: \code{observed} (observed values) and \code{predicted} (predicted values).
#' @param na_rm `boolean` \cr
#' Should missing values be removed when computing the weights?
#' @return
#' Object of class \code{weightedMeanLearner} with the vector of estimated weights pro layer.\cr
#'
#' @export
#' @examples
#' set.seed(20240624L)
#' x = data.frame(x1 = runif(n = 50L, min = 0, max = 1),
#'                x2 = runif(n = 50L, min = 0, max = 1))
#' y = sample(x = 0L:1L, size = 50L, replace = TRUE)
#' my_model = weightedMeanLearner(x = x, y = y)
#'
weightedMeanLearner = function (x,
                                y,
                                weighted = TRUE,
                                perf = NULL,
                                na_rm = FALSE) {
  if (!is.data.frame(x)) {
    # nocov start
    stop("x must be a data.frame.")
    # nocov end
  }
  if (is.null(perf)) {
    # y must be binomial. If dichotomy, first category (case) = 1 and
    # second (control) = 0
    if (is.numeric(y) & (length(unique(y)) > 2)) {
      score_values = lapply(X = x, FUN = function (predicted) {
        mean(x = (predicted - y)^2, na.rm = na_rm)
      })
    } else {
      if ((length(unique(y)) > 2) | is.character(y)) {
        # nocov start
        stop("y must be either binary or two level factor variable.\n")
        # nocov end
      } else {
        if (is.factor(y)) {
          y = as.integer(y) - 1
        } else {
          if (!all(y %in% 0:1)) {
            # nocov start
            stop("y must take its values between 0 and 1.\n")
            # nocov end
          }
        }
        score_values = lapply(X = x, FUN = function (predicted) {
          mean(x = (predicted - y)^2, na.rm = na_rm)
        })
      }
    }
    score_values = unlist(score_values)
  } else {
    # nocov start
    if (is.function(perf)) {
      arg_names <- names(formals(perf))
      if (arg_names %in% c("observed", "predicted")) {
        # Function has been provided to estimated performance of layer-specific learner
        score_values = lapply(X = x, FUN = function (predicted) {
          perf_estimate = do.call(what = perf, args = list(observed = y,
                                                           predicted = predicted))
          return(perf_estimate)
        })
        score_values = unlist(score_values)
      } else {
        stop("perf argument must be a function.")
      }
    } else {
      stop("Arguments of the perf function must be 'observed' and 'predicted'.")
    }
  }
  # nocov end
  # weights_values = (1 - score_values) / sum((1 - score_values))
  if (weighted) {
    weights_values = (1 / score_values) / sum((1 / score_values))
  } else {
    weights_values <- rep(1 / length(score_values), length(score_values))
  }
  names(weights_values) = names(x)
  class(weights_values) = "weightedMeanLearner"
  return(weights_values)
}

test_that("weightedMeanLearner works", {
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = sample(x = 0L:1L, size = 50L, replace = TRUE)
    my_model = weightedMeanLearner(x = x, y = y)
    x_new <- data.frame(x1 = rnorm(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = sample(x = 0L:1L, size = 50L, replace = TRUE)
    my_model = weightedMeanLearner(x = x, y = y, weighted = FALSE)
    x_new <- data.frame(x1 = rnorm(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })
  expect_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = sample(x = 0L:1L, size = 50L, replace = TRUE)
    my_model = weightedMeanLearner(x = x, y = y)
    x_new <- data.frame(x2 = rnorm(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = sample(x = 0L:2L, size = 50L, replace = TRUE)
    my_model = weightedMeanLearner(x = x, y = y)
  })
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = factor(sample(x = c("control", "case"), size = 50L, replace = TRUE))
    my_model = weightedMeanLearner(x = x, y = y)
  })
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = sample(x = c("0", "1"), size = 50L, replace = TRUE)
    my_model = weightedMeanLearner(x = x, y = factor(y))
  })
})

#' @title Training Class
#'
#' @description
#' This is a primary classes of fuseMLR. An object from this class
#' is designed to contain multiple training layers, but only one meta training layer.
#'
#'  The Training class is structured as followed:
#' * [TrainLayer]: Specific layer containing:
#'    - [Lrner]: Specific learner. This must be set by the user.
#'    - [TrainData]: Specific training dataset. This must be set up by the user.
#'    - [Model]: Specific model. This is set up by training the learner on the training data.
#' * [TrainMetaLayer]: Basically a [TrainLayer], but with some specific properties.
#'    - [Lrner]: This is the meta learner, it must be set up by the user.
#'    - [TrainData]: Specific modality-specific prediction data. This is set up internally after cross-validation.
#'    - [Model]: Specific meta model. This is set up by training the learner on the training data.
#'
#' Use the function \code{train} for training and \code{predict} for predicting.
#'
#' @export
#'
#' @importFrom R6 R6Class
#'
#' @seealso [TrainLayer]
Training <- R6Class("Training",
                    inherit = HashTable,
                    public = list(
                      #' @description
                      #' constructor
                      #'
                      #' @param id `character` \cr
                      #' @param ind_col `character` \cr
                      #' Name of column of individuals IDS.
                      #' @param target `character` \cr
                      #' Name of the target variable.
                      #' @param target_df `data.frame` \cr
                      #' Data frame with two columns: individual IDs and response variable values.
                      #' @param problem_type `character` \cr
                      #' Either "classification" or "regression".
                      #' @param verbose `boolean` \cr
                      #' Warning messages will be displayed if set to TRUE.
                      #' @seealso [Testing] and [Predicting]
                      initialize = function (id,
                                             ind_col,
                                             target,
                                             target_df,
                                             problem_type = "classification",
                                             verbose = TRUE) {
                        if (!is.logical(verbose)) {
                          stop("'verbose' must be bolean.")
                        }
                        if (!is.character(id)) {
                          stop("id must be a string")
                        }
                        if (!is.character(ind_col)) {
                          stop("ind_col must be a string")
                        }
                        if (!is.character(target)) {
                          stop("target must be a string")
                        }
                        if (!is.data.frame(target_df)) {
                          stop("target_df must be a data.frame.")
                        }
                        if ((ncol(target_df) != 2L)) {
                          stop("target_df must be a data.frame with two columns: individual IDs and response variable values.")
                        }
                        if (!all(c(ind_col, target) %in% names(target_df))) {
                          stop(sprintf("%s and %s not foung in target_df.",
                                       ind_col, target))
                        }
                        if (!(problem_type %in% c("classification", "regression"))) {
                          stop("The problem type must be either classification or regression")
                        } else {
                          target_values = target_df[ , target, drop = TRUE]
                          uniq_target_values = length(unique(target_values[complete.cases(target_values)]))
                          if ((problem_type == "classification") & (uniq_target_values > 2L)) {
                            stop("You set up a classification problem for more than two classes. Only binary classification problems are actually handled.\n")
                          }
                          if ((problem_type == "regression") & (uniq_target_values == 2L)) {
                            warning("You set up a regression problem for only two classes.\n")
                          }
                        }
                        super$initialize(id = id)
                        private$ind_col = ind_col
                        private$target = target
                        private$verbose = TRUE
                        private$problem_type = problem_type
                        private$target_obj = Target$new(id = "target",
                                                        data_frame = target_df[ , c(ind_col, target)],
                                                        training = self)
                        private$status = FALSE
                      },
                      #' @description
                      #' Printer
                      #'
                      #' @param ... `any`
                      #'
                      print = function (...) {
                        nb_layers = length(private$hash_table) - 1L
                        if (!private$status) {
                          status = "Not trained"
                        } else {
                          status = "Trained"
                          if (private$use_var_sel) {
                            on_var_sel = "Yes"
                          } else {
                            on_var_sel = "No"
                          }
                        }
                        cat(sprintf("Training        : %s\n", private$id))
                        cat(sprintf("Problem type    : %s\n", private$problem_type))
                        cat(sprintf("Status          : %s\n", status))
                        cat(sprintf("Number of layers: %s\n", nb_layers))
                        cat(sprintf("Layers trained  : %s\n", private$nb_trained_layer))
                        if (private$status) {
                          cat(sprintf("Var. sel. used  : %s\n", on_var_sel))
                        }
                        # cat(sprintf("n               : %s\n", nrow(private$target_obj$getData())))
                        layers = self$getKeyClass()
                        if (private$status) {
                          layers = layers[layers$class %in% c("TrainLayer", "TrainMetaLayer"), ]
                        } else {
                          layers = layers[layers$class %in% "TrainLayer", ]
                        }
                        nb_layers = nrow(layers)
                        if (nb_layers) {
                          layer_ps = NULL
                          layer_ns = NULL
                          lner_nas = NULL
                          for (k in layers$key) {
                            layer = self$getFromHashTable(key = k)
                            train_data = layer$getTrainData()
                            layer_p = ncol(train_data$getData())
                            layer_n = nrow(train_data$getData())
                            lner_na = layer$getLrner()$getNaAction()
                            max_width <- max(nchar(layer_p), nchar(layer_n))
                            layer_ps = c(layer_ps, format(layer_p, width = max_width, justify = "left"))
                            layer_ns = c(layer_ns, format(layer_n, width = max_width, justify = "left"))
                            lner_nas = c(lner_nas, format(lner_na, width = max_width, justify = "left"))
                          }
                          layer_ps = paste0(layer_ps, collapse = " | ")
                          layer_ns = paste0(layer_ns, collapse = " | ")
                          lner_nas = paste0(lner_nas, collapse = " | ")
                          cat(sprintf("p               : %s\n", layer_ps))
                          cat(sprintf("n               : %s\n", layer_ns))
                          cat(sprintf("na.action       : %s\n", lner_nas))
                        }
                        invisible(self)
                      },
                      #' @description
                      #' Train each layer of the current Training.
                      #'
                      #' @param ind_subset `character`\cr
                      #' Subset of individuals IDs to be used for training.
                      #' @param use_var_sel `boolean` \cr
                      #' If TRUE, selected variables available at each layer are used.
                      #' @param verbose `boolean` \cr
                      #' Warning messages will be displayed if set to TRUE.
                      #' @return
                      #' Returns the object itself, with a model for each layer.
                      #' @export
                      #'
                      #'
                      trainLayer = function (ind_subset = NULL,
                                             use_var_sel = FALSE,
                                             verbose = TRUE) {
                        layers = self$getKeyClass()
                        nb_layers = nrow(layers[layers$class %in% "TrainLayer", ])
                        if (nb_layers) {
                          # This code accesses each layer (except MetaLayer) level and trains the corres-
                          # ponding learner.
                          layers = layers[layers$class %in% "TrainLayer", ]
                          for (k in layers$key) {
                            layer = self$getFromHashTable(key = k)
                            layer$train(ind_subset = ind_subset,
                                        use_var_sel = use_var_sel,
                                        verbose = verbose)
                          }
                        } else {
                          stop("No existing layer in the current training object.")
                        }
                        invisible(self)
                      },
                      #' @description
                      #' Predicts values given new data.
                      #'
                      #' @param testing `TestData` \cr
                      #' Object of class [TestData].
                      #' @param ind_subset  `vector` \cr
                      #' Subset of individuals IDs to be used for training.
                      #'
                      #' @return
                      #' A new [Training] with predicted values for each layer.
                      #' @export
                      #'
                      predictLayer = function (testing,
                                               ind_subset = NULL) {
                        # Initialize a Training to store predictions
                        predicting = Predicting$new(id = testing$getId(),
                                                    ind_col = testing$getIndCol())
                        layers = testing$getKeyClass()
                        # This code accesses each layer (except MetaLayer) level
                        # and make predictions for the new layer in input.
                        layers = layers[layers$class %in% c("TestLayer", "TrainLayer"), ]
                        for (k in layers$key) {
                          new_layer = testing$getFromHashTable(key = k)
                          new_layer_kc = new_layer$getKeyClass()
                          m_layer = self$getFromHashTable(key = k)
                          pred_layer = m_layer$predict(new_layer = new_layer,
                                                       ind_subset = ind_subset,
                                                       use_var_sel = self$getUseVarSel())
                          # Add a new predicted layer to the predicted object
                          pred_layer$setPredicting(predicting)
                        }
                        return(predicting)
                      },
                      #' @description
                      #' Creates a meta training dataset and assigns it to the meta layer.
                      #'
                      #'
                      #' @param resampling_method `function` \cr
                      #' Function for internal validation.
                      #' @param resampling_arg `list` \cr
                      #' List of arguments to be passed to the function.
                      #' @param use_var_sel `boolean` \cr
                      #' If TRUE, selected variables available at each layer are used.
                      #' @param impute `boolean` \cr
                      #' If TRUE, mode or median based imputation is performed on the modality-specific predictions.
                      #'
                      #' @return
                      #' The current object is returned, with a meta training dataset assigned to the meta layer.
                      #' @export
                      #'
                      createMetaTrainData = function (resampling_method,
                                                      resampling_arg,
                                                      use_var_sel,
                                                      impute = TRUE) {
                        verbose = self$getVerbose()
                        layers = self$getKeyClass()
                        if (!("TrainMetaLayer" %in% layers$class)) {
                          stop("No existing meta layer. I cannot create meta training data.")
                        }
                        resampling = do.call(eval(parse(text = resampling_method)),
                                             resampling_arg)
                        if (!is.list(resampling)) {
                          stop("The resampling method must return a list of folds, with each fold containing a vector of training IDs.\n See example for details.")
                        } else {
                          if (verbose) {
                            message("Creating fold predictions.\n")
                            pb <- txtProgressBar(min = 0, max = length(resampling), style = 3)
                          }
                          train_layer_res_list = lapply(X = 1:length(resampling),
                                                        function (fold) {
                                                          if (verbose) {
                                                            percent <- round((fold / length(resampling)) * 100L, 1L)
                                                            setTxtProgressBar(pb = pb, value = fold, label = percent)
                                                          }
                                                          test_index = resampling[[fold]]
                                                          train_index = setdiff(unlist(resampling), test_index)
                                                          train_ids = self$getTargetValues()[train_index, 1L]
                                                          self$trainLayer(ind_subset = train_ids,
                                                                          use_var_sel = use_var_sel,
                                                                          verbose = FALSE)
                                                          test_ids = self$getTargetValues()[test_index, 1L]
                                                          # Note: The current object is not a TestStudy, but a Training object.
                                                          predicting = self$predictLayer(testing = self,
                                                                                         ind_subset = test_ids)
                                                          predicting_kc = predicting$getKeyClass()
                                                          ## Assess each layer and extract model
                                                          current_pred = NULL
                                                          for (k in predicting_kc$key) {
                                                            pred_layer = predicting$getFromHashTable(key = k)
                                                            # pred_layer = layer$getFromHashTable(key = "PredictLayer")
                                                            pred_data = pred_layer$getPredictData()
                                                            pred_values = pred_data$getPredictData()
                                                            current_pred = rbind(current_pred, pred_values)
                                                          }
                                                          return(current_pred)
                                                        })
                          close(pb)
                          predicted_values = data.frame(do.call(what = "rbind",
                                                                args = train_layer_res_list))
                          # Will transform modality-specific prediction data.frame into wide format. If predictions are data.frame like
                          # ranger predictions than only the first column will be consider.
                          predicted_values_wide = reshape(predicted_values,
                                                          idvar = colnames(predicted_values)[2],
                                                          timevar = colnames(predicted_values)[1],
                                                          direction = "wide")
                          colname_vector = gsub(pattern = "Prediction.",
                                                replacement = "",
                                                x = names(predicted_values_wide))
                          names(predicted_values_wide) = colname_vector
                          # target_df = self$getTargetValues()
                          target_df = private$target_obj$getData()
                          predicted_values_wide = merge(x = target_df,
                                                        y = predicted_values_wide,
                                                        by = colnames(target_df)[1L],
                                                        all.y = TRUE)
                          # Add layer specific predictions to meta layer
                          layers = self$getKeyClass()
                          meta_layer_key = layers[layers$class == "TrainMetaLayer" , "key"]
                          meta_layer = self$getFromHashTable(key = meta_layer_key)
                          meta_layer$openAccess()
                          # predicted20242806 this word just serves as temporary key
                          meta_layer$setTrainData(id = "predicted20242806",
                                                  ind_col = names(predicted_values_wide)[1L],
                                                  data_frame = predicted_values_wide)
                          if (impute) {
                            private$impute = impute
                            meta_layer$impute()
                          }
                          # meta_layer$set2NotTrained()
                          meta_layer$closeAccess()
                          return(predicted_values_wide)
                        }
                      },
                      #' @description
                      #' Trains the current object. All leaners and the meta learner are trained.
                      #'
                      #' @param ind_subset `vector` \cr
                      #' ID subset to be used for training.
                      #' @param use_var_sel `boolean` \cr
                      #' If TRUE, variable selection is performed before training.
                      #' @param resampling_method `function` \cr
                      #' Function for internal validation. If not specify, the \code{resampling} function from the package \code{caret} is used for a 10-folds cross-validation.
                      #' @param resampling_arg `list` \cr
                      #' List of arguments to be passed to the function.
                      #' @param seed `integer` \cr
                      #' Random seed. Default is NULL, which generates the seed from \code{R}.
                      #'
                      #' @return
                      #' The current object is returned, with each learner trained on each layer.
                      #' @export
                      #'
                      train = function (ind_subset = NULL,
                                        use_var_sel = FALSE,
                                        resampling_method = NULL,
                                        resampling_arg = list(),
                                        seed = NULL) {
                        if (!is.logical(use_var_sel)) {
                          stop("'use_var_sel' must be boolean.")
                        } else {
                          private$use_var_sel = use_var_sel
                        }
                        # Test that the training object contains overlapping individuals
                        if (!self$testOverlap()) {
                          stop("This Training object does not contain overlapping individuals.") #nocov
                        }
                        # Check wether all individuals have layer-data
                        layer_ids = self$getIndIDs()[ , 1L]
                        target_df = self$getTargetObj()$getData()
                        ids_in_target = target_df[ , private$ind_col]
                        ids_not_in_target = setdiff(ids_in_target, layer_ids)
                        if (length(ids_not_in_target)) {
                          if (length(ids_not_in_target) == nrow(target_df)) {
                            stop("There is no target individual(s) with modality observations.\n")
                          } else {
                            warning(sprintf("%d individual(s) with target value(s), but no modality observations identified and excluded.\n", length(ids_not_in_target)))
                          }
                          # Remove those individuals in target without modality data.
                          clean_target_df = target_df[ids_in_target %in% layer_ids, ]
                          self$getTargetObj()$setData(clean_target_df)
                        }
                        # 1) Create meta training data
                        if (is.null(resampling_method)) {
                          resampling_method = "caret::createFolds"
                          resampling_arg = list(y = self$getTargetValues()[ , self$getTarget()],
                                                k = 10L)
                        }
                        if (!is.null(seed)) {
                          set.seed(seed = seed)
                        }
                        self$createMetaTrainData(resampling_method,
                                                 resampling_arg,
                                                 use_var_sel = use_var_sel,
                                                 impute = private$impute)
                        cat("\n")
                        # 2) Train each layer
                        self$trainLayer(ind_subset = ind_subset,
                                        use_var_sel = use_var_sel,
                                        verbose = self$getVerbose())
                        # 3) Train the meta layer
                        # Add layer specific predictions to meta training layer
                        layers = self$getKeyClass()
                        meta_layer_key = layers[layers$class == "TrainMetaLayer" , "key"]
                        meta_layer = self$getFromHashTable(key = meta_layer_key)
                        meta_layer$train(ind_subset = ind_subset)
                        return(self)
                      },
                      #' @description
                      #' Compute predictions for a testing object.
                      #'
                      #' @param testing `Testing` \cr
                      #' A new testing object to be predicted.
                      #' @param ind_subset `vector` \cr
                      #' Vector of IDs to be predicted.
                      #'
                      #' @return
                      #' The predicted object. All layers and the meta layer are predicted. This is the final predicted object.
                      #' @export
                      #'
                      # Our predictions based on cross-validation are different from that coming from the original learning method; e.g. that coming from ranger.
                      predict = function (testing,
                                          ind_subset = NULL) {
                        # 0) Check consistency between training and testing layers
                        layer_training = self$getKeyClass()
                        layer_training_key = layer_training[layer_training$class == "TrainLayer", "key"]
                        layer_testing = testing$getKeyClass()
                        layer_testing_key = layer_testing[layer_testing$class == "TestLayer", "key"]
                        name_inter = intersect(layer_training_key, layer_testing_key)
                        testing_not_in = any(!(layer_testing_key %in% layer_training_key))
                        if (length(name_inter) != length(layer_training_key) | testing_not_in) {
                          stop("Inconsistent names identified between training and testing.")
                        }
                        # 1) Layer predictions
                        predicting = self$predictLayer(testing = testing,
                                                       ind_subset = ind_subset)
                        # 2) Meta layer predicted new data; resume layer specific
                        #    predictions and create a new data.
                        meta_layer_id = self$getTrainMetaLayer()$getId()
                        testing_meta_data = predicting$createMetaTestData(
                          meta_layer_id = meta_layer_id)
                        if (private$impute) {
                          # TODO: Can be moved into testMetaLayer; similarly to trainMetaLayer.
                          testing_meta_data$impute(impute_fct = NULL,
                                                   impute_param = NULL,
                                                   target_name = self$getTargetObj()$getTargetName())
                        }
                        # 3) Predict new meta layer by the trained meta layer.
                        layers = self$getKeyClass()
                        meta_layer_key = layers[layers$class == "TrainMetaLayer", "key"]
                        meta_layer = self$getFromHashTable(key = meta_layer_key)
                        # TODO: getTestLayer maybe rename it getLayer?
                        predicted_layer = meta_layer$predict(new_layer = testing_meta_data$getTestLayer(),
                                                             ind_subset = ind_subset)
                        # Store final meta predicted values on meta layer.
                        predicting$removeFromHashTable(key = predicted_layer$getId())
                        predicting$add2HashTable(key = predicted_layer$getId(),
                                                 value = predicted_layer,
                                                 .class = "PredictData")
                        # Reshape format of meta-predictions as those of layer-spec predictions
                        predicted_meta_values = predicted_layer$getPredictData()$getDataFrame()
                        predicted_meta_values_wide = reshape(predicted_layer$getPredictData()$getDataFrame(),
                                                             idvar = colnames(predicted_meta_values)[2L],
                                                             timevar = colnames(predicted_meta_values)[1L],
                                                             direction = "wide")
                        colname_vector = gsub(pattern = "Prediction.",
                                              replacement = "",
                                              x = names(predicted_meta_values_wide))
                        names(predicted_meta_values_wide) = colname_vector
                        predicted_values_wide = merge(x = testing_meta_data$getDataFrame(),
                                                      y = predicted_meta_values_wide,
                                                      by = self$getIndCol(),
                                                      all.y = TRUE)
                        return(list(predicting = predicting,
                                    predicted_values = predicted_values_wide))
                      },
                      #' @description
                      #' Variable selection on the current training object.
                      #'
                      #' @param ind_subset `vector` \cr
                      #' ID subset of individuals to be used for variable selection.
                      #' @param verbose `boolean` \cr
                      #' Warning messages will be displayed if set to TRUE.
                      #'
                      #' @return
                      #' The current layer is returned with the resulting model.
                      #' @export
                      #'
                      varSelection = function (ind_subset = NULL,
                                               verbose = TRUE) {
                        # Check wether all individuals have layer-data
                        layer_ids = self$getIndIDs()[ , 1L]
                        target_df = self$getTargetObj()$getData()
                        ids_in_target = target_df[ , private$ind_col]
                        ids_not_in_target = setdiff(ids_in_target, layer_ids)
                        if (length(ids_not_in_target)) {
                          if (length(ids_not_in_target) == nrow(target_df)) {
                            stop("There is no target individual(s) with modality observations.\n")
                          } else {
                            warning(sprintf("%d individual(s) with target value(s), but no modality observations identified and excluded.\n", length(ids_not_in_target)))
                          }
                          # Remove those individuals in target without modality data.
                          clean_target_df = target_df[ids_in_target %in% layer_ids, ]
                          self$getTargetObj()$setData(clean_target_df)
                        }
                        layers = self$getKeyClass()
                        nb_layers = nrow(layers[layers$class %in% "TrainLayer", ])
                        if (nb_layers) {
                          # This code accesses each layer (except MetaLayer) level and
                          # perform variable selection.
                          layers = layers[layers$class %in% "TrainLayer", ]
                          selected = NULL
                          for (k in layers$key) {
                            layer = self$getFromHashTable(key = k)
                            layer_var_sel = layer$varSelection(ind_subset = ind_subset,
                                                               verbose = self$getVerbose())
                            if (length(layer_var_sel)) {
                              # At least one variable selected
                              selected = rbind(selected,
                                               data.frame(Layer = layer$getId(),
                                                          variable = layer_var_sel))
                            }
                          }
                          private$var_sel_done = TRUE
                        } else {
                          stop("No existing layer in the current training object.")
                        }
                        return(selected)
                      },
                      #' @description
                      #' Gather target values from all layer.
                      #'
                      #' @return
                      #' A \code{data.frame} containing individuals IDs and corresponding target values.
                      #' @export
                      #'
                      getTargetValues = function() {
                        layers = self$getKeyClass()
                        # This code accesses each layer (except TrainMetaLayer) level
                        # and get the target variable
                        layers = layers[layers$class %in% "TrainLayer", ]
                        target_data = NULL
                        train_data = NULL
                        for (k in layers$key) {
                          layer = self$getFromHashTable(key = k)
                          target_data = as.data.frame(rbind(target_data,
                                                            layer$getTargetValues()))
                          train_data = layer$getTrainData()
                        }
                        target_data = target_data[!duplicated(target_data[ , train_data$getIndCol()]), ]
                        return(target_data)
                      },
                      #' @description
                      #' Gather individual IDs from all layer.
                      #'
                      #' @return
                      #' A \code{data.frame} containing individuals IDs.
                      #'
                      getIndIDs = function() {
                        layers = self$getKeyClass()
                        # This code accesses each layer (except TrainMetaLayer) level
                        # and get the individual IDs.
                        layers = layers[layers$class %in% "TrainLayer", ]
                        ids_data = NULL
                        current_data = NULL
                        for (k in layers$key) {
                          layer = self$getFromHashTable(key = k)
                          ids_data = as.data.frame(rbind(ids_data,
                                                         layer$getIndIDs()))
                        }
                        ids_data = ids_data[!duplicated(ids_data[ , 1L]), ,
                                            drop = FALSE]
                        return(ids_data)
                      },
                      #' @description
                      #' Get a layer of a given ID.
                      #'
                      #' @param id `character` \cr
                      #' The ID of the layer to be returned.
                      #'
                      #' @return
                      #' The [TrainLayer] object is returned for the given ID.
                      #' @export
                      #'
                      getLayer = function(id) {
                        layer = self$getFromHashTable(key = id)
                        return(layer)
                      },
                      #' @description
                      #' Getter of the meta layer.
                      #'
                      #' @return
                      #' Object from class [TrainMetaLayer]
                      #' @export
                      #'
                      getTrainMetaLayer = function () {
                        layers = self$getKeyClass()
                        meta_layer_key = layers[layers$class == "TrainMetaLayer", "key"]
                        meta_layer = self$getFromHashTable(key = meta_layer_key)
                        return(meta_layer)
                      },
                      #' @description
                      #' Retrieve models from all layer.
                      #'
                      #' @return
                      #' A \code{list} containing all (base and meta) models.
                      #' @export
                      #'
                      getModel = function() {
                        layers = self$getKeyClass()
                        # This code accesses each layer (except TrainMetaLayer) level
                        # and get the individual IDs.
                        layers = layers[layers$class %in% c("TrainLayer", "TrainMetaLayer"), ]
                        models = list()
                        for (k in layers$key) {
                          layer = self$getFromHashTable(key = k)
                          models[[layer$getId()]] = layer$getModel()$getBaseModel()
                        }
                        return(models)
                      },
                      #' @description
                      #' Retrieve modality-specific predictions.
                      #'
                      #' @return
                      #' A \code{list} containing all (base and meta) models.
                      #' @export
                      #'
                      getData = function() {
                        layers = self$getKeyClass()
                        layers = layers[layers$class %in% c("TrainLayer", "TrainMetaLayer"), ]
                        all_data = list()
                        for (k in layers$key) {
                          layer = self$getFromHashTable(key = k)
                          all_data[[layer$getId()]] = layer$getTrainData()$getDataFrame()
                        }
                        return(all_data)
                      },
                      #' @description
                      #' Remove a layer of a given ID.
                      #'
                      #' @param id `character` \cr
                      #' The ID of the layer to be removed.
                      #'
                      #' @return
                      #' The [TrainLayer] object is returned for the given ID.
                      #'
                      removeLayer = function(id) {
                        if (private$nb_trained_layer > 0L) {
                          private$nb_trained_layer = private$nb_trained_layer - 1L
                          warning(sprintf("%s was already trained. Do not forget to train it again to update its meta layer.", private$id))
                        }
                        self$removeFromHashTable(key = id)
                        invisible(TRUE)
                      },
                      #' @description
                      #' Remove the meta layer from the current [Training] object.
                      #'
                      removeTrainMetaLayer = function() {
                        layers = self$getKeyClass()
                        meta_layer_key = layers[layers$class == "TrainMetaLayer", "key"]
                        self$removeFromHashTable(key = meta_layer_key)
                        invisible(TRUE)
                      },
                      #' @description
                      #' Getter of the individual column name.
                      getIndCol = function () {
                        return(private$ind_col)
                      },
                      #' @description
                      #' Getter of the target variable name.
                      getTarget = function () {
                        return(private$target)
                      },
                      #' @description
                      #' Getter of the verbose setting.
                      getVerbose = function () {
                        return(private$verbose)
                      },
                      #' @description
                      #' Getter of the use_var_sel field.
                      getUseVarSel = function () {
                        return(private$use_var_sel)
                      },
                      #' @description
                      #' Getter of the use_var_sel field.
                      getVarSelDone = function () {
                        return(private$var_sel_done)
                      },
                      #' @description
                      #' Increase the number of trained layer.
                      increaseNbTrainedLayer = function () {
                        private$nb_trained_layer = private$nb_trained_layer + 1L
                        if (private$nb_trained_layer == length(private$hash_table) - 1L) {
                          private$status = TRUE
                        }
                      },
                      #' @description
                      #' Check whether a target object has already been stored.
                      #'
                      #' @return
                      #' Boolean value
                      #'
                      checkTargetExist = function () {
                        return(super$checkClassExist(.class = "Target"))
                      },
                      #' @description
                      #' Getter of the target object.
                      # TODO: Maybe rename this function getTarget and find another appropriate name for the current getTarget function.
                      getTargetObj = function () {
                        return(private$target_obj)
                      },
                      #' @description
                      #' Getter of the problem type.
                      getProblemTyp = function () {
                        return(private$problem_type)
                      },
                      #' @description
                      #' Set imputation action na.action.
                      #'
                      #' @param impute `character` \cr
                      #' How to handle missing values.
                      #'
                      setImpute = function(impute) {
                        private$impute = impute
                        invisible(self)
                      },
                      #' @description
                      #' Test that individuals overlap over layers.
                      #' At least five individuals must overlapped.
                      #'
                      testOverlap = function () {
                        layers = self$getKeyClass()
                        if (nrow(layers) == 1L) {
                          stop ("No layer found in this training object.")
                        }
                        # This code accesses each layer (except TrainMetaLayer) level
                        # and get the individual IDs.
                        layers = layers[layers$class %in% "TrainLayer", ]
                        ids_data = NULL
                        current_data = NULL
                        # TRUE if only one training layer
                        if (length(layers$key) == 1) {
                          return(TRUE)
                        }
                        for (k in layers$key) {
                          layer = self$getFromHashTable(key = k)
                          ids_data = as.data.frame(rbind(ids_data,
                                                         layer$getIndIDs()))
                        }
                        if (sum(duplicated(ids_data[ , 1L])) > 5L) {
                          return(TRUE)
                        } else {
                          return(FALSE) # nocov
                        }
                      },
                      #' @description
                      #' UpSet plot to show an overview of the overlap of individuals across various layers.
                      #'
                      #' @param ... `any` \cr
                      #' Further parameters to be passed to the \code{upset} function from package \code{UpSetR}.
                      #'
                      #' @export
                      #'
                      upset = function (...) {
                        layers = self$getKeyClass()
                        # This code accesses each layer (except TrainMetaLayer) level
                        # and get the individual IDs.
                        layers = layers[layers$class %in% "TrainLayer", ]
                        if (!nrow(layers) | (nrow(layers) == 1L)) {
                          stop("No or only one available layer in this training object.")
                        }
                        ids_list = lapply(layers$key, function (k) {
                          layer = self$getFromHashTable(key = k)
                          return(layer$getIndIDs()[ , 1L])
                        })
                        param_upset = list(...)
                        from_list_ids = do.call(eval(parse(text = "UpSetR::fromList")),
                                                list(input = ids_list))
                        names(from_list_ids) = layers$key
                        param_upset$data = from_list_ids
                        #TODO: Fix me once upset is up to-date with the latest version of ggplot2. For now, I use suppressWarnings to avoid the following warning: "Warning: Ignoring unknown aesthetics: x, y"
                        suppressWarnings({
                          print(do.call(eval(parse(text = "UpSetR::upset")),
                                        param_upset))
                        })
                        invisible(TRUE)
                      },
                      #' @description
                      #' Generate training summary
                      #'
                      #' @export
                      #'
                      summary = function () {
                        cat(sprintf("Training %s\n", self$getId()))
                        cat("----------------\n")
                        self$print()
                        cat("----------------\n")
                        cat("\n")
                        layers = self$getKeyClass()
                        layers = layers[layers$class %in% c("TrainLayer", "TrainMetaLayer"), ]
                        for (k in layers$key) {
                          layer = self$getFromHashTable(key = k)
                          layer$summary()
                          cat("\n")
                        }
                        invisible(self)
                      }
                    ),
                    private = list(
                      ind_col = character(0L),
                      target = character(0L),
                      target_obj = NULL,
                      nb_trained_layer = 0L,
                      impute = FALSE,
                      verbose = TRUE,
                      problem_type = "classification",
                      status = FALSE,
                      use_var_sel = FALSE,
                      var_sel_done = FALSE
                    ),
                    cloneable = FALSE
)

test_that("cobra works", {

  # Test 1: Basic functionality with valid inputs
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
    y = sample(x = 0L:1L, size = 50L, replace = TRUE)
    my_model = cobra(x = x, y = y, tune = "epsilon", k_folds = 5)
    x_new <- data.frame(x1 = runif(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })

  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
    y = sample(x = 0L:1L, size = 50L, replace = TRUE)
    my_model = cobra(x = x, y = y, tune = "alpha_epsilon", k_folds = 5)
    x_new <- data.frame(x1 = runif(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })

  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
    y = sample(x = 0L:1L, size = 50L, replace = TRUE)
    my_model = cobra(x = x, y = y, tune = "user", eps = 0.1)
    x_new <- data.frame(x1 = runif(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })

  # Test 2: Error if variable names in new data don't match model
  expect_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = sample(x = 0L:1L, size = 50L, replace = TRUE)
    my_model = cobra(x = x, y = y)
    x_new <- data.frame(x2 = rnorm(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })

  # Test 3: Error if target variable y is not binary
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = sample(x = 0L:2L, size = 50L, replace = TRUE)
    my_model = cobra(x = x, y = y, tune = "epsilon")
  })

  # Test 4: No error with factor binary target (e.g., "control" and "case")
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = factor(sample(x = c("control", "case"), size = 50L, replace = TRUE))
    my_model = cobra(x = x, y = y)
  })

  # Test 5: No error with string binary target (e.g., "0" and "1") converted to factor
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = rnorm(50L))
    y = sample(x = c("0", "1"), size = 50L, replace = TRUE)
    my_model = cobra(x = x, y = factor(y))
  })

  # Test 6: Error if test data is empty
  expect_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(50))
    y = sample(0:1, 50, replace = TRUE)
    my_model = cobra(x = x, y = y)
    x_new <- data.frame(x1 = numeric(0)) # Empty test data
    my_predictions <- predict(object = my_model, data = x_new)
  })

  # Test 7: Error if training data has no learners
  expect_error({
    set.seed(20240624L)
    x <- data.frame()  # No columns in training data
    y <- sample(0:1, 50, replace = TRUE)
    my_model <- cobra(x = x, y = y)
  })

  # Test 8: Error if `eps` parameter is invalid
  expect_error({
    set.seed(20240624L)
    x <- data.frame(x1 = runif(50))
    y <- sample(0:1, 50, replace = TRUE)
    my_model <- cobra(x = x, y = y, tune = "user", eps = -1)  # Invalid epsilon
    x_new <- data.frame(x1 = runif(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })

  # Test 9: Error if `eps` is not NULL, but user not equal to 'user'
  expect_error({
    set.seed(20240624L)
    x <- data.frame(x1 = runif(50))
    y <- sample(0:1, 50, replace = TRUE)
    my_model <- cobra(x = x, y = y, eps = 0.1)
    x_new <- data.frame(x1 = runif(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })

  # Test 10: Error if `k_folds` is not NULL, but user not equal to 'epsilon' or 'alpha_epsilon'
  expect_error({
    set.seed(20240624L)
    x <- data.frame(x1 = runif(50))
    y <- sample(0:1, 50, replace = TRUE)
    my_model <- cobra(x = x, y = y, tune = "user", k_folds = 2)
    x_new <- data.frame(x1 = runif(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })


  # Test 11: Error if invalid k_folds parameter
  expect_error({
    set.seed(20240624L)
    x <- data.frame(x1 = runif(50))
    y <- sample(0:1, 50, replace = TRUE)
    my_model <- cobra(x = x, y = y, tune = "epsilon", k_folds = 1)  # Invalid k_folds
    x_new <- data.frame(x1 = runif(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })

  # Test 12: Valid case with small number of observation
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 10L, min = 0, max = 1))
    y = sample(x = 0L:1L, size = 10L, replace = TRUE)
    my_model = cobra(x = x, y = y, tune = "epsilon", k_folds = 10)
    x_new <- data.frame(x1 = runif(5))
    my_predictions <- predict(object = my_model, data = x_new)
  })

  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 10L, min = 0, max = 1))
    y = sample(x = 0L:1L, size = 10L, replace = TRUE)
    my_model = cobra(x = x, y = y, tune = "alpha_epsilon", k_folds = 10)
    x_new <- data.frame(x1 = runif(5))
    my_predictions <- predict(object = my_model, data = x_new)
  })

  # Test 13: if k_folds is numeric and greater than 1
  expect_error(cobra(x = data.frame(x1 = 1:3), y = c(0, 1, 0),
                     tune = "alpha_epsilon", k_folds = 1))
  expect_error(cobra(x = data.frame(x1 = 1:3), y = c(0, 1, 0),
                     tune = "alpha_epsilon", k_folds = "invalid"))

  # Test 14: Expect message
  expect_message(cobra(x = data.frame(x1 = 1:3), y = c(0, 1, 0),
                       tune = "alpha_epsilon"))
  expect_message(cobra(x = data.frame(x1 = 1:3), y = c(0, 1, 0),
                       tune = "epsilon"))
  expect_message(cobra(x = data.frame(x1 = 1:3), y = c(0, 1, 0),
                       tune = "user"))

  # Test 15: Expect error
  expect_error(cobra(x = data.frame(x1 = 1:3), y = c(0, 1, 0),
                       tune = "alpha"))

  # Test 16: Ensure the test data contains at least one row
  expect_error(predict.cobra(object = cobra_model, data = data.frame()))

  # Test 15: Expect error: y should be only 0 or 1
  expect_error(cobra(x = data.frame(x1 = 1:3), y = c(1, 2, 1),
                     tune = "epsilon"))
  expect_error(cobra(x = data.frame(x1 = 1:3), y = as.character(c(1, 2, 1)),
                     tune = "epsilon"))
  expect_error(cobra(x = data.frame(x1 = 1:3), y = as.factor(c(1, 2, 1,3,3)),
                     tune = "epsilon"))

  # Test 16: Expect error: x should be data.frame
  expect_error(cobra(x = matrix(1:9, nrow= 3, ncol= 3),
                     y = sample(x = 0L:1L, size = 9L, replace = TRUE),
                     tune = "epsilon"))

  expect_error(cobra(x = list(x1 = 1:3), y = c(0, 1, 1), tune = "epsilon"))


})

# Training
training <- Training$new(id = "training",
                              ind_col = "IDS",
                              target = "disease",
                              target_df = multi_omics$training$target)
# Training layers
tl_geneexpr <- TrainLayer$new(id = "geneexpr",
                              training = training)

# Learner

expect_error({
  lrner_geneexpr <- Lrner$new(id = "ranger",
                              package = "ranger",
                              lrn_fct = "ranger",
                              param_train_list = list(probability = TRUE,
                                                      mtry = 1L,
                                                      num.trees = 1000L),
                              train_layer = "not_a_train_layer")
})

expect_error({
  lrner_geneexpr <- Lrner$new(id = "ranger",
                              package = "ranger",
                              lrn_fct = "ranger",
                              param_train_list = list(probability = TRUE,
                                                      mtry = 1L,
                                                      num.trees = 1000L),
                              na_rm = "not_logical",
                              train_layer = tl_geneexpr)
})

lrner_geneexpr <- Lrner$new(id = "ranger",
                            package = "ranger",
                            lrn_fct = "ranger",
                            param_train_list = list(probability = TRUE,
                                                    mtry = 1L,
                                                    num.trees = 1000L),
                            train_layer = tl_geneexpr)

expect_no_error({
  print(lrner_geneexpr)
  # Update
  lrner_geneexpr <- Lrner$new(id = "ranger",
                              package = "ranger",
                              lrn_fct = "ranger",
                              param_train_list = list(probability = TRUE,
                                                      mtry = 1L,
                                                      num.trees = 1000L),
                              train_layer = tl_geneexpr)
})

# Instantiate
test_that("Lrner instantiates correctly", {
  expect_true(R6::is.R6(lrner_geneexpr))
  expect_equal(class(lrner_geneexpr)[1], "Lrner")
})

# Training without training data does not work.
test_that("Lrner cannot be trained without a training data", {
  expect_error(lrner_geneexpr$train())
})

test_that("bestLayerLearner works", {
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
    y = sample(x = 0L:1L, size = 50L, replace = TRUE)
    my_model = bestLayerLearner(x = x, y = y)
    x_new <- data.frame(x1 = rnorm(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })
  expect_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
    y = sample(x = 0L:1L, size = 50L, replace = TRUE)
    my_model = bestLayerLearner(x = x, y = y)
    x_new <- data.frame(x2 = rnorm(10))
    my_predictions <- predict(object = my_model, data = x_new)
  })
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
    y = sample(x = 0L:2L, size = 50L, replace = TRUE)
    my_model = bestLayerLearner(x = x, y = y)
  })
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
    y = factor(sample(x = c("control", "case"), size = 50L, replace = TRUE))
    my_model = bestLayerLearner(x = x, y = y)
  })
  expect_no_error({
    set.seed(20240624L)
    x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
    y = sample(x = c("0", "1"), size = 50L, replace = TRUE)
    my_model = bestLayerLearner(x = x, y = factor(y))
  })
  expect_error(bestLayerLearner(x = data.frame(x1 = 1:3),
                                y = c(1, 2, 1)))
  expect_error(bestLayerLearner(x = data.frame(x1 = 1:3),
                                y = as.character(c(1, 2, 1))))
  expect_error(bestLayerLearner(x = data.frame(x1 = 1:3),
                                y = as.factor(c(1, 2, 1,3,3))))

  # Test 16: Expect error: x should be data.frame
  expect_error(bestLayerLearner(x = matrix(1:9, nrow= 3, ncol= 3),
                                y = sample(x = 0L:1L, size = 9L, replace = TRUE)))

  expect_error(bestLayerLearner(x = list(x1 = 1:3), y = c(0, 1, 1)))
})

data("multi_omics")
set.seed(5214)
test_that("Training: all tests", {
  # Create training
  expect_no_error({
    training <- Training$new(id = "training",
                                  ind_col = "IDS",
                                  target = "disease",
                                  target_df = multi_omics$training$target)
    print(training)
  })
  expect_error({
    Training$new(id = 1,
                   ind_col = "IDS",
                   target = "disease",
                   target_df = multi_omics$training$target)
  })
  expect_error({
    Training$new(id = "training",
                   ind_col = 1,
                   target = "disease",
                   target_df = multi_omics$training$target)
  })
  expect_error({
    Training$new(id = "training",
                   ind_col = "IDS",
                   target = 1,
                   target_df = multi_omics$training$target)
  })
  expect_error({
    Training$new(id = "training",
                   ind_col = "IDS",
                   target = "disease",
                   target_df = 12)
  })
  expect_error({
    Training$new(id = "training",
                   ind_col = "IDS",
                   target = "disease",
                   target_df = data.frame(x = 1:3, y = 1:3, z = 1:3))
  })
  expect_error({
    training$testOverlap()
  })
  expect_error({
    training <- Training$new(id = "training",
                                  ind_col = "IDS",
                                  target = "disease",
                                  target_df = multi_omics$training$target,
                                  problem_type = "test")
  })
  expect_error({
    tmp <- multi_omics$training$target
    tmp$disease <- sample(x = 1:3,
                          size = nrow(tmp),
                          replace = TRUE)
    training <- Training$new(id = "training",
                                  ind_col = "IDS",
                                  target = "disease",
                                  target_df = tmp,
                                  problem_type = "classification")
  })
  expect_warning({
    training <- Training$new(id = "training",
                                  ind_col = "IDS",
                                  target = "disease",
                                  target_df = multi_omics$training$target,
                                  problem_type = "regression")
  })
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for empty (no layer) training                      +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  expect_error({
    training$train()
  })
  expect_error({
    training$trainLayer()
  })
  expect_error({
    training$createMetaTrainData()
  })
  expect_error({
    training$varSelection()
  })
  expect_error({
    training$test_overlap()
  })
  expect_error({
    training$train(ind_subset = NULL,
                      use_var_sel = TRUE,
                      resampling_method = character(0L),
                      resampling_arg = list())
    training$train(ind_subset = NULL,
                      use_var_sel = TRUE,
                      resampling_method = character(0L),
                      resampling_arg = list())
  })

  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for training with empty layers.                    +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  expect_no_error({
    tl_ge <- TrainLayer$new(id = "geneexpr", training = training)
    tl_ge <- training$getLayer(id = "geneexpr")
    tl_pr <- TrainLayer$new(id = "proteinexpr", training = training)
    tl_methy <- TrainLayer$new(id = "methylation", training = training)
    training$removeLayer(id = "methylation")
    # We also prepare the meta layer for the meta analysis.
    tl_meta <- TrainMetaLayer$new(id = "meta_layer", training = training)
    # Test to remove and re-add
    training$removeTrainMetaLayer()
    tl_meta <- TrainMetaLayer$new(id = "meta_layer", training = training)
  })
  # Expected errors on training empty layers
  expect_error({
    training$train()
  })
  expect_error({
    training$testOverlap()
  })
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for training with loaded data layers               +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # TrainData can be added successfully
  expect_no_error({
    train_data_ge <- TrainData$new(id = "geneexpr",
                                   train_layer = tl_ge,
                                   data_frame = multi_omics$training$geneexpr)
    train_data_pr <- TrainData$new(id = "proteinexpr",
                                   train_layer = tl_pr,
                                   data_frame = multi_omics$training$proteinexpr)
  })
  # Upset plot works
  expect_no_error({
    training$upset(order.by = "freq")
  })
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for training with loaded variable selection                +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Variable selection works
  expect_warning({
    varsel_ge <- VarSel$new(id = "varsel_geneexpr",
                            package = "Boruta",
                            varsel_fct = "Boruta",
                            varsel_param = list(num.trees = 50L,
                                                mtry = 3L,
                                                getImp = Boruta::getImpRfZ),
                            train_layer = tl_ge)
    var_sel_res <- training$varSelection()
    print(var_sel_res)
  })
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for training with loaded learners.                 +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Lrner parameters
  # Lrner
  expect_no_error({
    lrner_ge <- Lrner$new(id = "ranger",
                          package = "ranger",
                          lrn_fct = "ranger",
                          param_train_list = list(probability = TRUE,
                                       mtry = 2L, num.trees = 25L),
                          train_layer = tl_ge)
    lrner_pr <- Lrner$new(id = "ranger",
                          package = "ranger",
                          lrn_fct = "ranger",
                          param_train_list = list(probability = TRUE,
                                                  mtry = 2L, num.trees = 25L),
                          train_layer = tl_pr)
    lrner_meta <- Lrner$new(id = "weighted",
                            lrn_fct = "weightedMeanLearner",
                            param_train_list = list(),
                            na_action = "na.keep",
                            train_layer = tl_meta)
  })

  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for training with for training.                    +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  expect_no_error({
    disease <- training$getTargetValues()$disease
  })
    no_resampling <- capture_warning(
      training$train(use_var_sel = TRUE)
    )
    three_warning <- capture_warnings(
      trained <- training$train(resampling_method = "caret::createFolds",
                                         resampling_arg = list(y = disease,
                                                               k = 2L),
                                         use_var_sel = TRUE)
    )
    print(trained)
    print(tl_ge)
    expect_equal(length(three_warning), 1L)

  expect_error({
    disease <- training$getTargetValues()$disease
    trained <- training$train(resampling_method = "stats::rnorm",
                                       resampling_arg = list(n = 10L),
                                       use_var_sel = TRUE)
  })
  expect_no_error({
    trained$getId()
    trained$getIndCol()
    trained$getTarget()
    trained$getTrainMetaLayer()
    trained$getIndIDs()
    trained$getTargetValues()
    print(tl_meta$getTrainData())
    tmp_lrner <- tl_ge$getLrner()
    tmp_lrner$getIndSubset()
    tmp_lrner$getVarSubset()
    tmp_model <- tl_ge$getModel()
    tmp_model$summary()
  })
  expect_no_error({
    training$summary()
  })
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for training with for predicting                   +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Prediction
  expect_no_error({
    testing <- Testing$new(id = "testing", ind_col = "IDS")
    print(testing)
  })

  expect_no_error({
    nl_ge <- TestLayer$new(id = "geneexpr", testing = testing)
    print(nl_ge)
    nl_pr <- TestLayer$new(id = "proteinexpr", testing = testing)
  })

  expect_error({
    TestLayer$new(id = "geneexpr", testing = "not_a_testing")
  })

  expect_error({
    nl_ge$getIndIDs()
  })

  expect_no_error({
    testing_data_ge <- TestData$new(id = "geneexpr",
                               new_layer = nl_ge,
                               data_frame = multi_omics$testing$geneexpr)
    testing_data_pr <- TestData$new(id = "proteinexpr",
                               new_layer = nl_pr,
                               data_frame = multi_omics$testing$proteinexpr)
    testing$summary()
    testing$upset()
  })

  expect_no_error({
    testing$getIndIDs()
  })

  expect_no_error({
    all_models <- training$getModel()
    all_data <- training$getData()
    new_predictions <- training$predict(testing = testing)
    print(new_predictions)
  })

  expect_no_error({
    testing$getTestMetaLayer()
  })
})

#' @title TestMetaLayer Class
#'
#' @description
#' This class implement a predicted meta layer. A [TestMetaLayer] can only exist as unique element of a [Training] object.
#'
#' A predicted meta layer can only contain a [TestData] object.
#'
#' @export
#' @importFrom R6 R6Class
TestMetaLayer <- R6Class("TestMetaLayer",
                        inherit = HashTable,
                        public = list(
                          #' @description
                          #' constructor
                          #'
                          #' @param id `character`\cr
                          #' Testing meta-layer id.
                          #' @param testing `Testing`\cr
                          #'
                          initialize = function (id, testing) {
                            super$initialize(id = id)
                            private$testing = testing
                            testing$add2HashTable(key = id,
                                                    value = self,
                                                    .class = "TestMetaLayer")
                          },
                          #' @description
                          #' Printer
                          #' @param ... `any`
                          #'
                          print = function(...) {
                            cat(sprintf("TestMetaLayer: %s\n", private$id))
                            cat(sprintf("Contains %s object\n", length(private$hash_table)))
                          },
                          #' @description
                          #' Getter of the current testing object.
                          #'
                          #' @return
                          #' The current testing object is returned.
                          #'
                          getTesting = function () {
                            return(private$testing)
                          },
                          #' @description
                          #' Getter of the training dataset stored on the current layer.
                          #'
                          #' @return
                          #' The stored [TestData] object is returned.
                          #' @export
                          #'
                          getTestData = function () {
                            layer_kc = self$getKeyClass()
                            if ("TestData" %in% layer_kc[ , "class"]) {
                              new_data_key = layer_kc[layer_kc$class == "TestData" ,
                                                      "key"]
                              new_data = self$getFromHashTable(key = new_data_key[1L])
                            } else {
                              stop(sprintf("No new data on layer %s.", self$getId()))
                            }
                            return(new_data)
                          },
                          #' @description
                          #' Open access to the meta layer. A meta learner is only
                          #' modifiable if the access is opened.
                          #'
                          #'
                          openAccess = function () {
                            private$access = TRUE
                            invisible(self)
                          },
                          #' @description
                          #' Close access to the meta layer to avoid accidental
                          #' modification.
                          #'
                          #'
                          closeAccess = function () {
                            private$access = FALSE
                            invisible(self)
                          },
                          #' @description
                          #' Getter of the current access to the meta layer.
                          #'
                          #' @export
                          getAccess = function () {
                            return(private$access)
                          },
                          #' @description
                          #' Create and set an [TestData] object to the current
                          #' new meta learner.
                          #'
                          #' @param id `character(1)` \cr
                          #' ID of the [TestData] object to be instanciated.
                          #' @param ind_col `character(1)` \cr
                          #' Name of individual column IDs.
                          #' @param data_frame  `data.frame(1)` \cr
                          #' \code{data.frame} of layer specific predictions.
                          #'
                          #' @export
                          # TODO: Please do not export me.
                          setTestData = function (id,
                                                 ind_col,
                                                 data_frame) {
                            TestData$new(id = id,
                                        ind_col = ind_col,
                                        data_frame = data_frame,
                                        meta_layer = self)
                            return(self)
                          },
                          #' @description
                          #' Check whether a new data has been already stored.
                          #'
                          #' @return
                          #' Boolean value
                          #'
                          checkTestDataExist = function () {
                            return(super$checkClassExist(.class = "TestData"))
                          }
                        ),
                        private = list(
                          # The current testing
                          testing = NULL,
                          # Access to the meta layer.
                          access = FALSE
                        ),
                        cloneable = FALSE
)

#' @title TestData Class
#'
#' @description
#' This class implements [TestData] object to be predicted.
#' A [TestData] object can only exist as a component of a [TestLayer] or a [TestMetaLayer] object.
#' @export
#' @importFrom R6 R6Class
#' @seealso [TrainData]
TestData <- R6Class("TestData",
                   inherit = Data,
                   public = list(
                     #' @description
                     #' Initialize a new object from the current class.
                     #'
                     #' @param id `character` \cr
                     #' Object ID.
                     #' @param ind_col `character`\cr
                     #' Column name containing individual IDs.
                     #' @param data_frame `data.frame`\cr
                     #' \code{data.frame} containing data.
                     #' @param new_layer `TestLayer` \cr
                     #' Layer where to store the current object.
                     # TODO: rename new_layer by test_layer
                     initialize = function (id,
                                           data_frame,
                                           new_layer) {
                       if (!any(c("TestLayer", "TestMetaLayer") %in% class(new_layer))) {
                         stop("A Testdata can be stored only on a TestLayer or a TestMetaLayer object.")
                       }
                       ind_col = new_layer$getTesting()$getIndCol()
                       if (!(ind_col %in% colnames(data_frame))) {
                         stop("Individual column IDS not found in the provided data.frame.")
                       }
                       super$initialize(id = id,
                                        ind_col = ind_col,
                                        data_frame = data_frame)
                       if (new_layer$checkTestDataExist()) {
                           key_class = new_layer$getKeyClass()
                           key = key_class[key_class$class == "TestData", "key"]
                           new_layer$removeFromHashTable(key = key)
                       }
                       private$new_layer = new_layer
                       # Add to object to ht
                       if ("TestMetaLayer" %in% class(new_layer)) {
                         if (new_layer$getAccess()) {
                           new_layer$add2HashTable(key = private$id,
                                               value = self,
                                               .class = "TestData")
                         } else {
                           stop("Testing data cannot not be added manually to a meta layer.") #nocov
                         }
                       } else {
                         new_layer$add2HashTable(key = private$id,
                                             value = self,
                                             .class = "TestData")
                       }
                     },
                     #' @description
                     #' Printer
                     #' @param ... `any`
                     #'
                     print = function (...) {
                       cat("Class     : TestData\n")
                       cat(sprintf("Layer     : %s\n", private$new_layer$id))
                       cat(sprintf("name      : %s\n", private$id))
                       cat(sprintf("ind. id.  : %s\n", private$ind_col))
                       cat(sprintf("n         : %s\n", nrow(private$data_frame)))
                       cat(sprintf("p         : %s\n", ncol(private$data_frame)))
                     },
                     #' @description
                     #' Getter of the current \code{data.frame} wihtout individual
                     #' ID variable.
                     #'
                     #' @return
                     #' The \code{data.frame} without individual ID nor target variables is returned.
                     #' @export
                     #'
                     getData = function () {
                       tmp_data <- private$data_frame
                       tmp_data[[private$ind_col]] <- NULL
                       return(tmp_data)
                     },
                     #' @description
                     #' Getter of the current layer.
                     #'
                     #' @return
                     #' The layer (from class [TestLayer]) on which the current train data are stored
                     #' is returned.
                     #' @export
                     #'
                     getTestLayer = function () {
                       return(private$new_layer)
                     }
                   ),
                   private = list(
                     # Current layer.
                     new_layer = NULL
                   ),
                   cloneable = TRUE
)

#' Simulated multiomics data for 70 training participants and 23 testing participants,
#' each with an effect size of 20 on each layer. Each layer includes 50 participants for
#' training and 20 for testing. Participants do not perfectly overlap across layers.
#' The simulation is based on the R package \code{interSIM}.

#'
#' The dataset is a list containing training and testing data,
#' called \code{training} and \code{testing} respectively. Each data is a list
#' containing the following multi_omics at each layer.
#'
#' \itemize{
#'   \item \code{methylation}: A \code{data.frame} containing the simulated methylation dataset.
#'   \item \code{genexpr}    : A \code{data.frame} containing the gene expression dataset.
#'   \item \code{proteinexpr}: A \code{data.frame} containing the protein expression dataset.
#'   \item \code{target}: A \code{data.frame} with two columns, containing patient IDs and values of target variable.
#' }
#'
#' @docType data
#' @keywords datasets
#' @name multi_omics
#' @usage data(multi_omics)
#' @format A list with training and testing data contaning methylation, gene expressions and protein expressions data.
"multi_omics"

#' @title TestLayer Class
#'
#' @description
#' This class implements a layer. A [TestLayer] object can only exist as a component of a [Predicting] object.
#'
#' A predicted layer can only contain [TestData].
#'
#'
#' @export
#' @importFrom R6 R6Class
#' @seealso [Training], [Lrner], [TrainData], [TestData] and [Model]
TestLayer <- R6Class("TestLayer",
                        inherit = HashTable,
                        public = list(
                          #' @description
                          #' constructor
                          #'
                          #' @param id `character`\cr
                          #' Testing layer id.
                          #' @param testing `Testing`\cr
                          #'
                          initialize = function (id, testing) {
                            super$initialize(id = id)
                            private$testing = testing
                            if ("Testing" %in% class(testing)) {
                              testing$add2HashTable(key = id,
                                                      value = self,
                                                      .class = "TestLayer")
                            } else {
                              stop("A TestLayer can only belong to a TestStudy.")
                            }
                          },
                          #' @description
                          #' Printer
                          #' @param ... `any`
                          #'
                          print = function (...){
                            cat(sprintf("TestLayer    : %s\n", private$id))
                            cat(sprintf("Contains %s object.\n", length(private$hash_table)))
                          },
                          #' @description
                          #' Getter of the current Testing object.
                          #'
                          #' @return
                          #' The current Testing object is returned.
                          #'
                          getTesting = function () {
                            return(private$testing)
                          },
                          #' @description
                          #' Getter of IDS from the current layer.
                          #'
                          #' @return
                          #' A \code{data.frame} containing individuals IDs values.
                          #' @export
                          #'
                          getIndIDs = function () {
                            layer_kc = self$getKeyClass()
                            # Stop if training data is missing on this layer.
                            if (("TestData" %in% layer_kc[ , "class"])) {
                              # Searching for layer specific new dataset
                              data_key = layer_kc[layer_kc$class == "TestData" ,
                                                  "key"]
                              current_data = self$getTestData()
                            } else {
                              stop(sprintf("No data on layer %s.", self$getId()))
                            }
                            current_data_frame = current_data$getDataFrame()
                            ids_data = current_data_frame[ , current_data$getIndCol(), drop = FALSE]
                            return(ids_data)
                          },
                          #' @description
                          #' Getter of the predicted data stored on the current layer.
                          #'
                          #' @return
                          #' The stored [TestData] object is returned.
                          #' @export
                          #'
                          getTestData = function () {
                            layer_kc = self$getKeyClass()
                            if ("TestData" %in% layer_kc[ , "class"]) {
                              predict_data_key = layer_kc[layer_kc$class == "TestData" ,
                                                          "key"]
                              predict_data = self$getFromHashTable(key = predict_data_key[1L])
                            } else {
                              stop(sprintf("No predicted data on layer %s.", self$getId()))
                            }
                            return(predict_data)
                          },
                          #' @description
                          #' Check whether a new data has been already stored.
                          #'
                          #' @return
                          #' Boolean value
                          #'
                          checkTestDataExist = function () {
                            return(super$checkClassExist(.class = "TestData"))
                          },
                          #' @description
                          #' Generate summary.
                          #'
                          #' @export
                          #'
                          summary = function () {
                            layer_kc = self$getKeyClass()
                            for (k in layer_kc[ , "key"]) {
                              current_obj = self$getFromHashTable(key = k)
                              print(current_obj)
                              cat("\n")
                            }
                          }
                        ),
                        private = list(
                          testing = NULL
                        ),
                        cloneable = FALSE
)

#' @title Lrner Class
#'
#' @description
#' This class implements a learner. A [Lrner] object can only exist as a component of a
#' [TrainLayer] or a [TrainMetaLayer] object.
#'
#' @export
#'
#' @importFrom R6 R6Class
Lrner <- R6Class("Lrner",
                 public = list(
                   #' @description
                   #' Initialize a default parameters list.
                   #'
                   #'
                   #'
                   #' @param id `character` \cr
                   #' Learner ID.

                   #' @param package `character` \cr
                   #' Package that implements the learn function. If NULL, the
                   #' @param lrn_fct `character` \cr
                   #' learn function is called from the current environment.
                   #' @param param_train_list `list` \cr
                   #' List of parameter for training.
                   #' @param param_pred_list `list` \cr
                   #' List of parameter for testing.
                   #' Learn parameters.
                   #' @param train_layer `TrainLayer` \cr
                   #' Layer on which the learner is stored.
                   #' @param na_action `character` \cr
                   #' Handling of missing values. Set to "na.keep" to keep missing values, "na.rm" to remove individuals with missing values or "na.impute" (only applicable on meta-data) to impute missing values in meta-data. Only median and mode based imputations are actually handled. With the "na.keep" option, ensure that the provided learner can handle missing values.
                   initialize = function (id,
                                          package = NULL,
                                          lrn_fct,
                                          param_train_list,
                                          param_pred_list = list(),
                                          train_layer,
                                          na_action = "na.rm") {
                     private$id = id
                     private$package = package
                     private$lrn_fct = lrn_fct
                     private$param_train = param_train_list
                     private$param_pred = param_pred_list
                     if (is.null(package)) {
                       if (!(exists(lrn_fct, envir = .GlobalEnv, inherits = TRUE) | is.function(get(lrn_fct, envir = .GlobalEnv)))) {
                         stop(sprintf("Function %s does not exists.\n Maybe you forget to specify its package?", lrn_fct))
                       }
                     }
                     if (!any(c("TrainLayer", "TrainMetaLayer") %in% class(train_layer))) {
                       stop("A Lrner can only belong to a TrainLayer or a TrainMetaLayer object.")
                     }
                     # if (!is.logical(na_rm)) {
                     #   stop("na.rm must be a logical value\n")
                     # } else {
                     #   private$na_rm = na_rm
                     # }
                     impute = FALSE
                     if (na_action == "na.keep") {
                       na_rm = FALSE
                     } else {
                       if (na_action == "na.rm") {
                         na_rm = TRUE
                       } else {
                         if (na_action == "na.impute") {
                           if ("TrainLayer" %in% class(train_layer)) {
                             stop("Imputation is not yet handled for data modalities. Please use either the 'na.keep' or the 'na.rm' option.")
                           }
                           na_rm = FALSE
                           impute = TRUE
                           # Imputation takes place in the Training class, since it
                           # happens after meta-data have been generated.
                           train_layer$getTraining()$setImpute(impute = impute)
                         } else {
                           stop("na_action must be one of 'na.fails', 'na.rm' or 'na.impute'.")
                         }
                       }
                     }
                     private$na_rm = na_rm
                     private$na_action = na_action
                     # Instantiate a Lrner object
                     # Remove learner if already existing
                     if (train_layer$checkLrnerExist()) {
                       key_class = train_layer$getKeyClass()
                       key = key_class[key_class$class == "Lrner", "key"]
                       train_layer$removeFromHashTable(key = key)
                     }
                     private$train_layer = train_layer
                     # Add to object to ht
                     train_layer$add2HashTable(key = private$id,
                                               value = self,
                                               .class = "Lrner")
                   },
                   #' @description
                   #' Printer
                   #' @param ... `any`
                   #'
                   print = function (...) {
                     cat(sprintf("Learner          : %s\n", private$id))
                     cat(sprintf("TrainLayer       : %s\n", private$train_layer$getId()))
                     cat(sprintf("Package          : %s\n", private$package))
                     cat(sprintf("Learn function   : %s\n", private$lrn_fct))
                   },
                   #' @description
                   #' Printer
                   #' @param ... `any`
                   #'
                   summary = function (...) {
                     cat(sprintf("      Learner          : %s\n", private$id))
                     cat(sprintf("      TrainLayer       : %s\n", private$train_layer$getId()))
                     cat(sprintf("      Package          : %s\n", private$package))
                     cat(sprintf("      Learn function   : %s\n", private$lrn_fct))
                   },
                   #' @description
                   #' Learner and prediction parameter interface. Use this function
                   #' to provide how the following parameters are named in the learning
                   #' function (\code{lrn_fct}) you provided when creating the learner, or in the predicting function.
                   #'
                   #' @param x `character` \cr
                   #' Name of the argument to pass the matrix of independent variables in the original learning function.
                   #' @param y `character` \cr
                   #' Name of the argument to pass the response variable in the original learning function.
                   #' @param object `character` \cr
                   #' Name of the argument to pass the model in the original predicting function.
                   #' @param data `character` \cr
                   #' Name of the argument to pass new data in the original predicting function.
                   #' @param extract_pred_fct `character` or `function` \cr
                   #' If the predict function that is called for the model does not return a vector, then
                   #' use this argument to specify a (or a name of a) function that can be used to extract vector of predictions.
                   #' Default value is NULL, if predictions are in a vector.
                   #' @export
                   #'
                   # TODO: Covr me
                   # nocov start
                   interface = function (x = "x",
                                         y = "y",
                                         object = "object",
                                         data = "data",
                                         extract_pred_fct = NULL) {
                     if (!is.character(x)) {
                       stop("String expected for x.")
                     }
                     if (!is.character(y)) {
                       stop("String expected for y.")
                     }
                     if (!is.character(object)) {
                       stop("String expected for object.")
                     }
                     if (!is.character(data)) {
                       stop("String expected for data.")
                     }
                     if (!is.character(extract_pred_fct) & !is.function(extract_pred_fct) & !is.null(extract_pred_fct)) {
                       stop("String or function expected for extract_pred_fct.")
                     } else {
                       if (!is.null(extract_pred_fct)) {
                         if (length(formals(extract_pred_fct)) > 1L) {
                           stop("Only one argument expected for the function specified in extract_pred_fct.")
                         }
                       }
                     }
                     param_interface = data.frame(standard = c("x_name", "y_name", "object_name", "data_name"),
                                                  original = c(x, y, object, data))
                     private$param_interface = param_interface
                     private$extract_pred_fct = extract_pred_fct
                   },
                   # nocov end
                   #' @description
                   #' Tains the current learner (from class [Lrner]) on the current training data (from class [TrainData]).
                   #'
                   #' @param ind_subset `vector` \cr
                   #' Individual ID subset on which the training will be performed.
                   #' @param use_var_sel `boolean` \cr
                   #' If TRUE, variable selection is performed before training.
                   #' @param verbose `boolean` \cr
                   #' Warning messages will be displayed if set to TRUE.
                   #'
                   #' @return
                   #' The resulting model, from class [Model], is returned.
                   #' @export
                   #'
                   train = function (ind_subset = NULL,
                                     use_var_sel = FALSE,
                                     verbose = TRUE) {
                     train_data = private$train_layer$getTrainData()
                     train_data = train_data$clone(deep = FALSE)
                     # Train only on complete data
                     if (private$na_rm) {
                       all_data = train_data$getDataFrame()
                       complete_data = train_data$getCompleteData()
                       train_data$setDataFrame(data_frame = complete_data)
                     }
                     if (is.null(private$package)) {
                       lrn = private$lrn_fct
                     } else {
                       lrn = sprintf('%s::%s', private$package, private$lrn_fct)
                     }
                     lrn_param = private$param_train
                     # Prepare training dataset: extract individual subset
                     if (!is.null(ind_subset)) {
                       train_data = train_data$getIndSubset(
                         var_name = train_data$getIndCol(),
                         value = ind_subset)
                       private$ind_subset = ind_subset
                     } else {
                       private$ind_subset = "ALL"
                     }
                     # Prepare training dataset: extract variable subset
                     if (use_var_sel) {
                       var_sel_obj = private$train_layer$getVarSel()
                       selected_var = var_sel_obj$getVarSubSet()
                       # Reduce features if at least one variable has been selected.
                       if (!is.null(selected_var)) {
                         var_list = c(selected_var, train_data$getTargetName())
                         train_data = train_data$getVarSubset(var_name = var_list)
                         private$var_subset = selected_var
                       } else {
                         # nocov start
                         if (verbose) {
                           warning("No selected variable found, so all variables have been used for training.\n")
                         }
                         private$var_subset = "ALL"
                         # nocov end
                       }
                     } else {
                       private$var_subset = "ALL"
                     }
                     # Set x and y parameters.
                     if (is.null(private$param_interface)) {
                       lrn_param$x = train_data$getData()
                       lrn_param$y = train_data$getTargetValues()
                     } else {
                       # TODO: covr me
                       # nocov start
                       x_name = private$param_interface[private$param_interface$standard == "x_name", "original"]
                       y_name = private$param_interface[private$param_interface$standard == "y_name", "original"]
                       lrn_param[[x_name]] = train_data$getData()
                       lrn_param[[y_name]] = train_data$getTargetValues()
                       # nocov end
                     }
                     base_model = do.call(eval(parse(text = lrn)), lrn_param)
                     model = Model$new(lrner = self,
                                       train_data = train_data,
                                       base_model = base_model,
                                       train_layer = private$train_layer)
                     private$ind_subset = ind_subset
                     # Reset all the data to the TrainData
                     # nocov start
                     if (private$na_rm) {
                       train_data$setDataFrame(data_frame = all_data)
                     }
                     # nocov end
                     # Update learner into the hash table
                     # TODO: Maybe not needed bacause addressing by reference
                     private$train_layer$add2HashTable(key = private$id,
                                                       value = self,
                                                       .class = "Lrner")
                     return(model)
                   },
                   #' @description
                   #' The current layer is returned.
                   #'
                   #' @return
                   #' [TrainLayer] object.
                   #' @export
                   #'
                   getTrainLayer = function () {
                     return(private$train_layer)
                   },
                   #' @description
                   #' The current layer is returned.
                   getNaRm = function () {
                     return(private$na_rm)
                   },
                   #' @description
                   #' The current layer is returned.
                   getNaAction = function () {
                     return(private$na_action)
                   },
                   #' @description
                   #' Getter of the current learner ID.
                   #'
                   #' @return
                   #' The current learner ID.
                   #' @export
                   #'
                   getId = function () {
                     return(private$id)
                   },
                   #' @description
                   #' Getter of the learner package implementing the learn function.
                   #'
                   #' @return
                   #' The name of the package implementing the learn function.
                   #'
                   #' @export
                   #'
                   getPackage = function () {
                     return(private$package)
                   },
                   #' @description
                   #' Getter of the learner package implementing the learn function.
                   #'
                   #' @return
                   #' The name of the package implementing the learn function.
                   #'
                   getIndSubset = function () {
                     return(private$ind_subset)
                   },
                   #' @description
                   #' Getter of the variable subset used for training.
                   #'
                   #' @return
                   #' The list of variables used for training is returned.
                   #'
                   getVarSubset = function () {
                     return(private$ind_subset)
                   },
                   #' @description
                   #' Getter predicting parameter list.
                   #'
                   #' @return
                   #' The list of predicting parameters.
                   #'
                   getParamPred = function () {
                     return(private$param_pred)
                   },
                   #' @description
                   #' The current parameter interface is returned.
                   #'
                   #' @return
                   #' A data.frame of interface.
                   #'
                   getParamInterface = function () {
                     return(private$param_interface)
                   },
                   #' @description
                   #' The function to extract predicted values is returned.
                   #'
                   #' @return
                   #' A data.frame of interface.
                   #'
                   getExtractPred = function () {
                     return(private$extract_pred_fct)
                   }
                 ),
                 private = list(
                   # ID field.
                   id = character(0L),
                   # Package defining the learner (like \code{ranger}).
                   package = NULL,
                   # Learn function name (like \code{ranger}).
                   lrn_fct = NULL,
                   # Parameters of the learn function.
                   param_train = list(0L),
                   # Parameters of the predict function.
                   param_pred = list(0L),
                   # Parameter interface to original names of arguments in original learning and predict function.
                   param_interface = NULL,
                   na_rm = NULL,
                   na_action = NULL,
                   # Training layer (from class [TainLayer] or [TrainMetaLayer]) of the current learner.
                   train_layer = NULL,
                   # Individuals subset IDs.
                   ind_subset = NULL,
                   # Variable subset IDs.
                   var_subset = NULL,
                   # Function to extract predictions.
                   extract_pred_fct = NULL
                 ),
                 cloneable = FALSE
)

data("multi_omics")
test_that("TrainData: all tests", {
  # Testing and layers
  training <- Training$new(id = "training",
                                ind_col = "IDS",
                                target = "disease",
                                target_df = multi_omics$training$target)
  # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
  # Start testing training data with empty layer                                #
  # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
  expect_error({
    train_data_ge <- TrainData$new(id = "geneexpr",
                                   train_layer = training,
                                   data_frame = multi_omics$training$geneexpr)
  })
  tl_ge <- TrainLayer$new(id = "geneexpr", training = training)
  tl_pr <- TrainLayer$new(id = "proteinexpr", training = training)
  tl_me <- TrainLayer$new(id = "methylation", training = training)
  print(tl_ge)
  # We also prepare the meta layer for the meta analysis.
  tl_meta <- TrainMetaLayer$new(id = "meta_layer", training = training)
  # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
  # Start testing training data                                                  #
  # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
  # TrainData can be added successfully
  expect_no_error({
    train_data_ge <- TrainData$new(id = "geneexpr",
                                   train_layer = tl_ge,
                                   data_frame = multi_omics$training$geneexpr)
    train_data_pr <- TrainData$new(id = "proteinexpr",
                                   train_layer = tl_pr,
                                   data_frame = multi_omics$training$proteinexpr)
    train_data_me <- TrainData$new(id = "methylation",
                                   train_layer = tl_me,
                                   data_frame = multi_omics$training$methylation)
    train_data_me$getTrainLayer()
    print(train_data_me)
  })

  # On wrong target or ID column
  expect_error({
    ts_wrong_target <- Training$new(id = "training",
                                      ind_col = "IDS",
                                      target = "diseases",
                                      target_df = multi_omics$training$target)
  })
  expect_error({
    tl_ge_wrong_target <- TrainLayer$new(id = "geneexpr",
                                         training = ts_wrong_target)
  })
  expect_error({
    TrainData$new(id = "geneexpr",
                  train_layer = tl_ge_wrong_target,
                  data_frame = multi_omics$training$geneexpr)
  })

  # No target variable
  ts_missing_target <- Training$new(id = "training",
                                    ind_col = "IDS",
                                    target = "disease",
                                    target_df = multi_omics$training$target)
  tl_ge_missing_target <- TrainLayer$new(id = "geneexpr",
                                       training = ts_missing_target)
  expect_no_error({
    tmp <- multi_omics$training$geneexpr
    # Re-build this test case
    # tmp$disease[1] <- NA
    TrainData$new(id = "geneexpr",
                  train_layer = tl_ge_missing_target,
                  data_frame = tmp)
  })
  # Update TrainData
  expect_no_error({
    TrainData$new(id = "proteinexpr",
                  train_layer = tl_ge_missing_target,
                  data_frame = tmp)
  })
  expect_error({
    TrainData$new(id = "proteinexpr",
                  train_layer = tl_meta,
                  data_frame = tmp)
  })

  # Target variable must be binary or dichotome
  expect_error({
    ts_not_bin_target <- Training$new(id = "training",
                                        ind_col = "IDS",
                                        target = "disease",
                                        target_df = multi_omics$training$target)
    tl_ge_not_bin_target <- TrainLayer$new(id = "geneexpr_no_bin",
                                           training = ts_not_bin_target)
    tmp <- multi_omics$training$geneexpr
    tmp$disease <- sample(x = letters[1L:3L],
                          size = length(tmp$disease),
                          replace = TRUE)
    TrainData$new(id = "geneexpr",
                  train_layer = tl_ge_not_bin_target,
                  data_frame = tmp)
  })
})

#' @title TrainMetaLayer Class
#'
#' @description
#' This class implement a meta meta layer. A [TrainMetaLayer] can only exist as unique element of a [Training] object.
#'
#' A layer is structured as followed:
#'
#' * [Lrner]: It is set by the user to be trained on the meta training data.
#' * [TrainData]: It are modality-specific prediction data, automatically created by the internal cross validation.
#' * [Model]: The meta model, result of training the learner on the training data, and therefore, not to be set by the user.
#' * [TestData]: The meta new data to be predicted, consisting in predictions obtained from each layer.
#'
#' A meta layer can train its meta learner on the meta training data and store the resulting meta model.
#' The meta layer can predict values given a new meta layer.
#'
#' @export
#' @importFrom R6 R6Class
TrainMetaLayer <- R6Class("TrainMetaLayer",
                          inherit = HashTable,
                          public = list(
                            #' @description
                            #' constructor
                            #'
                            #' @param id `character`\cr
                            #' Id of training meta-layer.
                            #' @param training `Training`\cr
                            #'
                            initialize = function (id, training) {
                              super$initialize(id = id)
                              private$training = training
                              training$add2HashTable(key = id,
                                                     value = self,
                                                     .class = "TrainMetaLayer")
                              private$status = FALSE
                            },
                            #' @description
                            #' Printer
                            #' @param ... `any`
                            #'
                            print = function (...) {
                              if (!private$status) {
                                status = "Not trained"
                              } else {
                                status = "Trained"
                              }
                              cat(sprintf("TrainMetaLayer    : %s\n", private$id))
                              cat(sprintf("Status            : %s\n", status))
                              stored_obj = self$getKeyClass()
                              if (!nrow(stored_obj)) {
                                cat("Empty layer.\n")
                              } else {
                                cat(sprintf("Nb. of objects stored : %s\n", nrow(stored_obj)))
                                print(stored_obj)
                              }
                            },
                            #' @description
                            #' Getter of the current training object.
                            #'
                            #' @return
                            #' The current training object is returned.
                            #'
                            getTraining = function () {
                              return(private$training)
                            },
                            #' @description
                            #' Getter of the target object.
                            #' @export
                            getTargetObj = function () {
                              return(private$training$getTargetObj())
                            },
                            #' @description
                            #' Trains the current layer.
                            #'
                            #' @param ind_subset `vector` \cr
                            #' ID subset of individuals to be used for training.
                            #' @param verbose `boolean` \cr
                            #' Warning messages will be displayed if set to TRUE.
                            #'
                            #' @return
                            #' The current layer is returned with the resulting model.
                            #' @export
                            #'
                            train = function (ind_subset = NULL,
                                              verbose = TRUE) {
                              layer_kc = self$getKeyClass()
                              # Stop if either learner of data is missing on this layer.
                              if (!("Lrner" %in% layer_kc[ , "class"])){
                                stop(sprintf("No learner on layer %s.", self$getId()))
                              } else {
                                if (!("TrainData" %in% layer_kc[ , "class"])) {
                                  stop(sprintf("No data on layer %s.", self$getId()))
                                }
                              }
                              # The learner is trained on the current dataset
                              lrner_key = layer_kc[layer_kc$class == "Lrner" , "key"]
                              lrner = self$getFromHashTable(key = lrner_key[1L])
                              model = lrner$train(ind_subset = ind_subset)
                              # Updating the training status.
                              if (!private$status) {
                                # The training layer has not been trained before.
                                private$training$increaseNbTrainedLayer()
                                private$status = TRUE
                              } else {
                                # The training layer has been trained before.
                                private$status = TRUE
                              }
                              return(model)
                            },
                            #' @description
                            #' Predicts values for the new layer taking as argument.
                            #'
                            #' @param new_layer `TrainLayer` \cr
                            #' A trained TrainLayer object.
                            #' @param ind_subset `vector` \cr
                            #' Index subset.
                            #'
                            #' @return
                            #' A new object with the predicted values is returned.
                            #' @export
                            #'
                            predict = function (new_layer,
                                                ind_subset = NULL) {
                              k = self$getId()
                              # Layer IDs must match together.
                              if (k == new_layer$getId()) {
                                m_layer = self$getModel()
                              } else {
                                # This code part is not externally accessible by user, so untestable.
                                # nocov start
                                stop("The new layer ID does not match with the current layer ID.")
                                # nocov end
                              }
                              testing_data = new_layer$getTestData()
                              # Predicting: Data and model exist on this layer.
                              # Initialize a layer to store predictions
                              # pred_layer = HashTable$new(id = self$getId())
                              pred_layer = PredictLayer$new(id = self$getId())
                              model = self$getModel()
                              # Layer specific prediction
                              pred = model$predict(testing_data = testing_data,
                                                   ind_subset = ind_subset)
                              # Store predictions

                              pred_layer$add2HashTable(key = "predict",
                                                       value = pred,
                                                       .class = "PredictData")

                              return(pred_layer)
                            },
                            #' @description
                            #' Imputes missing values in modality-specific predictions.
                            #' Only mode and median based imputations are actually supported.
                            #'
                            #' @param impute_fct `character` \cr
                            #' An imputation function to use instead of median or mode imputation.
                            #' This parameter is actually not used.
                            #' This corresponds to median or mode based imputation.
                            #' @param impute_param `list` \cr
                            #' The list of parameters to call the imputation function. Not yet implemented!
                            #' @return
                            #' A new object with the predicted values is returned.
                            #'
                            impute = function (impute_fct = NULL,
                                               impute_param = NULL) {
                              train_data = self$getTrainData()
                              train_data$impute(impute_fct = impute_fct,
                                                impute_param = impute_param,
                                                target_name = self$getTraining()$getTargetObj()$getTargetName())
                              invisible(self)
                            },
                            #' @description
                            #' Getter of the training dataset stored on the current layer.
                            #'
                            #' @return
                            #' The stored [TrainData] object is returned.
                            #' @export
                            #'
                            getTrainData = function () {
                              layer_kc = self$getKeyClass()
                              if ("TrainData" %in% layer_kc[ , "class"]) {
                                train_data_key = layer_kc[layer_kc$class == "TrainData" ,
                                                          "key"]
                                train_data = self$getFromHashTable(key = train_data_key[1L])
                              } else {
                                stop(sprintf("No train data on layer %s.", self$getId()))
                              }
                              # call_stack <- sys.calls()
                              # caller_name <- as.character(call_stack[[length(call_stack) - 1]])[1]
                              # stop(sprintf("%s", caller_name))
                              return(train_data)
                            },
                            #' @description
                            #' Getter of the learner.
                            #'
                            #' @return
                            #' The stored [Lrner] object is returned.
                            #' @export
                            #'
                            getLrner = function () {
                              layer_kc = self$getKeyClass()
                              if (!("Lrner" %in% layer_kc[ , "class"])) {
                                stop(sprintf("No Lrner on layer %s.", self$getId()))
                              } else {
                                lrner_key = layer_kc[layer_kc$class == "Lrner" ,
                                                     "key"]
                                lrner = self$getFromHashTable(key = lrner_key[1L])
                              }
                              return(lrner)
                            },
                            #' @description
                            #' Getter of the model.
                            #'
                            #' @return
                            #' The stored [Model] object is returned.
                            #' @export
                            getModel = function () {
                              layer_kc = self$getKeyClass()
                              if (!("Model" %in% layer_kc[ , "class"])) {
                                stop(sprintf("No Model on layer %s.", self$getId()))
                              } else {
                                model_key = layer_kc[layer_kc$class == "Model" ,
                                                     "key"]
                                model = self$getFromHashTable(key = model_key[1L])
                              }
                              return(model)
                            },
                            #' @description
                            #' Open access to the meta layer. A meta learner is only
                            #' modifiable if the access is opened.
                            #'
                            #'
                            openAccess = function () {
                              private$access = TRUE
                              invisible(self)
                            },
                            #' @description
                            #' Close access to the meta layer to avoid accidental
                            #' modification.
                            #'
                            #'
                            closeAccess = function () {
                              private$access = FALSE
                              invisible(self)
                            },
                            #' @description
                            #' Getter of the current access to the meta layer.
                            #'
                            #' @export
                            getAccess = function () {
                              return(private$access)
                            },
                            #' @description
                            #' Create and set an [TrainData] object to the current
                            #' meta learner.
                            #'
                            #' @param id `character` \cr
                            #' ID of the [TrainData] object to be instanciated.
                            #' @param ind_col `character` \cr
                            #' Name of individual column IDs.
                            #' @param data_frame  `data.frame` \cr
                            #' \code{data.frame} of layer specific predictions.
                            #'
                            #' @export
                            # TODO: Please do not export me.
                            setTrainData = function (id,
                                                     ind_col,
                                                     data_frame) {
                              # nocov start
                              if (sum(!complete.cases(data_frame)) == nrow(data_frame)) {
                                warning("No individual fully overlaps across all layers.")
                              }
                              # nocov end
                              if (self$getLrner()$getNaRm()) {
                                TrainData$new(id = id,
                                              data_frame = data_frame[complete.cases(data_frame), ],
                                              train_layer = self)
                              } else {
                                TrainData$new(id = id,
                                              data_frame = data_frame,
                                              train_layer = self)
                              }
                              return(self)
                            },
                            #' @description
                            #' Check whether a training data has been already stored.
                            #'
                            #' @return
                            #' Boolean value
                            #'
                            checkLrnerExist = function () {
                              return(super$checkClassExist(.class = "Lrner"))
                            },
                            #' @description
                            #' Check whether a model has been already stored.
                            #'
                            #' @return
                            #' Boolean value
                            #'
                            checkModelExist = function () {
                              return(super$checkClassExist(.class = "Model"))
                            },
                            #' @description
                            #' Check whether a training data has been already stored.
                            #'
                            #' @return
                            #' Boolean value
                            #'
                            checkTrainDataExist = function () {
                              # Fix predicted20242806 as reserved word
                              test = super$checkClassExist(.class = "TrainData") & ("predicted20242806" %in% private$key_class$class)
                            },
                            #' @description
                            #' Only usefull to reset status FALSE after cross validation.
                            set2NotTrained = function () {
                              private$status = FALSE
                            },
                            #' @description
                            #' Generate summary.
                            #'
                            #' @export
                            #'
                            summary = function () {
                              cat("   MetaLayer\n")
                              cat("   ----------------\n")
                              if (!private$status) {
                                status = "Not trained"
                              } else {
                                status = "Trained"
                              }
                              cat(sprintf("   TrainMetaLayer    : %s\n", private$id))
                              cat(sprintf("   Status            : %s\n", status))
                              stored_obj = self$getKeyClass()
                              if (!nrow(stored_obj)) {
                                cat("   Empty layer.\n")
                              } else {
                                cat(sprintf("   Nb. of objects stored : %s\n", nrow(stored_obj)))
                              }
                              cat("\n")
                              cat("   ----------------\n")
                              layer_kc = self$getKeyClass()
                              cat("   Object(s) on MetaLayer\n\n")
                              if (!nrow(layer_kc)) {
                                cat("      Empty layer\n")
                              }
                              for (k in layer_kc[layer_kc$class != "Model", "key"]) {
                                cat("      ----------------\n")
                                current_obj = self$getFromHashTable(key = k)
                                current_obj$summary()
                                cat("      ----------------\n")
                                cat("\n")
                              }
                            }
                          ),
                          private = list(
                            # The current training object.
                            training = NULL,
                            # Access to the meta layer.
                            access = FALSE,
                            status = FALSE
                          ),
                          cloneable = FALSE
)

data("multi_omics")
test_that("Model: all tests", {
  # Prepare training object and layers
  training <- Training$new(id = "training",
                                ind_col = "IDS",
                                target = "disease",
                                target_df = multi_omics$training$target)
  tl_ge <- TrainLayer$new(id = "geneexpr", training = training)
  lrner_ge <- Lrner$new(id = "ranger",
                        package = "ranger",
                        lrn_fct = "ranger",
                        param_train_list = list(probability = TRUE,
                                                mtry = 2L,
                                                num.trees = 10L),
                        train_layer = tl_ge)
  train_data_ge <- TrainData$new(id = "geneexpr",
                                 train_layer = tl_ge,
                                 data_frame = multi_omics$training$geneexpr)
  expect_no_error({
    tl_ge$train()
  })
  expect_no_error({
    my_model <- tl_ge$getModel()
    print(my_model)
    my_train_data <- my_model$getTrainData()
    my_train_label <- my_model$getTrainLabel()
    my_model$setId(id = "test")
  })

  testing <- Testing$new(id = "testing", ind_col = "IDS")
  nl_ge <- TestLayer$new(id = "wrong_id", testing = testing)
  testing_data_ge <- TestData$new(id = "geneexpr",
                             new_layer = nl_ge,
                             data_frame = multi_omics$testing$geneexpr)
  expect_error({
    my_model$predict(testing_data_ge)
  })
})

#' Create weights for COBRA Predictions
#'
#' The \code{createWeights} function is used to calculate weights for predictions.
#'
#' @param train A \code{matrix} representing the training data. Rows represent observations,
#' and columns contain predictions from individual learners for these observations.
#' In cases where a prediction is unavailable for a specific observation, \code{NA} is used.
#' @param test A \code{matrix} representing the test data. Rows represent observations,
#' and columns contain predictions from individual learners for these observations.
#' In cases where a prediction is unavailable for a specific observation, \code{NA} is used.
#' @param n_train An \code{integer} specifying the number of training observations.
#' @param n_test An \code{integer} specifying the number of test observations.
#' @param nlearners An \code{integer} representing the number of learners.
#' @param eps A \code{numeric} value representing the threshold for proximity between two predictions.
#' @param alpha A value that determines the optimal number of learners in the neighborhood (only for alpha optimization).
#'
createWeights = function(train, test, n_train, n_test,
                          nlearners, eps, alpha) {

  if(is.null(alpha)){

    # Initialize weights matrix
    weights = matrix(0, nrow = n_test, ncol = n_train)

    for (i in 1:n_test) {
      # Vector to store the number of matching learners for each training point
      vres = rep(0, n_train)
      for (j in 1:n_train) {
        for (k in 1:nlearners) {
          # Check if values are not NA and if the absolute difference is within eps
          if (!is.na(train[n_train*(k-1)+j]) & !is.na(test[n_test*(k-1)+i])){
            if (abs(train[n_train*(k-1)+j]-test[n_test*(k-1)+i]) <= eps){
              vres[j] = vres[j] + 1
            }
          }
        }
      }

      max_matches = max(vres)
      # If no matches found, skip this test point
      if (max_matches == 0) {
        next
      }

      # Iteratively find matches
      for (matches in seq(max_matches, 1, -1)) {
        for (j in 1:n_train) {
          if (vres[j] == matches) {
            weights[i, j] = 1
          }
        }

        # If weights are set for this test point, move to the next one
        if (any(weights[i, ] == 1)) {
          break
        }
      }
    }
  } else {
      # Threshold for number of learners
      ler = alpha * nlearners
      weights = matrix(0, nrow = n_test, ncol = n_train)

      for (i in 1:n_test) {
        # Vector to store the number of matching learners for each training point
        vres = rep(0, n_train)
        for (j in 1:n_train) {
          for (k in 1:nlearners) {
            # Check if values are not NA and if the absolute difference is within eps
            if (!is.na(train[n_train*(k-1)+j]) & !is.na(test[n_test*(k-1)+i]))  {
              if (abs(train[n_train*(k-1)+j]-test[n_test*(k-1)+i]) <= eps){
                vres[j] = vres[j] + 1
              }
            }
          }
          # If the number of matches is greater than or equal to the threshold, set the weight
          if (vres[j] >= ler) {
            weights[i,j] = 1
          }
        }
      }
  }
  return(weights)
}

test_that("createWeights works correctly", {

  # Test 1: Check if 'ler' threshold is correctly calculated
  alpha <- 1/3
  nlearners <- 3
  ler <- alpha * nlearners
  expect_equal(ler, 1)  # ler should be 1 based on alpha * nlearners

  # Test 2: Check if weights matrix is initialized with correct dimensions
  n_train <- 5
  n_test <- 3
  weights <- matrix(0, nrow = n_test, ncol = n_train)
  expect_equal(dim(weights), c(n_test, n_train))  # Check matrix dimensions


  # Test 3: Check if the function handles the case with no matches correctly (weights should be 0)
  train <- c(0.1, 0.2, 0.3, 0.4, 0.5)
  test <- c(0.1, 0.25, 0.35)
  vres_no_match <- rep(0, length(train))
  weights_no_match <- matrix(0, nrow = 3, ncol = 5)  # No matches expected

  for (i in 1:3) {
    for (j in 1:5) {
      if (vres_no_match[j] >= ler) {
        weights_no_match[i, j] <- 1
      }
    }
  }

  # Check that no weights are set when there are no matches
  expect_equal(weights_no_match, matrix(0, nrow = 3, ncol = 5))  # Should return a matrix of 0s

  # Test 4: Check if the function returns weights when alpha is NULL
  eps <- 0.1
  alpha <- NULL  # Test without the alpha parameter (default behavior)
  weights_no_alpha <- createWeights(train, test, n_train = n_train,
                                     n_test = n_test, nlearners = nlearners,
                                     eps = eps, alpha = alpha)

  # Test if the function returns a matrix (it should for any valid input)
  expect_true(is.matrix(weights_no_alpha))
  expect_equal(dim(weights_no_alpha), c(n_test, n_train))  # The dimensions should match

  # Test 5: Check if the function returns weights
  eps <- 0.1
  alpha <- 1/3
  weights <- createWeights(train, test, n_train = n_train,
                                     n_test = n_test, nlearners = nlearners,
                                     eps = eps, alpha = alpha)

  # Test if the function returns a matrix (it should for any valid input)
  expect_true(is.matrix(weights))
  expect_equal(dim(weights), c(n_test, n_train))  # The dimensions should match

})

#' @title Predicting Class
#'
#' @description
#' This class is designed for predictions.
#'
#'  The Predicting is structured as followed:
#' * [PredictLayer]: Exists for each modality.
#'    - [PredictData]: Related class for modality-specific predictions.
#' * [PredictMetaLayer]: Related class for meta predictions.
#'    - [PredictData]: Specific to the meta layer, it is set up internally after cross-validation.
#'
#' Use the function \code{train} for training and \code{predict} for predicting.
#'
#'TODO: Do not export me.
#' @export
#'
#' @importFrom R6 R6Class
#'
#' @seealso [TrainLayer]
Predicting <- R6Class("Predicting",
                    inherit = HashTable,
                    public = list(
                      #' @description
                      #' constructor
                      #'
                      #' @param id `character`\cr
                      #' Predicting id.
                      #' @param ind_col `character`
                      #' Name of column of individuals IDS
                      initialize = function (id, ind_col) {
                        super$initialize(id = id)
                        private$ind_col = ind_col
                      },
                      #' @description
                      #' Printer
                      #'
                      #' @param ... `any`
                      #'
                      print = function (...) {
                        cat(sprintf("Predicting   : %s\n", private$id))
                        cat(sprintf("Nb. layers   : %s\n", length(private$hash_table)))
                      },
                      #' @param meta_layer_id (`character(1)`) \cr
                      #' ID of the meta layer where the testing meta data will be stored.
                      #'
                      #' @description
                      #' Creates a new modality-specific predictions dataset based on layer predictions.
                      #'
                      #' @return
                      #' A [TestData] is returned.
                      #' @export
                      #'
                      createMetaTestData = function (meta_layer_id) {
                        key_class_predicting = self$getKeyClass()
                        predicted_values = NULL
                        for (k in key_class_predicting[ , "key"]) {
                          pred_layer = self$getFromHashTable(key = k)
                          pred_data = pred_layer$getPredictData()
                          pred_values = pred_data$getPredictData()
                          predicted_values = data.frame(rbind(predicted_values,
                                                              pred_values))
                        }
                        # Will transform meta data.frame into wide format. In case of data.frame, only the first column is considered.
                        predicted_values_wide = reshape(predicted_values,
                                                        idvar = colnames(predicted_values)[2L],
                                                        timevar = colnames(predicted_values)[1L],
                                                        direction = "wide")
                        colname_vector = gsub(pattern = "Prediction.",
                                              replacement = "",
                                              x = names(predicted_values_wide))
                        names(predicted_values_wide) = colname_vector
                        ind_ids = self$getIndIDs()
                        predicted_values_wide = merge(x = ind_ids,
                                                      y = predicted_values_wide,
                                                      by = colnames(ind_ids)[1L],
                                                      all.y = TRUE)
                        # Add layer specific predictions to a new predicted meta layer
                        testing_meta_layer = TestMetaLayer$new(id = meta_layer_id,
                                                       testing = self)
                        # FIXME: Move this: Data should be created by the layer.
                        testing_meta_layer$openAccess()
                        new_meta_data = TestData$new(id = "predicted",
                                                    new_layer = testing_meta_layer,
                                                    data_frame = predicted_values_wide)
                        testing_meta_layer$closeAccess()
                        return(new_meta_data)
                      },
                      #' @description
                      #' Gather individual IDs from all layer.
                      #'
                      #' @return
                      #' A \code{data.frame} containing individuals IDs.
                      #' @export
                      #'
                      getIndIDs = function() {
                        # FIXME: Adjust to the Predict class
                        layers = self$getKeyClass()
                        # This code accesses each layer (except MetaLayer) level
                        # and get the target variable
                        # FIXME: Replace "Predict" by "PredictLayer"
                        layers = layers[layers$class %in% "PredictLayer", ]
                        ids_data = NULL
                        current_data = NULL
                        for (k in layers$key) {
                          layer = self$getFromHashTable(key = k)
                          ids_data = as.data.frame(rbind(ids_data,
                                                         layer$getIndIDs()))
                        }
                        ids_data = ids_data[!duplicated(ids_data[ , 1L]), ,
                                            drop = FALSE]
                        return(ids_data)
                      },
                      #' @description
                      #' Getter of the meta layer.
                      #'
                      #' @return
                      #' Object from class [PredictMetaLayer]
                      #' @export
                      #'
                      getPredictMetaLayer = function () {
                        layers = self$getKeyClass()
                        # FIXME: Use PredictMetaLayer instead of class MetaLayer
                        meta_layer_key = layers[layers$class == "PredictMetaLayer" , "key"]
                        meta_layer = self$getFromHashTable(key = meta_layer_key)
                        return(meta_layer)
                      },
                      #' @description
                      #' Getter of the individual column name.
                      #' @export
                      getIndCol = function () {
                        return(private$ind_col)
                      }
                    ),
                    private = list(
                      ind_col = NULL
                    ),
                    cloneable = FALSE
)

#' @title Best specific Learner prediction.
#'
#' @description
#' Predict function for models from class \code{bestLayerLearner}.
#'
#' @include bestLayerLearner.R
#'
#' @param object `bestLayerLearner` \cr
#' An object from class [bestLayerLearner]
#' @param data `data.frame` \cr
#' New data to predicted.
#' @param ... \code{any}\cr
#' Further arguments passed to or from other methods.
#' @return
#' Predicted target values are returned.
#'
#' @export
#' @method predict bestLayerLearner
#'
#' @examples
#' set.seed(20240625)
#' x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
#' y <- sample(x = 0:1, size = 50L, replace = TRUE)
#' my_model <- bestLayerLearner(x = x, y = y)
#' x_new <- data.frame(x1 = rnorm(10L))
#' my_predictions <- predict(object = my_model, data = x_new)
#'
#'
predict.bestLayerLearner = function (object, data, ...) {
  if (!inherits(object, "bestLayerLearner")) {
    stop("'object' must be a 'bestLayerLearner'.")
  }
  if (!inherits(data, "data.frame")) {
    stop("Error: 'newdata' must be an object from the class 'data.frame'.")
  }
  if (all(names(object) %in% names(data))) {
    pred = apply(data[ , names(object), drop = FALSE], 1L, function (tmp_row) {
      for (rank in seq_along(object)) {
        model_name = names(object)[which(object == rank)]
        if (!is.na(tmp_row[model_name])) {
          return(tmp_row[model_name])
        }
      }
    })
    return(list(predictions = unlist(pred)))
  } else {
    stop("Names of weights do not match with name columns in data.")
  }
}

#' Create Difference
#'
#'The \code{createDif} function computes the difference between the maximum and minimum predictions in a dataset.
#' @param x Predictions vector

createDif = function (x) {

  # Check if the vector is empty or contains only NA values
  if (length(x) == 0 || all(is.na(x))) {
    stop("Error: Input vector is either empty or contains only NA values.")
  } else if (!is.numeric(x)) {
    stop("Error: Input must be a numeric vector.")
  }

  return(max(x, na.rm = TRUE)-min(x, na.rm = TRUE))
}

test_that("createDif works", {

  # Test 1: Basic functionality with positive numbers
  x <- c(1, 2, 3, 4, 5)
  expect_equal(createDif(x), 4) # max - min = 5 - 1

  # Test 2: Mixed positive and negative numbers
  x <- c(-5, -1, 0, 3, 10)
  expect_equal(createDif(x), 15) # max - min = 10 - (-5)

  # Test 3: Only one value
  x <- c(7)
  expect_equal(createDif(x), 0) # max - min = 7 - 7

  # Test 4: All values are the same
  x <- c(4, 4, 4, 4)
  expect_equal(createDif(x), 0) # max - min = 4 - 4

  # Test 5: NA values included
  x <- c(NA, 1, 2, 3, NA)
  expect_equal(createDif(x), 2) # max - min = 3 - 1, ignoring NA

  # Test 6: All values are NA
  x <- c(NA, NA, NA)
  expect_error(createDif(x))

  # Test 7: Empty vector
  x <- numeric(0)
  expect_error(createDif(x))

  # Test 8: Non-numeric input
  x <- c("a", "b", "c")
  expect_error(createDif(x))

})

#' @title The best layer-specific model is used as meta model.
#'
#' @description
#' The meta learner is the best layer-specific learner. This function is intended to be (internally) used as meta-learner in fuseMLR.
#'
#' @param x `data.frame` \cr
#' \code{data.frame} of predictors.
#' @param y `vector` \cr
#' True target observations. Either binary or two level factor variable.
#' @param perf `function` \cr
#' Function to compute layer-specific performance of learners. If NULL, the Brier Score (classification) or a mean squared error (regression) is used by default as performance measure.
#' Otherwise, the performance function must accept two parameters: \code{observed} (observed values) and \code{predicted} (predicted values).
#'
#' @return
#' A model object of class \code{weightedMeanLeaner}.
#'
#' @export
#'
#' @examples
#' set.seed(20240624L)
#' x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
#' y = sample(x = 0L:1L, size = 50L, replace = TRUE)
#' my_best_model = bestLayerLearner(x = x, y = y)
bestLayerLearner = function (x, y, perf = NULL) {
  if (!is.data.frame(x)) {
    stop("x must be a data.frame.")
  }
  if (is.null(perf)) {
    if (is.numeric(y) & (length(unique(y)) > 2)) {
      perf_values = lapply(X = x, FUN = function (predicted) {
        value_esti <- mean(x = (predicted - y)^2, na.rm = TRUE)
        # nocov start
        if (value_esti == 0) {
          # Avoid Brier score equals value
          value_esti = .Machine$double.eps
        }
        return(value_esti)
        # nocov end
      })
    } else {
      if ((length(unique(y)) > 2) | is.character(y)) {
        stop("y must be either binary or two level factor variable.\n")
      } else {
        if (is.factor(y)) {
          y = as.integer(y) - 1
        } else {
          if (!all(y %in% 0:1)) {
            stop("y must take its values between 0 and 1.\n")
          }
        }
        perf_values = lapply(X = x, FUN = function (predicted) {
          mean(x = (predicted - y)^2, na.rm = TRUE)
        })
      }
    }
    perf_values = unlist(perf_values)
  } else {
    # nocov start
    if (is.function(perf)) {
      arg_names <- names(formals(perf))
      if (all(arg_names %in% c("observed", "predicted"))) {
        # Function has been provided to estimated performance of layer-specific learner
        perf_values = lapply(X = x, FUN = function (predicted) {
          perf_estimate = do.call(what = perf, args = list(observed = y,
                                                           predicted = predicted))
          return(perf_estimate)
        })
        perf_values = unlist(perf_values)
      } else {
        stop("perf argument must be a function.")
      }
    } else {
      stop("Arguments of the perf function must be 'observed' and 'predicted'.")
    }
  }
  # nocov end
  weights_values = (1L / perf_values) / sum((1L / perf_values))
  ranking = order(-weights_values)
  ordered_weights = rep(0L, length(weights_values))
  ordered_weights[ranking] = seq_along(weights_values)
  names(ordered_weights) = names(x)
  class(ordered_weights) = "bestLayerLearner"
  return(ordered_weights)
}

#' @title Model Class
#'
#' @description
#' This class implements a model. A [Model] object can only exist as element of a
#' [TrainLayer] or a [TrainMetaLayer] object. A [Model] object is
#' automatically created by fitting a learner on a training data.
#'
#' A [Model] object can compute predictions for a [TestData] object. See the \code{predict} function below.
#'
#' @export
#'
#' @importFrom R6 R6Class
#'
Model <- R6Class("Model",
                 public = list(
                   #' @description
                   #' Constructor of Model class.
                   #'
                   #' @param lrner `Lrner` \cr
                   #' The learner.
                   #' @param train_data `TrainData(1)` \cr
                   #' Training data.
                   #' @param base_model `object` \cr
                   #' Base model as returned by the original learn function.
                   #' @param train_layer `TrainLayer` \cr
                   #' The current training layer on which the model is stored.
                   #'
                   #' @return
                   #' An object is returned.
                   #'
                   #'
                   initialize = function (lrner,
                                          train_data,
                                          base_model,
                                          train_layer) {
                     private$lrner = lrner
                     private$train_data = train_data
                     private$base_model = base_model
                     private$train_layer = train_layer
                     if (any(c("TrainLayer", "TrainMetaLayer") %in% class(train_layer))) {
                       if (train_layer$checkModelExist()) {
                         train_layer$removeFromHashTable(key = sprintf("%sMo", lrner$getId()))
                       }
                       train_layer$add2HashTable(key = sprintf("%sMo", lrner$getId()),
                                                 value = self,
                                                 .class = "Model")
                     } else {
                       # nocov start
                       stop("A Model can only belong to a TrainLayer or a TrainMetaLayer.")
                       # nocov end
                     }
                   },
                   #' @description
                   #' Printer
                   #' @param ... `any`
                   #'
                   print = function (...) {
                     cat("Class           : Model\n\n")
                     cat("Learner info.   \n")
                     cat("-----------------------\n")
                     print(private$lrner)
                     cat("\n")
                     cat("Train data info.      \n")
                     cat("-----------------------\n")
                     print(private$train_data)
                   },
                   #' @description
                   #' Summary
                   #' @param ... `any`
                   #'
                   summary = function (...) {
                     cat("      Model           \n\n")
                     cat("      -----------------------\n")
                     cat(sprintf("Individual(s) used : %s\n", length(private$lrner$getVarSubset())))
                     cat(sprintf("Variable(s) used   : %s\n", length(private$lrner$getIndSubset())))
                     cat("      -----------------------\n")
                   },
                   #' @description
                   #' Getter of the base model
                   #'
                   #' @export
                   #'
                   getBaseModel = function () {
                     return(private$base_model)
                   },
                   #' @description
                   #' Getter of the traning data
                   #'
                   #' @export
                   #'
                   getTrainData = function () {
                     return(private$train_data)
                   },
                   #' @description
                   #' Getter of the individual ID column in the training data.
                   #'
                   #' @param ... `any`
                   #'
                   getTrainLabel = function () {
                     train_data = private$train_data
                     return(train_data$getDataFrame()[ , train_data$getIndCol()])
                   },
                   #' @description
                   #' Getter of the learner use to fit the model.
                   #'
                   #' @export
                   #'
                   getLrner = function () {
                     return(private$lrner)
                   },
                   #' @description
                   #' Setter of the model ID.
                   #'
                   #' @param id `character` \cr
                   #' ID value
                   #'
                   #' @export
                   #'
                   setId = function (id) {
                     private$id = id
                     invisible(self)
                   },
                   #' @description
                   #' Predict target values for the new data
                   #' (from class [TestData]) taken as into.
                   #'
                   #' @param testing_data `TestData` \cr
                   #' An object from class [TestData].
                   #' @param use_var_sel `boolean` \cr
                   #' If TRUE, selected variables available at each layer are used.
                   #' @param ind_subset `vector` \cr
                   #' Subset of individual IDs to be predicted.
                   #' @param ...
                   #' Further parameters to be passed to the basic predict function.
                   #'
                   #' @return
                   #' The predicted object are returned. The predicted object must be either a vector or a list
                   #' containing a field predictions with predictions.
                   #'
                   #' @export
                   #'
                   predict = function (testing_data,
                                       use_var_sel,
                                       ind_subset = NULL) {
                     tmp_lrner = self$getLrner()
                     if(tmp_lrner$getTrainLayer()$getId() != testing_data$getTestLayer()$getId()) {
                       stop("Learner and data must belong to the same layer.")
                     }
                     # Incomplete data are removed if required.
                     testing_data = testing_data$clone(deep = FALSE)
                     # Restrict variables to the subset used for training
                     # if (tmp_lrner$getNaRm()) {
                     #   complete_data = testing_data$getCompleteData()
                     #   testing_data$setDataFrame(data_frame = complete_data)
                     # }
                     # Prepare new dataset
                     if (is.null(ind_subset)) {
                       missing_ind = NULL
                       testing_data = testing_data
                       ind_subset = testing_data$getDataFrame()[ , testing_data$getIndCol()]
                     } else {
                       # Filter individuals with missing values on this layer
                       missing_ind = testing_data$getSetDiff(
                         var_name = testing_data$getIndCol(),
                         value = ind_subset)
                       # Keeping only individuals with observations
                       ind_subset = setdiff(ind_subset, missing_ind)
                       testing_data = testing_data$getIndSubset(
                         var_name = testing_data$getIndCol(),
                         value = ind_subset)
                     }
                     pred_param <- private$lrner$getParamPred()
                     pred_param_object = self$getBaseModel()
                     # Predict using the subset of variables utilized for training
                     training_var = colnames(private$train_data$getData())
                     restricted_testing_data = testing_data$getData()
                     if ("TrainLayer" %in% class(private$train_layer)) {
                       if (private$train_layer$checkVarSelExist() & use_var_sel) {
                         var_sel_obj = private$train_layer$getVarSel()
                         var_sel = var_sel_obj$getVarSubSet()
                         if (!is.null(var_sel)) {
                           restricted_testing_data = restricted_testing_data[, var_sel, drop = FALSE]
                         }
                       }
                     }
                     pred_param_data = restricted_testing_data
                     # Use parameter interface to predict.
                     param_interface = private$lrner$getParamInterface()
                     # Set object and data parameters.
                     if (is.null(param_interface)) {
                       pred_param[["object"]] = pred_param_object
                       pred_param[["data"]] = pred_param_data
                     } else {
                       # TODO: covr me
                       # nocov start
                       object_name = param_interface[param_interface$standard == "object_name", "original"]
                       data_name = param_interface[param_interface$standard == "data_name", "original"]
                       pred_param[[object_name]] = pred_param_object
                       pred_param[[data_name]] = pred_param_data
                       # nocov end
                     }
                     lrn_package = private$lrner$getPackage()
                     if (is.null(lrn_package)) {
                       predict_fct = "predict"
                     } else {
                       predict_fct = sprintf('%s:::%s',
                                             lrn_package,
                                             sprintf("predict.%s", class(self$getBaseModel())[1]))
                     }
                     predicted_obj = do.call(eval(parse(text = predict_fct)),
                                             pred_param)
                     # Extract predictions if necessary.
                     extract_pred_fct = private$lrner$getExtractPred()
                     if (!is.null(extract_pred_fct)) {
                       if (is.character(extract_pred_fct)) {
                         extract_pred_fct = eval(parse(text = extract_pred_fct))
                       }
                       param_extract = list()
                       param_extract[[names(formals(extract_pred_fct))]] = predicted_obj
                       predicted_obj = do.call(what = extract_pred_fct,
                                               args = param_extract)
                     }
                     # The predicted object must be either a vector or a list
                     # containing a field predictions with predictions.
                     if (is.vector(predicted_obj)|is.factor(predicted_obj)) {
                       predicted_obj = data.frame(
                         layer = private$lrner$getTrainLayer()$getId(),
                         id = ind_subset,
                         pred = predicted_obj)
                       pred_colnames = c("Layer",
                                         testing_data$getIndCol(),
                                         "Prediction")
                       names(predicted_obj) = pred_colnames
                     } else {
                       if (is.list(predicted_obj)) {
                         # nocov start
                         if (is.null(predicted_obj$predictions)) {
                           stop("Predicted object must either be a vector or a list containing a vector named 'predictions'.")
                         } else {
                           if (!is.vector(predicted_obj$predictions) & !(any(class(predicted_obj$predictions) %in% c("data.frame", "matrix")))) {
                             stop("Predicted object must either be a vector or a list containing a vector named 'predictions'.")
                           } else {
                             # Choose the second column from data.frame predictions as those returned by ranger.
                             if (any(class(predicted_obj$predictions) %in% c("data.frame", "matrix")) & (private$train_layer$getTraining()$getProblemTyp() == "classification")) {
                               predicted_obj$predictions = predicted_obj$predictions[ , 2L]
                             }
                           }
                           # nocov end
                           predicted_obj = data.frame(
                             layer = private$lrner$getTrainLayer()$getId(),
                             id = ind_subset,
                             pred = predicted_obj$predictions)
                           pred_colnames = c("Layer",
                                             private$train_data$getIndCol(),
                                             "Prediction")
                           names(predicted_obj) = pred_colnames
                         }
                       } else {
                         # nocov start
                         stop("Predicted object must either be a vector or a list containing a field named 'predictions'")
                         # nocov end
                       }
                     }
                     # Ignore all other columns than layer, individual ids and
                     # predicted values
                     predicted_obj = predicted_obj[, pred_colnames]
                     # Add eventual individuals with missing values
                     if (length(missing_ind)) {
                       predicted_obj_missing = data.frame(
                         layer = private$lrner$getTrainLayer()$getId(),
                         id = missing_ind,
                         pred = NA)
                       names(predicted_obj_missing) = pred_colnames
                     } else {
                       predicted_obj_missing = NULL
                     }
                     predicted_obj = data.frame(
                       rbind(predicted_obj,
                             predicted_obj_missing))
                     # Prediction is finished. Re-set the all data if na.rm was TRUE
                     # nocov start
                     predicted_data = PredictData$new(
                       id = testing_data$getId(),
                       ind_col = testing_data$getIndCol(),
                       data_frame = predicted_obj
                     )
                     # nocov end
                     return(predicted_data)
                   }
                 ),
                 private = list(
                   id = character(0L),
                   lrner = NULL,
                   train_data = NULL,
                   base_model = NULL,
                   train_layer = NULL
                 ),
                 cloneable = TRUE
)

#' @title PredictMetaLayer Class
#'
#' @description
#' This class implement a predicted meta layer. A [PredictMetaLayer] can only exist as unique element of a [Training] object.
#'
#' A predicted meta layer can only contain a [PredictData] object.
#'
#' @export
#' @importFrom R6 R6Class
PredictMetaLayer <- R6Class("PredictMetaLayer",
                     inherit = HashTable,
                     public = list(
                       #' @description
                       #' constructor
                       #'
                       #' @param id `character`\cr
                       #' @param predicting `Predicting`\cr
                       #'
                       initialize = function (id, predicting) {
                         super$initialize(id = id)
                         private$predicting = predicting
                         if ("Predicting" %in% class(predicting)) {
                           predicting$add2HashTable(key = id,
                                                       value = self,
                                                       .class = "PredictMetaLayer")
                         } else {
                           stop("A predicted meta layer can only belong to a PredictStudy.")
                         }
                       },
                       #' @description
                       #' Printer
                       #' @param ... `any`
                       #'
                       print = function(...) {
                         cat("Class: PredictMetaLayer\n")
                         cat("Do not modify its instances manually.\n")
                         cat(sprintf("id: %s\n", private$id))
                         cat(sprintf("Contains %s object", length(private$hash_table)))
                       },
                       #' @description
                       #' Getter of the current predicting object
                       #'
                       #' @return
                       #' The current predicting object is returned.
                       #'
                       getPredicting = function () {
                         return(private$predicting)
                       },
                       #' @description
                       #' Getter of IDS from the current layer.
                       #'
                       #' @return
                       #' A \code{data.frame} containing individuals IDs values.
                       #' @export
                       #'
                       getIndIDs = function () {
                         layer_kc = self$getKeyClass()
                         # Stop if training data is missing on this layer.
                         if (("PredictData" %in% layer_kc[ , "class"])) {
                           # Searching for layer specific new dataset
                           data_key = layer_kc[layer_kc$class == "PredictData" ,
                                               "key"]
                           current_data = self$getPredictData()
                         } else {
                           stop(sprintf("No data on layer %s.", self$getId()))
                         }
                         current_data_frame = current_data$getDataFrame()
                         ids_data = current_data_frame[ , current_data$getIndCol(), drop = FALSE]
                         return(ids_data)
                       },
                       #' @description
                       #' Getter of the predicted data.
                       #'
                       #' @return
                       #' The stored [PredictData] object is returned.
                       #' @export
                       #'
                       getPredictData = function () {
                         layer_kc = self$getKeyClass()
                         if (any(c("PredictData") %in% layer_kc[ , "class"])) {
                             predict_data_key = layer_kc[layer_kc$class == "PredictData" ,
                                                     "key"]
                             predict_data = self$getFromHashTable(key = predict_data_key[1L])
                           } else {
                           stop(sprintf("No predicted data on layer %s.", self$getId()))
                         }
                         return(predict_data)
                       },
                       #' @description
                       #' Open access to the meta layer. A meta learner is only
                       #' modifiable if the access is opened.
                       #'
                       #'
                       openAccess = function () {
                         private$access = TRUE
                         invisible(self)
                       },
                       #' @description
                       #' Close access to the meta layer to avoid accidental
                       #' modification.
                       #'
                       #'
                       closeAccess = function () {
                         private$access = FALSE
                         invisible(self)
                       },
                       #' @description
                       #' Getter of the current access to the meta layer.
                       #'
                       #' @export
                       getAccess = function () {
                         return(private$access)
                       }
                     ),
                     private = list(
                       # The current predicting object.
                       predicting = NULL,
                       # Access to the meta layer.
                       access = FALSE
                     ),
                     cloneable = FALSE
)

#' @title TrainData Class
#'
#' @description
#' This class implements the training data. A [TrainData] object can only
#' exist as a component of a [TrainLayer] or a [TrainMetaLayer] object.
#'
#' @export
#'
#' @importFrom R6 R6Class
#' @seealso [TrainLayer], [Lrner], [Model], [TestData]
TrainData <- R6Class("TrainData",
                     inherit = Data,
                     public = list(
                       #' @description
                       #' Initialize a new object from the current class.
                       #'
                       #' @param id `character` \cr
                       #' The Object ID.
                       #' @param data_frame `data.frame`\cr
                       #' \code{data.frame} containing data.
                       #' @param train_layer `TrainLayer` \cr
                       #' Training layer where to store the current object.
                       initialize = function (id,
                                              data_frame,
                                              train_layer) {
                         if (!any(c("TrainLayer", "TrainMetaLayer") %in% class(train_layer))) {
                           stop("A Traindata can only belong a TrainLayer or a TrainMetaLayer object.\n")
                         }
                         if (!is.data.frame(data_frame)) {
                           stop("data_frame must be a data.frame.\n")
                         }
                         target_obj = train_layer$getTargetObj()
                         target_df = target_obj$getData()
                         ind_col = train_layer$getTraining()$getIndCol()
                         if (!(ind_col %in% colnames(data_frame))) {
                           stop("Individual column ID not found in the provided data.frame.\n")
                         }
                         target = train_layer$getTraining()$getTarget()
                         if ((target %in% colnames(data_frame))) {
                           if ("TrainLayer" %in% class(train_layer)) {
                             warning("The target variable must be set using the Target class, so the one in this data.frame will be ignored.\n")
                           }
                           data_frame[ , target] = NULL
                         }
                         # nocov start
                         if (!all(data_frame[ , ind_col] %in% target_df[ , ind_col])) {
                           cat("The following IDs do not exist in target:\n")
                           id_not_in = !(data_frame[ , ind_col] %in% target_df[ , ind_col])
                           cat(data_frame[id_not_in , ind_col], "\n")
                           if (length(id_not_in) == nrow(data_frame)) {
                             stop("None of the provided IDs have a match in the target.\n")
                           } else {
                             if (length(id_not_in) <= nrow(data_frame)) {
                               data_frame = data_frame[!(data_frame[ , ind_col] %in% id_not_in), ]
                               warning("Not matching ID(s) removed:\n")
                             }
                           }
                         }
                         # nocov end
                         if (train_layer$checkTrainDataExist()) {
                           # Remove TrainData if already existing
                           key_class = train_layer$getKeyClass()
                           key = key_class[key_class$class == "TrainData", "key"]
                           train_layer$removeFromHashTable(key = key)
                         }
                         private$train_layer = train_layer
                         missing_id = is.na(data_frame[ , ind_col])
                         if (any(missing_id)) {
                           data_frame = data_frame[!missing_id, ]
                         }
                         data_frame = merge(x = target_df,
                                            y = data_frame,
                                            by = ind_col,
                                            all.y = TRUE)
                         missing_target = is.na(data_frame[ , target])
                         if (any(missing_target)) {
                           data_frame = data_frame[!missing_target, ]
                         }
                         super$initialize(id = id,
                                          ind_col = train_layer$getTraining()$getIndCol(),
                                          data_frame = data_frame)
                         private$target = train_layer$getTraining()$getTarget()
                         # Add the object to ht
                         if ("TrainMetaLayer" %in% class(train_layer)) {
                           if (train_layer$getAccess()) {
                             train_layer$add2HashTable(key = private$id,
                                                       value = self,
                                                       .class = "TrainData")
                           } else {
                             stop("Training data cannot not be added manually to a meta training layer.")
                           }
                         } else {
                           train_layer$add2HashTable(key = private$id,
                                                     value = self,
                                                     .class = "TrainData")
                         }
                         # nocov start
                         if (any(missing_target)) {
                           warning(sprintf("%s individual(s) with missing target value(s) recognized and removed.\n",
                                           sum(missing_target)))
                         }
                         if (any(missing_id)) {
                           warning(sprintf("%s individual(s) with missing ID value(s) recognized and removed.\n",
                                           sum(missing_id)))
                         }
                         # nocov end
                       },
                       #' @description
                       #' Printer
                       #' @param ... `any`
                       #'
                       print = function (...) {
                         if ("TrainMetaLayer" %in% class(private$train_layer)) {
                           cat(sprintf("TrainData : %s\n", "modality-specific prediction data"))
                         } else {
                           cat(sprintf("TrainData : %s\n", private$id))
                         }
                         cat(sprintf("Layer     : %s\n", private$train_layer$getId()))
                         cat(sprintf("ind. id.  : %s\n", private$ind_col))
                         cat(sprintf("target    : %s\n", private$target))
                         cat(sprintf("n         : %s\n", nrow(private$data_frame)))
                         cat(sprintf("Missing   : %s\n", sum(!complete.cases(private$data_frame))))
                         cat(sprintf("p         : %s\n", ncol(private$data_frame) - 2L))
                       },
                       #' @description
                       #' Summary
                       #' @param ... `any`
                       #'
                       summary = function (...) {
                         if ("TrainMetaLayer" %in% class(private$train_layer)) {
                           cat(sprintf("      TrainData : %s\n", "modality-specific predictions"))
                         } else {
                           cat(sprintf("      TrainData : %s\n", private$id))
                         }
                         cat(sprintf("      Layer      : %s\n", private$train_layer$getId()))
                         cat(sprintf("      Ind. id.   : %s\n", private$ind_col))
                         cat(sprintf("      Target     : %s\n", private$target))
                         cat(sprintf("      n          : %s\n", nrow(private$data_frame)))
                         cat(sprintf("      Missing    : %s\n", sum(!complete.cases(private$data_frame))))
                         cat(sprintf("      p          : %s\n", ncol(private$data_frame) - 2L))
                       },
                       #' @description
                       #' Getter of the current \code{data.frame} wihtout individual
                       #' ID nor target variables.
                       #'
                       #' @return
                       #' The \code{data.frame} without individual ID nor target variables is returned.
                       #' @export
                       #'
                       getData = function () {
                         tmp_data <- private$data_frame
                         tmp_data[[private$ind_col]] <- NULL
                         tmp_data[[private$target]] <- NULL
                         return(tmp_data)
                       },
                       #' @description
                       #' Getter of target values stored on the current training layer.
                       #'
                       #' @return
                       #' The observed target values stored on the current training layer are returned.
                       #' @export
                       #'
                       getTargetValues = function () {
                         return(private$data_frame[[private$target]])
                       },
                       #' @description
                       #' Getter of the target variable name.
                       #'
                       #' @export
                       #'
                       getTargetName = function () {
                         return(private$target)
                       },
                       #' @description
                       #' Getter of the current training layer.
                       #'
                       #' @return
                       #' The training layer (from class [TrainLayer]) on which the current train data are stored
                       #' is returned.
                       #' @export
                       #'
                       getTrainLayer = function () {
                         return(private$train_layer)
                       },
                       #' @description
                       #' Getter of the current layer.
                       #'
                       #' @return
                       #' The layer (from class [TestLayer]) on which the current train data are stored
                       #' is returned.
                       #' @export
                       #'
                       #TODO: Maybe rename getTestLayer, getTrainLayer and getPredictLayer as getLayer?
                       getTestLayer = function () {
                         return(private$train_layer)
                       },
                       #' @description
                       #' Set a new \code{data.frame} to the current object.
                       #'
                       #' @param data_frame `data.frame`
                       #'
                       #' @return
                       #' The current object is returned.
                       #'
                       #'
                       setDataFrame = function (data_frame) {
                         private$data_frame = data_frame
                         invisible(self)
                       }
                     ),
                     private = list(
                       target = character(0L),
                       train_layer = NULL
                     ),
                     cloneable = TRUE
)

#' Create COBRA Predictions
#'
#' The \code{createCobraPred} function calculates predictions by averaging the target
#' values of all the nearest candidates in the training dataset. Only the
#' training points that are within the specified proximity (\code{eps}) to the test
#' point are used to determine the prediction. If no suitable training points
#' are found, the function returns \code{NA} as the prediction.
#'
#' @param train A \code{matrix} representing the training data. Rows represent observations,
#' and columns contain predictions from individual learners for these observations.
#' In cases where a prediction is unavailable for a specific observation, \code{NA} is used.
#' @param test A \code{matrix} representing the test data. Rows represent observations,
#' and columns contain predictions from individual learners for these observations.
#' In cases where a prediction is unavailable for a specific observation, \code{NA} is used.
#' @param n_train An \code{integer} specifying the number of training observations.
#' @param n_test An \code{integer} specifying the number of test observations.
#' @param nlearners An \code{integer} representing the number of learners.
#' @param eps A \code{numeric} value representing the threshold for proximity between two predictions.
#' @param alpha A value that determines the optimal number of learners in the neighborhood (only for alpha optimization).
#' @param train_target A \code{vector} containing the target values for the training dataset


createCobraPred = function (train,
                            test,
                            n_train,
                            n_test,
                            nlearners,
                            eps,
                            alpha,
                            train_target) {
  # Initialize prediction vector
  res = numeric(n_test)
  train = unlist(train)
  test = unlist(test)

  # Calculate weights
  weights = createWeights(train, test,
                          n_train, n_test, nlearners,
                          eps, alpha)


  # Normalize weights so that they sum to 1 for each test point
  weights = sweep(weights,1,rowSums(weights),FUN="/")

  for (i in 1:nrow(weights)) {
    if (all(is.nan(weights[i, ]))) {
      # If all values in the row are NaN, set the result to NA
      res[i] = NA
    } else {
      # Replace NaN values in the row with 0
      weights[i, ][is.nan(weights[i, ])] = 0
      # Calculate prediction
      res[i] = weights[i, ] %*% train_target
    }
  }

  return(res)
}

#' @title Abstract class Data
#'
#' @description
#' As abstract, a [Data] object cannot be stored on any layer. Instead, extended
#' [TrainData] or [TestData] objects can be stored on a layer.
#'
#' @export
#'
#' @importFrom R6 R6Class
#' @seealso [TrainData] and [TestData]
Data <- R6Class("Data",
                public = list(
                  #' @description
                  #' Constructor of class Data.
                  #'
                  #' @param id `character` \cr
                  #' Object ID.
                  #' @param ind_col `character` \cr
                  #' Column name containing individual IDs.
                  #' @param data_frame `data.frame` \cr
                  #' `data.frame` containing data.
                  initialize = function (id, ind_col, data_frame) {
                    private$id = id
                    private$ind_col = ind_col
                    if (is.data.frame(data_frame)) {
                      private$data_frame = data_frame
                    } else {
                      stop("'data_frame' must be a data.frame.")
                    }
                  },
                  #' @description
                  #' Printer
                  #' @param ... `any`
                  #'
                  print = function(...) { # nocov start
                    cat("Class     : Data\n")
                    cat(sprintf("name      : %s\n", private$id))
                    cat(sprintf("ind. id.  : %s\n", private$ind_col))
                    cat(sprintf("n         : %s\n", nrow(private$data_frame)))
                    cat(sprintf("p         : %s\n", ncol(private$data_frame)))
                  }, # nocov end
                  #' @description
                  #' Retrieve a data subset for a given variable name and values, a data subset.
                  #'
                  #' @param var_name `character` \cr
                  #' Variable name of interest.
                  #' @param value `vector` \cr
                  #' Values of interest.
                  #'
                  #' @return
                  #' The data subset is returned.
                  #'
                  #' @export
                  #'
                  getIndSubset = function (var_name, value) {
                    subset_data = self$clone(deep = FALSE)
                    index = which(subset_data$getDataFrame()[[var_name]] %in% value)
                    data_frame = subset_data$getDataFrame()[index, ]
                    subset_data$setDataFrame(data_frame = data_frame)
                    return(subset_data)
                  },
                  #' @description
                  #' Imputes missing values in modality-specific predictions.
                  #' Only mode and median based imputations are actually supported.
                  #'
                  #' @param impute_fct `character` \cr
                  #' An imputation function to use instead of median or mode imputation. Not yet implemented!
                  #' @param impute_param `list` \cr
                  #' @param target_name `character` \cr
                  #' Name of the target variable.
                  #' The list of parameters to call the imputation function.
                  #' @return
                  #' A new object with the predicted values is returned.
                  #'
                  impute = function (impute_fct,
                                     impute_param,
                                     target_name) {
                    # nocov start
                    ### print("I am here before....")
                    ### print(private$data_frame)
                    current_data = private$data_frame
                    current_data[ , private$ind_col] = NULL
                    # R is faster when working column wise.
                    name_current_data = names(current_data)
                    if (target_name %in% name_current_data) {
                      target_values = current_data[ , target_name]
                      current_data[ , target_name] = NULL
                    }
                    name_modality_data = names(current_data)
                    current_data = as.data.frame(t(current_data))
                    if (is.null(impute_fct)) {
                      imputed_data = lapply(current_data, function(col_var) {
                        if (is.factor(col_var) | is.character(col_var)) {
                          col_no_na = na.omit(col_var)
                          freq_table = table(col_no_na)
                          mode = names(freq_table[freq_table == max(freq_table)])[1L]
                          col_var[is.na(col_var)] = mode
                          return(col_var)
                        } else {
                          if (is.numeric(col_var)) {
                            col_median = median(col_var, na.rm = TRUE)
                            col_var[is.na(col_var)] = col_median
                            return(col_var)
                          } else {
                            warning("You try mode imputation for a non-categorical variable.")
                          }
                        }
                      })
                      # imputed_data = as.data.frame(t(as.data.frame(imputed_data)))
                      imputed_data = do.call(what = "rbind", args = imputed_data)
                      ### print("I can print colnames of imputed data...")
                      ### print(name_modality_data)
                      private$data_frame[ , name_modality_data] = imputed_data
                      if (target_name %in% name_current_data) {
                        private$data_frame[ , target_name] = target_values
                      }
                    } else {
                      # impute_fct has been set, but actually not implemented.
                      warning("Only mode and median imputations are actually supported.")
                    }
                    invisible(self)
                    # nocov end
                  },
                  #' @description
                  #' Retrieve a subset of variables from data.
                  #'
                  #' @param var_name `character` \cr
                  #' Variable names of interest.
                  #'
                  #' @return
                  #' The data subset is returned.
                  #'
                  #' @export
                  #'
                  getVarSubset = function (var_name) {
                    if (!(private$ind_col %in% var_name)) {
                      var_name = c(private$ind_col, var_name)
                    }
                    subset_data = self$clone(deep = FALSE)
                    data_frame = subset_data$getDataFrame()[ , var_name]
                    subset_data$setDataFrame(data_frame = data_frame)
                    return(subset_data)
                  },
                  #' @description
                  #' For the given variable name, non existing values in the
                  #' current dataset are returned.
                  #'
                  #' @param var_name `character` \cr
                  #' Variable name of interest.
                  #' @param value `vector` \cr
                  #' Values of interest.
                  #'
                  #' @return
                  #' The subset difference is returned.
                  #'
                  #' @export
                  #'
                  getSetDiff = function (var_name, value) {
                    subset_diff <- self$clone(deep = FALSE)
                    index = which(!(value %in% subset_diff$getDataFrame()[[var_name]]))
                    if (length(index)) {
                      return(value[index])
                    } else {
                      return(integer(0L))
                    }
                  },
                  #' @description
                  #' Getter of the \code{data.frame}.
                  #'
                  #' @return
                  #' The \code{data.frame} of the current object is returned.
                  #'
                  #' @export
                  #'
                  getDataFrame = function () {
                    return(private$data_frame)
                  },
                  #' @description
                  #' Set a new \code{data.frame} to the current object.
                  #'
                  #' @param data_frame `data.frame`
                  #'
                  #' @return
                  #' The current object is returned.
                  #'
                  #'
                  setDataFrame = function (data_frame) {
                    private$data_frame = data_frame
                    return(self)
                  },
                  #' @description
                  #' Getter of the complete dataset without missing values.
                  #'
                  #' @return
                  #' The complete dataset is returned.
                  #'
                  #' @export
                  #'
                  getCompleteData = function () {
                    tmp_data = self$getDataFrame()
                    tmp_data = tmp_data[complete.cases(tmp_data), ]
                    return(tmp_data)
                  },
                  #' @description
                  #' Getter of the current object ID.
                  #'
                  #' @return
                  #' The current object ID is returned.
                  #'
                  #' @export
                  #'
                  getId = function () {
                    return(private$id)
                  },
                  #' @description
                  #' Getter of the current Data. This function is re-implemented
                  #' by [TrainData] and [TestData].
                  #'
                  #' @return
                  #' Do not use on this class.
                  #' @export
                  #'
                  getData = function () {
                    stop("Not implemented for this object.")
                  },
                  #' @description
                  #' Getter of the individual column variable.
                  #'
                  #' @export
                  #'
                  getIndCol = function () {
                    return(private$ind_col)
                  }
                ),
                private = list(
                  # Data ID.
                  id = character(0L),
                  # Individual column name.
                  ind_col = NA,
                  # \code{data.frame} containing data.
                  data_frame = NA
                ),
                cloneable = TRUE
)

data("multi_omics")
test_data <- Data$new(id = "geneexpr",
                      ind_col = "IDS",
                      data_frame = multi_omics$training$geneexpr[-11L, ])

test_that("Data: initialize is correct", {
  expect_true(R6::is.R6(test_data))
  expect_equal(class(test_data)[1L], "Data")
  expect_no_error(print(test_data))
})


test_that("Data: error if not data frame", {
  expect_error(
    test_data <- Data$new(id = "geneexpr",
                          ind_col = "IDS",
                          data_frame = matrix(data = 1L:4L, 2L, 2L))
  )
})

test_that("Data: getIndSubset",{
  expect_no_error(test_data$getIndSubset(var_name = "IDS",
                                         value = c("patient67", "patient20")))
})

test_that("Data: getVarSubset",{
  expect_no_error(test_data$getVarSubset(var_name = "IDS"))
  tmp <- test_data$getSetDiff(var_name = "IDS",
                              value = c("patient67"))
  # expect_equal(length(tmp), 0L)
  # tmp <- test_data$getSetDiff(var_name = "IDS",
  #                             value = c("patient678"))
  # expect_equal(length(tmp), 1L)
})

# test_that("Data: getSetDiff",{
#   tmp <- test_data$getSetDiff(var_name = "IDS",
#                               value = c("patient67"))
#   # expect_equal(length(tmp), 0L)
# })

test_that("Data: getDataFrame",{
  expect_no_error(test_data$getDataFrame())
})

test_that("Data: getIndCol",{
  expect_no_error(test_data$getIndCol())
})

test_that("Data: setDataFrame",{
  expect_no_error(test_data$setDataFrame(
    data_frame = test_data$getDataFrame()
  ))
})

test_that("Data: getCompleteData",{
  expect_no_error(test_data$getCompleteData())
})

test_that("Data: getId",{
  expect_no_error(test_data$getId())
})

test_that("Data: getData",{
  expect_error(test_data$getData())
})

#' Cobra Meta Learner
#'
#' The function \code{cobra} implements the COBRA (COmBined Regression Alternative),
#' an aggregation method for combining predictions from multiple individual learners.
#' This method aims to tune key parameters for achieving optimal predictions
#' by averaging the target values of similar candidates in the training dataset's predictions.
#' Only the training points that are sufficiently similar to the test point
#' (based on the proximity threshold \code{epsilon}) are used for prediction.
#' If no suitable training points are found, the function returns \code{NA}.
#'
#' @param x `data.frame` \cr
#' A training data, where rows are observations and
#' columns are predictions from individual learners. Use \code{NA} for missing predictions.
#' @param y `vector` \cr
#' A vector containing the training targets. This can be a binary or two-level factor variable.
#' @param tune `character` \cr
#' A character value specifying the tuning mode:
#' - \code{"alpha_epsilon"}: Tunes both \code{alpha} (number of learners) and \code{epsilon} (proximity threshold) via cross-validation.
#' - \code{"epsilon"}: Tunes \code{epsilon} only via cross-validation.
#' - \code{"user"}: No tuning; the user provides an optimal \code{epsilon}.
#' #' The default value is \code{epsilon}.
#' @param eps `numeric` \cr
#' A numeric value for the proximity threshold, used only when \code{tune = "user"}.
#' Defaults to \code{0.1}.
#' @param k_folds `integer` \cr
#' Number of folds for cross-validation when \code{tune = "alpha_epsilon"} or \code{"epsilon"}.
#' Default is \code{10}.
#' @return An object of class \code{cobra} containing the training data, target values, and chosen parameters.
#' @references
#' Biau, G., Fischer, A., Guedj, B., & Malley, J. D. (2014). COBRA: A combined regression strategy. The Journal of Multivariate Analysis 46:18-28
#' @examples
#' # Example usage
#' set.seed(123)
#' x_train <- data.frame(a = runif(10L), b = runif(10L))
#' y_train <- sample(0L:1L, size = 10L, replace = TRUE)
#'
#' # Train the model with epsilon optimization
#' cobra_model <- cobra(x = x_train, y = y_train, tune = "epsilon", k_folds = 2)
#'
#' # Make predictions on new data
#' set.seed(156)
#' x_new <- data.frame(a = runif(5L), b = runif(5L))
#' prediction <- predict(object = cobra_model, data = x_new)
#'
#' @export
#'
#'
cobra = function(x, y, tune = "epsilon", k_folds = NULL, eps = NULL){

  # Validate input parameters
  checkmate::assert_data_frame(x)
  checkmate::assert_data_frame(x = x, min.rows = 2, min.cols = 1)
  checkmate::assert_vector(x = y, any.missing = FALSE)

  # Ensure the tune parameter is valid
  if (!tune %in% c("alpha_epsilon", "epsilon", "user")) {
    stop("Invalid 'tune' value. Use 'alpha_epsilon', 'epsilon', or 'user'.")
  }

  # Check the eps parameter: only allowed if tune == "user"
  if (!is.null(eps) && tune != "user") {
    stop("The 'eps' parameter can only be specified when 'tune' is set to 'user'.")
  }

  # Check the k_folds parameter: only allowed if tune == "epsilon" or "alpha_epsilon"
  if (!is.null(k_folds) && !tune %in% c("epsilon", "alpha_epsilon")) {
    stop("The 'k_folds' parameter can only be specified when 'tune' is 'epsilon' or 'alpha_epsilon'.")
  }

  # Handle parameter requirements based on the chosen tuning mode
  if(tune== "user"){
    # User-defined epsilon: eps must be provided or defaults to 0.1
    if(is.null(eps)){
      eps = 0.1
    } else if (!is.numeric(eps) || eps <= 0) {
        stop("Parameter 'eps' must be a positive numeric value.")
    }
    k_folds = NULL # No cross-validation required for user-defined epsilon

  } else if (tune %in% c("epsilon", "alpha_epsilon")) {
    # Cross-validation mode: k_folds must be specified or defaults to 10
    if (is.null(k_folds)) {
      k_folds = 10
    } else if (!is.numeric(k_folds) || k_folds <= 1) {
      stop("Parameter 'k_folds' must be a numeric value greater than 1.")
    }
    # eps will be determined during cross-validation
    eps = NULL
  }


  # Ensure the target variable y is binary and numeric
  # numeric
  if (is.numeric(y) & (length(unique(y)) > 2)) {
    y = y
  } else {
    if ((length(unique(y)) > 2) | is.character(y)) {
      stop("y must be either binary or two level factor variable.\n")
    } else {
      if (is.factor(y)) {
        y = as.integer(y) - 1
      } else {
        if (!all(y %in% 0:1)) {
          stop("y must take its values between 0 and 1.\n")
        }
      }
    }
  }

      # Convert factor or non-binary y to binary values
      # if (!all(y %in% 0:1)) {
      #   y = as.integer(y) -1
      #   } else {
      #     if (is.factor(y)) {
      #       y = as.integer(y) - 1
      #       } else {
      #         y = y
      #       }
      #     }

      # Create the COBRA model object
      cobra_object = list(train = x,
                           train_target = y,
                           tune = tune,
                           k_folds = k_folds,
                           eps = eps)

      class(cobra_object) = "cobra"

      # Provide a message about the chosen tuning mode
      if (tune == "alpha_epsilon") {
        message("Tuning 'alpha' and 'epsilon' via cross-validation with ", k_folds, " folds.")
      } else if (tune == "epsilon") {
        message("Tuning 'epsilon' via cross-validation with ", k_folds, " folds.")
      } else if (tune == "user") {
        message("Using user-defined 'epsilon' = ", eps, ".")
      }


  return(cobra_object)
}

#' @title Weighted mean prediction.
#'
#' @description
#' Predict function for models from class \code{weightedMeanLearner}.
#'
#' @include weightedMeanLearner.R
#'
#' @param object `weightedMeanLearner(1)` \cr
#' An object from class [weightedMeanLearner]
#' @param data `data.frame` \cr
#' \code{data.frame} to be predicted.
#' @param na_rm `boolean` \cr
#' Removes NAs when TRUE.
#' @param ... `any` \cr
#' Further arguments.
#'
#' @return
#' Predicted target values are returned.
#'
#' @export
#' @method predict weightedMeanLearner
#'
#' @importFrom stats weighted.mean complete.cases
#'
#' @examples
#' set.seed(20240625)
#' x <- data.frame(x1 = rnorm(50L))
#' y <- sample(x = 0:1, size = 50L, replace = TRUE)
#' my_model <- weightedMeanLearner(x = x, y = y)
#' x_new <- data.frame(x1 = rnorm(10L))
#' my_predictions <- predict(object = my_model, data = x_new)
#'
predict.weightedMeanLearner = function (object, data, na_rm = FALSE, ...) {
  if (all(names(object) %in% names(data))) {
    pred = apply(data[ , names(object), drop = FALSE], 1L, function (tmp_row) {
      return(weighted.mean(x = tmp_row, w = object, na.rm = na_rm))
    })
    return(list(predictions = pred))
  } else {
    stop("Names of weights do not match with name columns in data")
  }
}

#' @title Varsel Class
#'
#' @description
#' This class implements a learner. A [VarSel] object can only exist as a component of a
#' [TrainLayer] or a [TrainMetaLayer] object.
#'
#' @export
#'
#' @importFrom R6 R6Class
VarSel <- R6Class("VarSel",
                  public = list(
                    #' @description
                    #' Variable selection parameter list.
                    #'
                    #'
                    #' Learner ID.
                    #' @param id `character` \cr
                    #' Package that implements the variable selection function.
                    #'  If NULL, the variable selection  function is called from
                    #'   the current environment.
                    #' @param package `character` \cr
                    #' Variable selection function name. Note: Variable selection functions, except \code{Boruta}, must return a vector of selected variables.
                    #' @param varsel_fct `character` \cr
                    #' Variable selection parameters.
                    #' @param varsel_param `list` \cr
                    #' Layer on which the learner is stored.
                    #' @param train_layer `TrainLayer` \cr
                    #'  The training layer where to store the learner.
                    #' @param na_action `character` \cr
                    #' Handling of missing values in meta-data. Set to "na.keep" to keep missing values, "na.rm" to remove individuals with missing values or "na.impute" (only applicable on meta-data) to impute missing values in meta-data. Only median and mode based imputations are actually handled. With the "na.keep" option, ensure that the provided learner can handle missing values.
                    #' If \code{TRUE}, the individuals with missing predictor values will be removed from the training dataset.
                    initialize = function (id,
                                           package = NULL,
                                           varsel_fct,
                                           varsel_param,
                                           train_layer,
                                           na_action = "na.rm") {
                      private$id = id
                      private$package = package
                      private$varsel_fct = varsel_fct
                      private$param = varsel_param
                      if (is.null(package)) {
                        if (!(exists(varsel_fct, envir = .GlobalEnv, inherits = TRUE) | is.function(get(varsel_fct, envir = .GlobalEnv)))) {
                          stop(sprintf("Function %s does not exists.\n Maybe you forget to specify its package?", varsel_fct))
                        }
                      }
                      if (!any(c("TrainLayer") %in% class(train_layer))) {
                        stop("A variable selection tool can only belong to object of class TrainLayer.")
                      }
                      # if (!is.logical(na_rm)) {
                      #   stop("na.rm must be a logical value\n")
                      # } else {
                      #   private$na_rm = na_rm
                      # }

                      if (na_action == "na.keep") {
                        # nocov start
                        na_rm = FALSE
                        #nocov end
                      } else {
                        if (na_action == "na.rm") {
                          na_rm = TRUE
                        } else {
                          # nocov start
                          if (na_action == "na.impute") {
                            stop("Imputation is not yet handled for data modalities. Please use either the 'na.keep' or the 'na.rm' option.")
                          } else {
                            stop("na_action must be one of 'na.fails' or 'na.rm'.")
                          }
                          # nocov end
                        }
                      }
                      private$na_rm = na_rm
                      # Remove VarSel if already existing
                      if (train_layer$checkVarSelExist()) {
                        key_class = train_layer$getKeyClass()
                        key = key_class[key_class$class == "VarSel", "key"]
                        train_layer$removeFromHashTable(key = key)
                      }
                      private$train_layer = train_layer
                      # Add object to ht
                      train_layer$add2HashTable(key = private$id,
                                                value = self,
                                                .class = "VarSel")
                    },
                    #' @description
                    #' Printer
                    #' @param ... `any`
                    #'
                    print = function (...) {
                      cat(sprintf("VarSel           : %s\n", private$id))
                      cat(sprintf("TrainLayer       : %s\n", private$train_layer$getId()))
                      cat(sprintf("Package          : %s\n", private$package))
                      cat(sprintf("Function         : %s\n", private$varsel_fct))
                    },
                    #' @description
                    #' Summary
                    #' @param ... `any`
                    #'
                    summary = function (...) {
                      cat(sprintf("      VarSel           : %s\n", private$id))
                      cat(sprintf("      TrainLayer       : %s\n", private$train_layer$getId()))
                      cat(sprintf("      Package          : %s\n", private$package))
                      cat(sprintf("      Function         : %s\n", private$varsel_fct))
                    },
                    #' @description
                    #' Learner and prediction parameter interface. Use this function
                    #' to provide how the following parameters are named in the learning
                    #' function (\code{lrn_fct}) you provided when creating the learner, or in the predicting function.
                    #'
                    #' @param x `string` \cr
                    #' Name of the argument to pass the matrix of independent variables in the original learning function.
                    #' @param y `string` \cr
                    #' Name of the argument to pass the response variable in the original learning function.
                    #' @param object `string` \cr
                    #' Name of the argument to pass the model in the original predicting function.
                    #' @param data `character` \cr
                    #' Name of the argument to pass new data in the original predicting function.
                    #' @param extract_var_fct `character` or `function` \cr
                    #' If the variable selection function that is called does not return a vector, then
                    #' use this argument to specify a (or a name of a) function that can be used to extract vector of selected variables.
                    #' Default value is NULL, if selected variables are in a vector.
                    #' @export
                    #'
                    # TODO: Convr me
                    # nocov start
                    interface = function (x = "x",
                                          y = "y",
                                          object = "object",
                                          data = "data",
                                          extract_var_fct = NULL) {
                      if (!is.character(x)) {
                        stop("String expected for x.")
                      }
                      if (!is.character(y)) {
                        stop("String expected for y.")
                      }
                      if (!is.character(object)) {
                        stop("String expected for object.")
                      }
                      if (!is.character(data)) {
                        stop("String expected for data.")
                      }
                      if (!is.character(extract_var_fct) & !is.function(extract_var_fct) & !is.null(extract_var_fct)) {
                        stop("String or function expected for extract_var_fct.")
                      } else {
                        if (!is.null(extract_var_fct)) {
                          if (length(formals(extract_var_fct)) > 1L) {
                            stop("Only one argument expected for the function specified in extract_var_fct.")
                          }
                        }
                      }
                      param_interface = data.frame(standard = c("x_name", "y_name", "object_name", "data_name"),
                                                   original = c(x, y, object, data))
                      private$param_interface = param_interface
                      private$extract_var_fct = extract_var_fct
                    },
                    # nocov end
                    #' @description
                    #' Tains the current learner (from class [Lrner]) on the current training data (from class [TrainData]).
                    #'
                    #' @param ind_subset `vector` \cr
                    #' Individual ID subset on which the training will be performed.
                    #'
                    #' @return
                    #' The resulting model, from class [Model], is returned.
                    #' @export
                    #'
                    varSelection = function (ind_subset = NULL) {
                      train_data = private$train_layer$getTrainData()
                      # Variable selection only on complete data
                      train_data = train_data$clone(deep = FALSE)
                      if (private$na_rm) {
                        complete_data = train_data$getCompleteData()
                        train_data$setDataFrame(data_frame = complete_data)
                      }
                      if (is.null(private$package)) {
                        # nocov start
                        varsel = private$varsel_fct
                        # nocov end
                      } else {
                        varsel = sprintf('%s::%s', private$package,
                                         private$varsel_fct)
                      }
                      varsel_param = private$param
                      # Prepare training dataset
                      if (!is.null(ind_subset)) {
                        train_data = train_data$getIndSubset(
                          var_name = train_data$getIndCol(),
                          value = ind_subset)
                        private$ind_subset = ind_subset
                      } else {
                        private$ind_subset = "ALL"
                      }
                      # varsel_param$x = train_data$getData()
                      # varsel_param$y = train_data$getTargetValues()
                      # Set x and y parameters.
                      if (is.null(private$param_interface)) {
                        varsel_param$x = train_data$getData()
                        varsel_param$y = train_data$getTargetValues()
                      } else {
                        # TODO: covr me
                        # nocov start
                        x_name = private$param_interface[private$param_interface$standard == "x_name", "original"]
                        y_name = private$param_interface[private$param_interface$standard == "y_name", "original"]
                        varsel_param[[x_name]] = train_data$getData()
                        varsel_param[[y_name]] = train_data$getTargetValues()
                        # nocov end
                      }
                      varselected = do.call(eval(parse(text = varsel)),
                                            varsel_param)
                      # Only confirmed variables are remained
                      if (!is.null(private$package)) {
                        if ((private$package == "Boruta")) {
                          tmp_param = list(x = varselected, withTentative = FALSE)
                          get_varsel = sprintf('%s::getSelectedAttributes',
                                               private$package)
                          # Get selected variables as vector
                          varselected = do.call(eval(parse(text = get_varsel)),
                                                tmp_param)
                        } else {
                          # Extract selected variables using function provided by user.
                          # TODO: cover me
                          # nocov start
                          extract_var_fct = self$getExtractVar()
                          if (!is.null(extract_var_fct)) {
                            if (is.character(extract_var_fct)) {
                              extract_var_fct = eval(parse(text = extract_var_fct))
                            }
                            param_extract = list()
                            param_extract[[names(formals(extract_var_fct))]] = varselected
                            varselected = do.call(what = extract_var_fct,
                                                  args = param_extract)
                          }
                          # nocov end
                        }
                      } else {
                        # Systematic test is challenging for external variable
                        # Have been test in interactive session
                        # nocov start
                        if (!is.vector(varselected)) {
                          stop("Variable selection function should return a vector of selected variables.")
                        }
                        # nocov end
                      }
                      private$ind_subset = ind_subset
                      if (!length(varselected)) {
                        # Systematic test is challenging for external variable
                        # Have been test in interactive session
                        # nocov start
                        warning(sprintf("No variable selected on layer %s", private$train_layer$getId()))
                        private$var_subset = NULL
                        # nocov end
                      } else {
                        private$var_subset = varselected
                        train_data_frame = train_data$getDataFrame()
                        train_data_frame = train_data_frame[ , c(train_data$getIndCol(), varselected, train_data$getTargetName())]
                        private$train_layer$getTrainData()$setDataFrame(data_frame = train_data_frame)
                      }
                      return(varselected)
                    },
                    #' @description
                    #' The current layer is returned.
                    #'
                    #' @return
                    #' [TrainLayer] object.
                    #' @export
                    #'
                    getTrainLayer = function () {
                      return(private$train_layer)
                    },
                    #' @description
                    #' Getter of the current learner ID.
                    #'
                    #' @return
                    #' The current learner ID.
                    #' @export
                    #'
                    getId = function () {
                      return(private$id)
                    },
                    #' @description
                    #' Getter of the variable selection package implementing the variable selection function.
                    #'
                    #' @return
                    #' The name of the package implementing the variable selection function.
                    #'
                    #' @export
                    #'
                    getPackage = function () {
                      return(private$package)
                    },
                    #' @description
                    #' Getter of the list of selected variables.
                    #'
                    #' @return
                    #' List of selected variables..
                    #'
                    #' @export
                    #'
                    getVarSubSet = function () {
                      return(private$var_subset)
                    },
                    #' @description
                    #' The current parameter interface is returned.
                    #'
                    #' @return
                    #' A data.frame of interface.
                    #'
                    getParamInterface = function () {
                      return(private$param_interface)
                    },
                    #' @description
                    #' The current layer is returned.
                    getNaAction = function () {
                      return(private$na_action)
                    },
                    #' @description
                    #' The function to extract selected variables is returned.
                    #'
                    #' @return
                    #' A data.frame of interface.
                    #'
                    # nocov start
                    getExtractVar = function () {
                      return(private$extract_var_fct)
                    }
                    # nocov end
                  ),
                  private = list(
                    # ID field.
                    id = character(0L),
                    # Package defining the learner (like \code{ranger}).
                    package = NULL,
                    # Learn function name (like \code{ranger}).
                    varsel_fct = NULL,
                    # Parameters of the variable selection function.
                    param = NULL,
                    # Parameter interface to original names of arguments in original learning and predict function.
                    param_interface = NULL,
                    na_rm = NULL,
                    na_action = NULL,
                    # Training layer (from class [TainLayer] or [TrainMetaLayer]) of the current learner.
                    train_layer = NULL,
                    # Individuals subset IDs.
                    ind_subset = NULL,
                    # Variable subset IDs.
                    var_subset = NULL,
                    # Function to extract selected variables.
                    extract_var_fct = NULL
                  ),
                  cloneable = FALSE
)

test_that("VarSel: with package for variable selection", {
  training <- Training$new(id = "training",
                                ind_col = "IDS",
                                target = "disease",
                                target_df = multi_omics$training$target)
  tl_ge <- TrainLayer$new(id = "geneexpr", training = training)
  train_data_ge <- TrainData$new(id = "geneexpr",
                                 train_layer = tl_ge,
                                 data_frame = multi_omics$training$geneexpr)
  expect_error({
    var_sel <- VarSel$new(id = "varsel_geneexpr",
                          package = "Boruta",
                          varsel_fct = "Boruta",
                          varsel_param = list(num.trees = 50L,
                                              mtry = 3L),
                          train_layer = "not_a_Train_layer")
  })
  expect_no_error({
    varsel_ge <- VarSel$new(id = "varsel_geneexpr",
                            package = "Boruta",
                            varsel_fct = "Boruta",
                            varsel_param = list(num.trees = 50L,
                                                mtry = 3L,
                                                getImp = Boruta::getImpRfZ),
                            train_layer = tl_ge)
    print(varsel_ge)
  })
  expect_no_error({
    varsel_ge$getTrainLayer()
  })
  expect_no_error({
    varsel_ge$getPackage()
  })
  # Update
  expect_no_error({
    varsel_ge <- VarSel$new(id = "varsel_geneexpr",
                            package = "Boruta",
                            varsel_fct = "Boruta",
                            varsel_param = list(num.trees = 50L,
                                                mtry = 3L,
                                                getImp = Boruta::getImpRfZ),
                            train_layer = tl_ge)
  })
  expect_no_error({
    skip_on_cran()
    varsel_ge$varSelection(ind_subset = multi_omics$training$geneexpr$IDS[1:45])
    varsel_ge$getId()
  })
})

test_that("VarSel: with function for variable selection", {
  training <- Training$new(id = "training",
                                ind_col = "IDS",
                                target = "disease",
                                target_df = multi_omics$training$target)
  tl_ge <- TrainLayer$new(id = "geneexpr", training = training)
  train_data_ge <- TrainData$new(id = "geneexpr",
                                 train_layer = tl_ge,
                                 data_frame = multi_omics$training$geneexpr)

  expect_error({
    #TODO: check why it only works in interactiv mode
    test_var_sel <-  function (x, y) {return(matrix(1:4, 2L, 2L))}
    var_sel <- VarSel$new(id = "varsel_geneexpr",
                          varsel_fct = "test_var_sel",
                          varsel_param = list(),
                          train_layer = tl_ge)
    tl_ge$varSelection()
  })
})

#' @title createTesting
#' @description
#' Creates a [Testing] object.
#'
#' @param id `character` \cr
#' Testing id.
#' @param ind_col `character` \cr
#' Name of column of individuals IDs in testing `data.frame`.
#' @param verbose `boolean` \cr
#' Warning messages will be displayed if set to `TRUE`.
#' @return
#' A [Testing] object.
#' @export
createTesting = function (id,
                          ind_col,
                          verbose = TRUE) {
  testing = Testing$new(
    id = id,
    ind_col = ind_col,
    verbose = verbose
  )
  return(testing)
}


#' @title createTestLayer
#' @description
#' Creates and stores a [TestLayer] on the [Testing] object passed as argument.
#'
#' @param testing `Testing` \cr
#' Testing object where the created layer will be stored.
#' @param test_layer_id `character` \cr
#' ID of the testing layer to be created.
#' @param test_data `data.frame` \cr
#' Data modality to be stored in [TestData].
#'
#' @return
#' The updated [Testing] object (with the new layer) is returned.
#' @export
#'
createTestLayer = function (testing,
                            test_layer_id,
                            test_data) {
  # Instantiate a layer
  test_layer = TestLayer$new(id = test_layer_id,
                             testing = testing)
  # Instantiate a TestData
  test_data = TestData$new(id = sprintf("%s_data", test_layer_id),
                           data_frame = test_data,
                           new_layer = test_layer)
  if (testing$getVerbose()) {
    return(testing)
  } else {
    invisible(TRUE)
  }
}

#' @title Testing object Summaries
#' @description
#' Summaries a `fuseMLR` [Testing] object.
#'
#' @param object `Testing` \cr
#' The [Testing] object of interest.
#' @param ... `any` \cr
#' Further arguments.
#'
#' @export
#' @method summary Testing
summary.Testing = function (object, ...) {
  return(object$summary())
}

#' Predict Using COBRA object
#'
#' #' The `predict.cobra` function makes predictions on new data using a trained COBRA object.
#'
#' @param object An object of class "cobra" created by the \code{cobra} function.
#' @param data A \code{data.frame} of new data, where rows are observations and
#' columns are predictions from individual learners. Use \code{NA} for missing predictions.
#' @param ... Additional arguments (currently not used).
#' @return A vector of predictions for the new data.
#' @examples
#' # Example usage
#' set.seed(123)
#' x_train <- data.frame(a = rnorm(10L), b = rnorm(10L))
#' y_train <- sample(0L:1L, size = 10L, replace = TRUE)
#'
#' # Train the model with epsilon optimization
#' cobra_model <- cobra(x = x_train, y = y_train, tune = "epsilon")
#'
#' # Make predictions on new data
#' set.seed(156)
#' x_new <- data.frame(a = rnorm(5L), b = rnorm(5L))
#' prediction <- predict(object = cobra_model, data = x_new)
#' @export
#' @method predict cobra

predict.cobra = function(object, data, ...){
  # Ensure the object is of class 'cobra'
  if (!inherits(object, "cobra")) {
    stop("The provided object is not of class 'cobra'.")
  }
  # Check if column names in the training data match those in the test data
  if (!all(names(object$train) %in% names(data))) {
    stop("Names of layers in the training data do not match the names of layers in the new data.")
  }

  # Extract parameters from the cobra object
  train_data = as.matrix(object$train)
  n_train =  nrow(train_data)
  train_target = object$train_target
  test_data = as.matrix(data)
  n_test = nrow(data)
  n_learners = ncol(train_data)

  #  Input validation

  if (n_test <= 0) {
    stop("The test data must contain at least one observation.")
  }

  if (object$tune %in% c("alpha_epsilon", "epsilon")) {
    # nocov start
    # Use cross-validation to optimize epsilon (and optionally alpha)


    # Check if target is binary
    is_binary_target = all(train_target %in% c(0, 1))

    if (is_binary_target) {
      # Binary target: Adjust epsilon range for probabilities
      emin = max(min(diff(sort(as.vector(train_data))),
                      diff(sort(as.vector(test_data)))),
                  1e-300) # Minimum epsilon
      emax = 1.0 # Maximum epsilon for probabilities
      calibr_grid = 100 # Use a grid for binary probabilities

    } else {
      # Continuous target: Use the original calculation
      emin = max(min(diff(sort(as.vector(train_data))),
                      diff(sort(as.vector(test_data)))),
                  1e-300) # Minimum epsilon
      emax = 2 * max(createDif(train_data),
                      createDif(test_data)) # Maximum epsilon
      calibr_grid = 200 # Grid
    }

    estep = (emax - emin)/(calibr_grid-1)
    # Epsilon vector
    evect = seq(emin, emax, estep)
    n_e = length(evect)

    # nocov end

    # Perform cross-validation
    k_folds = object$k_folds
    folds = caret::createFolds(train_target, k = k_folds, list = TRUE)
    if (object$tune == "alpha_epsilon") {
      # Optimize both alpha and epsilon
      alphaseq = seq(1, n_learners)/n_learners # Range of alpha values

      loss_cv = array(0, dim = c(n_e, n_learners, k_folds)) # Loss matrix for cross-validation
      epsoptvec = array(0, dim = c(n_learners, k_folds))

      # cross validation for parameter optimization
      for (fold_idx in seq_along(folds)) {
        # Split data into training and test folds
        test_idx = folds[[fold_idx]]
        train_idx = setdiff(seq_along(train_target), test_idx)

        train_target_fold = train_target[train_idx]
        test_target_fold = train_target[test_idx]
        train_pred_fold = train_data[train_idx, , drop = FALSE]
        test_pred_fold = train_data[test_idx, , drop = FALSE]

        for(a in alphaseq){
          alpha = a
          ilearner = a*n_learners
          pred = sapply(X = evect, FUN = function(e) createCobraPred(eps = e,
                                                                        train = train_pred_fold,
                                                                        test = test_pred_fold,
                                                                        n_train = length(train_idx),
                                                                        n_test = length(test_idx),
                                                                        nlearners = n_learners,
                                                                        alpha = alpha,
                                                                        train_target = train_target_fold))
          # loss
          if (is.vector(pred)) { # if one observation in test_pred_fold
            loss_cv[, ilearner, fold_idx] = sapply(1:length(pred), function(i) createLoss(pred[i],
                                                                                  target = test_target_fold))
          } else {
            loss_cv[, ilearner, fold_idx] = apply(X = pred, MARGIN = 2, FUN = createLoss,
                                                   target = test_target_fold)
          }

          epsoptvec[ilearner, fold_idx] = evect[which.min(loss_cv[, ilearner, fold_idx])]

        }
      }

      # Calculate mean loss and find optimal alpha and epsilon
      loss_avg = apply(loss_cv, c(1, 2), mean)
      epsoptvec_avg = rowMeans(epsoptvec)

      alphaopt_idx = which.min(loss_avg)%/%n_e+1
      alpha_opt = alphaseq[alphaopt_idx]
      eps_opt = epsoptvec_avg[alphaopt_idx]

      message('Optimal alpha: ', round(alpha_opt, 3) , ' (', alpha_opt * n_learners, ' Learner(s))',
              '. Optimal epsilon: ', round(eps_opt, 3), ". Tuning with ", k_folds, " folds.")

      # final predictions
      pred = createCobraPred(eps = eps_opt,
                                train = train_data,
                                test = test_data,
                                n_train = n_train,
                                n_test = n_test,
                                nlearners = n_learners,
                                alpha = alpha_opt,
                                train_target = train_target)


    } else { # If tuning epsilon only
      loss_cv = array(0, dim = c(n_e, k_folds))

      # cross validation for parameter optimization
      for (fold_idx in seq_along(folds)) {
        # Split data into training and test folds
        test_idx = folds[[fold_idx]]
        train_idx = setdiff(seq_along(train_target), test_idx)

        train_target_fold = train_target[train_idx]
        test_target_fold = train_target[test_idx]
        train_pred_fold = train_data[train_idx, , drop = FALSE]
        test_pred_fold = train_data[test_idx, , drop = FALSE]

        pred = sapply(X = evect, FUN = function(e) createCobraPred(eps = e,
                                                                      train = train_pred_fold,
                                                                      test = test_pred_fold,
                                                                      n_train = length(train_idx),
                                                                      n_test = length(test_idx),
                                                                      nlearners = n_learners,
                                                                      alpha = NULL,
                                                                      train_target = train_target_fold))
        # loss
        # loss_cv[, fold_idx] = apply(X = pred, MARGIN = 2, FUN = createLoss,
        #                              target = test_target_fold)

        if (is.vector(pred)) { # if one observation in test_pred_fold
          loss_cv[, fold_idx] = sapply(1:length(pred), function(i) createLoss(pred[i],
                                                                                target = test_target_fold))
        } else {
          loss_cv[, fold_idx] = apply(X = pred, MARGIN = 2, FUN = createLoss,
                                       target = test_target_fold)
        }
      }

      # Calculate mean loss and find optimal epsilon
      loss_avg = apply(loss_cv, 1, mean)
      eps_opt = evect[which.min(loss_avg)]

      message('Optimal epsilon: ', round(eps_opt, 3), ". Tuning with ", k_folds, " folds.")

      # final predictions
      pred = createCobraPred(eps = eps_opt,
                                train = train_data,
                                test = test_data,
                                n_train = n_train,
                                n_test = n_test,
                                nlearners = n_learners,
                                alpha = NULL,
                                train_target = train_target)

    }
  } else  { # If tune = "user"
    eps = object$eps

    # final predictions
    pred = createCobraPred(eps = eps,
                              train = train_data,
                              test = test_data,
                              n_train = n_train,
                              n_test = n_test,
                              nlearners = n_learners,
                              alpha = NULL,
                              train_target = train_target)

  }

  predictions = as.vector(pred)
  return(predictions)
}

test_that("predict.cobra works", {

  # Dummy data for testing##
  object <- list(
    train = matrix(runif(100), nrow = 10, ncol = 10),
    train_target = runif(10),
    k_folds = NULL,
    tune = NULL,
    eps = 0.3
  )
  # Test data
  data <- matrix(runif(30), nrow = 3, ncol = 10)

  # Test 1: Check if an error is thrown when the object is not of class 'cobra'
  expect_error(predict.cobra(list(train = matrix(1), train_target = c(1),
                       k_folds = 1, tune = "epsilon"), data))


})

#' @title PredictData Class
#'
#' @description
#' This class implements [PredictData] object to be predicted.
#' A [PredictData] object can only exist as a component of a [PredictLayer] or a [PredictMetaLayer] object.
#' @export
#' @importFrom R6 R6Class
#' @seealso [TrainData], [TestData]
PredictData <- R6Class("PredictData",
                       inherit = Data,
                       public = list(
                         #' @description
                         #' Initialize a new object from the current class.
                         #'
                         #' @param id `character` \cr
                         #' Object ID.
                         #' @param ind_col `character`\cr
                         #' Column name containing individual IDs.
                         #' @param data_frame `data.frame`\cr
                         #' \code{data.frame} containing data.
                         initialize = function (id,
                                                ind_col,
                                                data_frame) {
                           super$initialize(id = id,
                                            ind_col = ind_col,
                                            data_frame = data_frame)
                         },
                         #' @description
                         #' Printer
                         #' @param ... `any`
                         #'
                         print = function (...) {
                           cat("Class     : PredictData\n")
                           cat(sprintf("Layer     : %s\n", private$predict_layer$id))
                           cat(sprintf("name      : %s\n", private$id))
                           cat(sprintf("ind. id.  : %s\n", private$ind_col))
                           cat(sprintf("n         : %s\n", nrow(private$data_frame)))
                           cat(sprintf("p         : %s\n", ncol(private$data_frame)))
                         },
                         #' @description
                         #' Getter of the current predicted \code{data.frame} wihtout individual
                         #' ID variable.
                         #'
                         #' @return
                         #' The \code{data.frame} without individual ID nor target variables is returned.
                         #' @export
                         #'
                         getPredictData = function () {
                           return(private$data_frame)
                         },
                         #' @description
                         #' Getter of the current layer.
                         #'
                         #' @return
                         #' The layer (from class [PredictLayer]) on which the current train data are stored
                         #' is returned.
                         #' @export
                         #'
                         getPredictLayer = function () {
                           return(private$predict_layer)
                         },
                         #' @description
                         #' Assigns a predicted layer to the predicted data.
                         #'
                         #' @param predict_layer `PredictLayer(1)` \cr
                         #'
                         #' @return
                         #' The current object
                         #'
                         setPredictLayer = function (predict_layer) {
                           private$predict_layer = predict_layer
                           predict_layer$add2HashTable(key = private$id,
                                                       value = self,
                                                       .class = "PredictData")
                           return(self)
                         }
                       ),
                       private = list(
                         # Current predicted layer.
                         predict_layer = NULL
                       ),
                       cloneable = TRUE
)

#' @title PredictLayer Class
#'
#' @description
#' This class implements a layer. A [PredictLayer] object can only exist as a component of a [Predicting] object.
#'
#' A predicted layer can only contain [PredictData].
#'
#'
#' @export
#' @importFrom R6 R6Class
#' @seealso [Training], [Lrner], [TrainData], [TestData] and [Model]
PredictLayer <- R6Class("PredictLayer",
                        inherit = HashTable,
                        public = list(
                          #' @description
                          #' constructor
                          #'
                          #' @param id `character` \cr
                          #' The layer ID.
                          initialize = function (id) {
                            super$initialize(id = id)
                          },
                          #' @description
                          #' Printer
                          #' @param ... `any`
                          #'
                          print = function (...){
                            cat(sprintf("PredictLayer    : %s\n", private$id))
                            cat(sprintf("Contains %s object.\n", length(private$hash_table)))
                          },
                          #' @description
                          #' Getter of the current predicting object
                          #'
                          #' @return
                          #' The current predicting object is returned.
                          #'
                          getPredicting = function () {
                            return(private$predicting)
                          },
                          #' @description
                          #' Getter of IDS from the current layer.
                          #'
                          #' @return
                          #' A \code{data.frame} containing individuals IDs values.
                          #' @export
                          #'
                          getIndIDs = function () {
                            layer_kc = self$getKeyClass()
                            # Stop if training data is missing on this layer.
                            if (("PredictData" %in% layer_kc[ , "class"])) {
                              # Searching for layer specific new dataset
                              data_key = layer_kc[layer_kc$class == "PredictData" ,
                                                  "key"]
                              current_data = self$getPredictData()
                            } else {
                              stop(sprintf("No data on layer %s.", self$getId()))
                            }
                            current_data_frame = current_data$getDataFrame()
                            ids_data = current_data_frame[ , current_data$getIndCol(), drop = FALSE]
                            return(ids_data)
                          },
                          #' @description
                          #' Getter of the predicted data stored on the current layer.
                          #'
                          #' @return
                          #' The stored [PredictData] object is returned.
                          #' @export
                          #'
                          getPredictData = function () {
                            layer_kc = self$getKeyClass()
                            if ("PredictData" %in% layer_kc[ , "class"]) {
                              predict_data_key = layer_kc[layer_kc$class == "PredictData" ,
                                                          "key"]
                              predict_data = self$getFromHashTable(key = predict_data_key[1L])
                            } else {
                              stop(sprintf("No predicted data on layer %s.", self$getId()))
                            }
                            return(predict_data)
                          },
                          #' @description
                          #' Assigns a predicting object to the predicted layer.
                          #'
                          #' @param predicting `Predicting` \cr
                          #'
                          #' @return
                          #' The current object
                          #'
                          setPredicting = function (predicting) {
                            if (!is.null(private$predicting)) {
                              stop(sprintf("This layer already belong to ",
                                           private$predicting$getId()))
                            } else {
                              if ("Predicting" %in% class(predicting)) {
                                predicting$add2HashTable(key = private$id,
                                                            value = self,
                                                            .class = "PredictLayer")
                              } else {
                                stop("A PredictLayer can only belong to a Predicting.")
                              }
                            }
                            return(self)
                          },
                          #' @description
                          #' Generate summary.
                          #'
                          #' @export
                          #'
                          summary = function () {
                            layer_kc = self$getKeyClass()
                            for (k in layer_kc[ , "key"]) {
                              current_obj = self$getFromHashTable(key = k)
                              print(current_obj)
                              cat("\n")
                            }
                          }
                        ),
                        private = list(
                          predicting = NULL
                        ),
                        cloneable = FALSE
)

#' @title createTraining
#' @description
#' Creates a [Training] object. A training object is designed to encapsulate training layers
#' and training meta-layer. Functions [createTrainLayer] and [createTrainMetaLayer] are available
#' to add the training layer and the training meta-layer to a training object.
#'
#' @param id `character` \cr
#' Training's ID.
#' @param target_df `data.frame` \cr
#' Observed target values. A data frame with two columns: individual IDs and response variable values.
#' @param ind_col `character` \cr
#' Name of column of individuals IDs.
#' @param target `character` \cr
#' Name of the target variable.
#' @param problem_type `character` \cr
#' Either "classification" or "regression".
#' @param verbose `boolean` \cr
#' Warning and processing information (including those of cross-validation) will be displayed if set to TRUE.
#' @return
#' The created [Training] object is returned.
#' @seealso [createTrainLayer], [createTrainMetaLayer] and [fusemlr].
#' @export
createTraining = function (id,
                           target_df,
                           ind_col,
                           target,
                           problem_type = "classification",
                           verbose = TRUE) {
  training = Training$new(
    id = id,
    ind_col = ind_col,
    target = target,
    target_df = target_df,
    problem_type = problem_type,
    verbose = verbose
  )
  return(training)
}

#' @title createTrainLayer
#' @description
#' Creates and stores a [TrainLayer] on the [Training] object passed as argument.
#' The main components of a training layer are training data modality, a variable
#' selection methods, and a modality-specific learner.
#'
#' @param training `Training` \cr
#' Training object for storing the created layer.
#' @param train_layer_id `character` \cr
#' ID of the [TrainLayer] to be created.
#' @param train_data `data.frame` \cr
#' Data modality to be stored on the layer.
#' @param varsel_package `character` \cr
#' Package name containing the variable selection algorithm function.
#' Defaults to `NULL` if the function exists in the current working environment.\cr
#' @param varsel_fct `character` \cr
#' Variable selection function name. Default value is `NULL` for no variable selection.
#' If specified, the function must accept at least two parameters: `x` (predictors)
#' and `y` (response values), and return a vector of selected variables.
#' Alternatively, use the interface parameters `x_varsel` and `y_varsel` to map
#' the original argument names, and `extract_var_fct` to specify how to extract
#' the vector of selected variables. An exception is made for the `Boruta` function,
#' which includes an internal adjustment and requires no additional modifications.
#' @param varsel_param `list` \cr
#' List of arguments to be passed to \code{varsel_fct}.
#' @param lrner_package `character` \cr
#' Name of the package containing the learning algorithm function. Defaults to
#' `NULL` if the function is available in the current working environment.
#' @param lrn_fct  `character` \cr
#' Name of the learning function. The function must accept at least two parameters:
#' `x` (predictors) and `y` (response values) and return a model. Alternatively,
#' use the interface parameters `x_lrn` and `y_lrn` to map these names to the
#' original arguments in your function.
#' The returned model must support the generic `predict` function (with arguments
#' `object` and `data`) to generate predictions for new data. Predictions should
#' be either a vector or a `list` containing a vector named `predictions` with
#' the predicted values.\cr
#' If the arguments `object` and `data` have different names in your `predict`
#' function, use the interface parameters below to map them to the original names.
#' Additionally, if predictions are stored as a `matrix` or `data.frame`
#' (e.g., predicted probabilities for dichotomous classification), only the second
#' column (assumed to be class `1` probabilities) will be used.
#' If the predicted values are not returned in one of the formats mentioned above,
#' use the `extract_pred_fct` argument below to specify how to extract the predicted
#' values from the prediction object.
#' @param param_train_list  `character` \cr
#' List of arguments to be passed to \code{lrn_fct}.
#' @param param_pred_list   `character` \cr
#' List of arguments to be passed to \code{predict} when generating predictions.
#' @param na_action `character`\cr
#' Handling of missing values in data during training. Set to `"na.keep"` to retain
#' missing values, or `"na.rm"` to remove instances with missing values.
#' @param object `character` \cr
#' The generic function `predict` uses the parameter `object` to pass a model.
#' If the corresponding argument is named differently in your `predict` function, specify its name.
#' @param data `character` \cr
#' The generic function \code{predict} uses a parameter \code{data} to pass new data.
#' If the corresponding argument is named differently in your predict function, specify the name.
#' @param extract_pred_fct `character or function` \cr
#' If the `predict` function called for the model does not return a vector,
#' use this argument to specify a function (or the name of a function) to extract
#' the vector of predictions. The default value is `NULL` if predictions are returned as a vector.
#' @param extract_var_fct `character or function` \cr
#' If the variable selection function does not return a vector, use this argument
#'  to specify a function (or the name of a function) to extract the vector of selected variables.
#' @param x_varsel `character` \cr
#' If the name of the argument used by the provided original variable selection function to pass
#' the matrix of independent variable is not \code{x}, use this argument to specify how it is called in the provided function.
#' @param y_varsel `character` \cr
#' If the name of the argument used by the provided original variable selection function to pass
#' the target variable is not \code{y}, use this argument to specify how it is called in the provided function.
#' @param x_lrn `character` \cr
#' If the name of the argument used by the provided original learning function to pass
#' the matrix of independent variable is not \code{x}, use this argument to specify how it is called in the provided function.
#' @param y_lrn `character` \cr
#' If the name of the argument used by the provided original learning function to pass
#' the target variable is not \code{y}, use this argument to specify how it is called in the provided function.
#' @return
#' The updated [Training] object (with the new layer) is returned.
#' @seealso [createTrainMetaLayer] and [fusemlr].
#' @export
#'
#' @references
#' Fouodo, C. J. K., Bleskina, M. & Szymczak, S. fuseMLR: an R package for integrative prediction modeling of multi-omics data. BMC Bioinformatics 26, 221 (2025). https://doi.org/10.1186/s12859-025-06248-4
createTrainLayer = function (training,
                             train_layer_id,
                             train_data,
                             varsel_package = NULL,
                             varsel_fct = NULL,
                             varsel_param = list(),
                             lrner_package = NULL,
                             lrn_fct,
                             param_train_list = list(),
                             param_pred_list = list(),
                             na_action = "na.rm",
                             x_varsel = "x",
                             y_varsel = "y",
                             x_lrn = "x",
                             y_lrn = "y",
                             object = "object",
                             data = "data",
                             extract_pred_fct = NULL,
                             extract_var_fct = NULL) {
  # Instantiate a layer
  train_layer = TrainLayer$new(id = train_layer_id,
                               training = training)
  # Instantiate a TrainData
  train_data = TrainData$new(id = sprintf("%s_data", train_layer_id),
                             data_frame = train_data,
                             train_layer = train_layer)
  # Instantiate a VarSel object
  if (!is.null(varsel_fct)) {
    var_sel = VarSel$new(
      id = sprintf("%s_varsel", train_layer_id),
      package = varsel_package,
      varsel_fct = varsel_fct,
      varsel_param = varsel_param,
      train_layer = train_layer,
      na_action = na_action
    )
    var_sel$interface(x = x_varsel,
                      y = y_varsel,
                      object = object,
                      data = data,
                      extract_var_fct = extract_var_fct)
  }
  # Instantiate a Lrner object
  lrner = Lrner$new(
    id = sprintf("%s_lrner", train_layer_id),
    package = lrner_package,
    lrn_fct = lrn_fct,
    param_train_list = param_train_list,
    param_pred_list = param_pred_list,
    train_layer = train_layer,
    na_action = na_action
  )
  lrner$interface(x = x_lrn,
                  y = y_lrn,
                  object = object,
                  data = data,
                  extract_pred_fct = extract_pred_fct)
  if (training$getVerbose()) {
    return(training)
  } else {
    invisible(TRUE)
  }
}



#' @title createTrainMetaLayer
#' @description
#' Creates and store a [TrainMetaLayer] on the [Training] object passed as argument.
#' The meta-layer encapsulates the meta-learner and the fold predictions (internally created) of the layer-specific
#' base models.
#'
#' @param training `Training` \cr
#' Training object for storing the created meta-layer.
#' @param meta_layer_id `character` \cr
#' ID of the layer to be created.
#' @param lrner_package `character` \cr
#' Package name containing the variable selection algorithm function.
#' Defaults to `NULL` if the function exists in the current working environment.
#' @param lrn_fct  `character` \cr
#' Name of the learning function. The function must accept at least two
#' parameters: `x` (predictors) and `y` (response values), and return a model.
#' If not, use the interface parameters `x_lrn` and `y_lrn` below to map these
#' argument names to the original arguments in your function. The returned model
#' must support the generic `predict` function (with arguments `object` and `data`)
#' to make predictions for new data, and the predictions should be a vector or
#' a `list` containing a vector called `predictions` with the predicted values.
#' If the arguments `object` and `data` are named differently in your predict
#' function, use the interface parameters `object` and `data` below to specify
#' the original names. See the details below about meta-learners.
#' @param param_train_list  `character` \cr
#' List of arguments to be passed to \code{lrn_fct}.
#' @param param_pred_list   `list` \cr
#' List of arguments to be passed to \code{predict} when computing predictions.
#' @param na_action `character`\cr
#' Handling of missing values in modality-specific predictions during training.
#' Set to `"na.keep"` to keep missing values, `"na.rm"` to remove individuals
#' with missing values or `"na.impute"` to impute missing values in modality-specific
#'  predictions. Only median and mode based imputations are actually handled.
#'  With the `"na.keep"` option, ensure that the provided meta-learner can handle missing values.
#' @param x_lrn `character` \cr
#' If the argument name used by the provided original function to pass the matrix
#' of independent variables is not `x`, use this argument to specify the name used
#' in the function.
#' @param y_lrn `character` \cr
#' If the argument name used by the provided original function to pass the target
#'  variable is not `y`, use this argument to specify the name used in the function.
#' @param object `character` \cr
#' The generic function \code{predict} uses a parameter \code{object} to pass a model.
#' If the corresponding argument is named differently in your predict function, specify the name.
#' @param data `character` \cr
#' The generic function \code{predict} uses a parameter \code{data} to pass new data.
#' If the corresponding argument is named differently in your predict function, specify the name.
#' @param extract_pred_fct `character or function` \cr
#' If the predict function that is called for the model does not return a vector, then
#' use this argument to specify a (or a name of a) function that can be used to extract vector of predictions.
#' Defaults to NULL, if predictions are a vector.
#' @details
#'
#' Internal meta-learners are available in the package.
#'
#' The [cobra] meta-learner implements the COBRA (COmBined Regression Alternative),
#' an aggregation method for combining predictions from multiple individual learners (Biau et al. 2014).
#' This method aims to tune key parameters for achieving optimal predictions
#' by averaging the target values of similar candidates in the training dataset's predictions.
#' Only the training points that are sufficiently similar to the test point
#' (based on the proximity threshold \code{epsilon}) are used for prediction.
#' If no suitable training points are found, the function returns \code{NA}.
#'
#' The [weightedMeanLearner] evaluates the prediction performance of modality-specific
#' learners and uses these estimates to weight the base models, aggregating their
#' predictions accordingly.
#'
#' The [bestLayerLearner] evaluates the prediction performance of modality-specific
#' learners and returns predictions made by the best learner as the meta-prediction.
#'
#' Beyond the internal meta-learners, any other learning algorithm can be used.
#'
#' @return
#' The updated [Training] object (with the new layer) is returned.
#' @seealso [createTrainLayer], [varSelection], and [fusemlr].
#' @references
#' Fouodo, C. J. K., Bleskina, M. & Szymczak, S. fuseMLR: an R package for integrative prediction modeling of multi-omics data. BMC Bioinformatics 26, 221 (2025). https://doi.org/10.1186/s12859-025-06248-4 \cr
#' Biau, G., Fischer, A., Guedj, B., & Malley, J. D. (2014). COBRA: A combined regression strategy. The Journal of Multivariate Analysis 46:18-28
#' @export
#'
createTrainMetaLayer = function (training,
                                 meta_layer_id,
                                 lrner_package = NULL,
                                 lrn_fct,
                                 param_train_list = list(),
                                 param_pred_list = list(),
                                 na_action = "na.impute",
                                 x_lrn = "x",
                                 y_lrn = "y",
                                 object = "object",
                                 data = "data",
                                 extract_pred_fct = NULL) {
  # Instantiate a layer
  train_meta_layer = TrainMetaLayer$new(id = meta_layer_id,
                                        training = training)
  meta_lrner = Lrner$new(
    id = sprintf("%s_lrner", meta_layer_id),
    package = lrner_package,
    lrn_fct = lrn_fct,
    param_train_list = param_train_list,
    param_pred_list = param_pred_list,
    train_layer = train_meta_layer,
    na_action = na_action
  )
  meta_lrner$interface(x = x_lrn,
                       y = y_lrn,
                       object = object,
                       data = data,
                       extract_pred_fct = extract_pred_fct)
  return(training)
}

#' @title varSelection
#' @description
#' Variable selection on the training object passed as argument.
#'
#' @param training `Training` \cr
#' Training object for storing the created layer.
#' @param ind_subset `vector` \cr
#' ID subset of individuals to be used for variable selection.
#'
#' @return
#' A \code{data.frame} with two columns: layer and selected variables.
#' @export
#'
#' @references
#' Fouodo, C. J. K., Bleskina, M. & Szymczak, S. fuseMLR: an R package for integrative prediction modeling of multi-omics data. BMC Bioinformatics 26, 221 (2025). https://doi.org/10.1186/s12859-025-06248-4 \cr
varSelection = function (training,
                         ind_subset = NULL) {
  selected = training$varSelection(ind_subset = ind_subset,
                                   verbose = training$getVerbose())
  return(selected)
}

#' @title fusemlr
#' @description
#' Trains the [Training] object passed as argument. A training object must contain
#' the training layers and a training meta-layer. A training layer encapsulates
#' data modalities, a variable selection method and a learner. Use the function
#' [createTraining] to create a training object, [createTrainLayer] to add training
#' layers to the created training object, and [createTrainMetaLayer] to add a meta-layer
#' with the corresponding meta-learner to the training object. The function `fusemlr`
#' is designed to train all training layers and the meta-learner. After training
#' the layer-specific base models and the meta-model will be stored in the training
#' object which can be used for predictions.
#'
#' @param training `Training` \cr
#' Training object for storing training layers.
#' @param ind_subset `vector` \cr
#' ID subset to be used for training.
#' @param use_var_sel `boolean` \cr
#' If `TRUE` and no variable selection has been performed for the provide training object,
#' then a variable selection will proceed the training.
#' Otherwise, if variable selection has been previously performed, the selected variables will be used for training.
#' @param resampling_method `function` \cr
#' Function for internal validation. If not specify, the \code{resampling} function
#' from the package \code{caret} is used for a 10-folds cross-validation.
#' @param resampling_arg `list` \cr
#' List of arguments to be passed to the function.
#' @param seed `integer` \cr
#' Random seed used for resampling. Default is NULL, which generates the seed from \code{R}.
#'
#' @return
#' The current object is returned, with each learner trained on each layer.
#' @seealso [createTrainLayer], [createTrainMetaLayer], [extractModel] and [extractData].
#' @export
#'
#' @references
#' Fouodo C.J.K, Bleskina M. and Szymczak S. (2024). fuseMLR: An R package for integrative prediction modeling of multi-omics data, paper submitted.
fusemlr = function (training,
                    ind_subset = NULL,
                    use_var_sel = FALSE,
                    resampling_method = NULL,
                    resampling_arg = list(),
                    seed = NULL) {
  if (!training$getVarSelDone() & use_var_sel) {
    training$varSelection(ind_subset = ind_subset,
                          verbose = training$getVerbose())
  }
  training$train(ind_subset = ind_subset,
                 use_var_sel = use_var_sel,
                 resampling_method = resampling_method,
                 resampling_arg = resampling_arg,
                 seed = seed)
  invisible(training)
}



#' @title predict.Training
#' @description
#' Computes predictions for the [Testing] object passed as argument.
#'
#' @param object `Training` \cr
#' A trained Training object to be used to compute predictions.
#' @param testing `Testing` \cr
#' A new testing object to be predicted.
#' @param ind_subset `vector` \cr
#' Vector of IDs to be predicted.
#' @param ... `any` \cr
#' Further arguments passed to or from other methods.
#'
#' @return
#' The final predicted object. All layers and the meta layer are predicted.
#' @export
#' @method predict Training
predict.Training = function (object,
                             testing,
                             ind_subset = NULL,
                             ...) {
  if (!inherits(object, "Training")) {
    stop("Error: 'object' must be a Training object.")
  }
  predictions = object$predict(testing = testing,
                               ind_subset = ind_subset)
  return(predictions)
}

#' @title extractModel
#' @description
#' Extracts models stored on each layers; base and meta models are extracted.
#'
#' @param training `Training` \cr
#' The [Training] object of interest.
#'
#' @return
#' A list of models is returned.
#' @export
#'
extractModel = function (training) {
  return(training$getModel())
}

#' @title extractData
#' @description
#' Extracts data stored on each layers; base data and modality-specific predictions (for Training) are extracted.
#'
#' @param object `Training or Testing` \cr
#' The object of interest.
#'
#' @return
#' A list of data is returned.
#' @export
#'
extractData = function (object) {
  return(object$getData())
}

#' @title Training object Summaries
#' @description
#' Summaries a `fuseMLR` [Training] object.
#'
#' @param object `Training` \cr
#' The [Training] object of interest.
#' @param ... `any` \cr
#' Further arguments.
#'
#' @export
#'
summary.Training = function (object, ...) {
  object$summary()
}

#' @title upsetplot
#' @description
#' An upset plot of overlapping individuals.
#'
#' @param object `Training or Testing` \cr
#' Training or testing object for each the upset plot will be created.
#' @param ... `any` \cr
#' Further arguments to be passed to the \code{upset} function from package \code{UpSetR}.
#'
#' @export
#'
upsetplot = function (object, ...) {
  object$upset(...)
  invisible(TRUE)
}

#' @title Class HashTable
#' @description Hashtable to contain object modalities. Storage objects like [Training] and [TrainLayer] are extensions of this class.
#'
#' @export
#' @importFrom R6 R6Class
#'
#' @importFrom digest digest
HashTable <- R6Class("HashTable",
                     public = list(
                       #' @description
                       #' Initialize a default parameters list.
                       #'
                       #' @param id `character` \cr
                       #'  ID of the hash table. It must be unique.
                       #'
                       #' @export
                       initialize = function (id) {
                         private$id = id
                         private$key_class = data.frame(key = character(0L),
                                                        class = character(0L))
                         private$hash_table = new.env(hash = TRUE,
                                                      parent = emptyenv())
                       },
                       #' @description
                       #' Printer
                       #' @param ... `any`
                       #'
                       #' @export
                       #'
                       print = function (...) {
                         cat("Class: HashTable\n")
                         cat(sprintf("id: %s\n", private$id))
                         cat("-----------------\n")
                         print(private$key_class)
                       },
                       #' @description
                       #' Function to add a key-value pair to the hash table.
                       #' @param key `character` \cr
                       #' The key to be added.
                       #' @param value `object` \cr
                       #' Object to be added.
                       #' @param .class `character` \cr
                       #' Class of the object to be added.
                       #' @export
                       #'
                       add2HashTable = function (key, value, .class) {
                         hash_key = digest(key, algo = "xxhash64")
                         private$hash_table[[hash_key]] = value
                         # Tests if the instance has already been stored and
                         # updates table accordingly.
                         if (!nrow(self$getKeyClass())) {
                           private$key_class[nrow(private$key_class) + 1L, ] = c(key,
                                                                                 .class)
                         } else {
                           key_exists = (key %in% self$getKeyClass()$key)
                           class_exists = (.class %in% self$getKeyClass()$class)
                           if (!(key_exists & class_exists)) {
                             private$key_class[nrow(private$key_class) + 1L, ] = c(key,
                                                                                   .class)
                           }
                         }
                         invisible(private)
                       },
                       #' @description
                       #' Getter of the object which the key passed as argument.
                       #'
                       #' @param key `character` \cr
                       #' Key of the required object.
                       #'
                       #' @export
                       #'
                       getFromHashTable = function (key) {
                         hash_key = digest(key, algo = "xxhash64")
                         if (exists(hash_key, envir = private$hash_table)) {
                           return(private$hash_table[[hash_key]])
                         } else {
                           return(NULL)
                         }
                       },
                       #' @description
                       #' Getter of the \code{data.frame} that stores all key class pairs.
                       #'
                       #' @return
                       #' [data.frame]
                       #'
                       #' @export
                       #'
                       getKeyClass = function () {
                         return(private$key_class)
                       },
                       #' @description
                       #' Remove the object with the corresponding key from the hashtable.
                       #'
                       #' @param key
                       #' Key of the object to be removed.
                       #'
                       #' @export
                       #'
                       removeFromHashTable = function (key) {
                         hash_key = digest(key, algo = "xxhash64")
                         if (exists(hash_key, envir = private$hash_table)) {
                           rm(list = hash_key, envir = private$hash_table)
                           rm_index = which(private$key_class$key == key)
                           private$key_class = private$key_class[- rm_index , ]
                         }
                         invisible(TRUE)
                       },

                       #' @description
                       #' Getter of the current object ID.
                       #'
                       #' @export
                       #'
                       getId = function () {
                         return(private$id)
                       },
                       #' @description
                       #' Getter of the current hashtable.
                       #'
                       #' @export
                       getHashTable = function () {
                         return(private$hash_table)
                       },
                       #' @description
                       #' Check whether object from a class has already been stored.
                       #'
                       #' @param .class `character` \cr
                       #'
                       #' @return
                       #' Boolean value
                       #'
                       checkClassExist = function (.class) {
                         return(.class %in% private$key_class$class)
                       }
                     ),
                     private = list(
                       id = character(0L),
                       hash_table = NULL,
                       key_class = NULL
                     ),
                     # TODO: define a deep_clone function for this class.
                     cloneable = FALSE
)

hashtable <- HashTable$new(id = "test")

test_that("HashTable: initialize", {
  expect_s3_class(hashtable, "HashTable")
  print(hashtable)
})

test_that("HashTable: add2HashTable", {
  expect_no_error(hashtable$add2HashTable(key = "obj1",
                                          value = 2L,
                                          .class = "test"))
  expect_no_error(hashtable$add2HashTable(key = "obj1",
                                          value = 2L,
                                          .class = "test"))
  expect_no_error(hashtable$add2HashTable(key = "obj1",
                                          value = 2L,
                                          .class = "test2"))
})

test_that("HashTable: getFromHashTable", {
  expect_no_error(hashtable$getFromHashTable(key = "obj1"))
  expect_no_error(hashtable$getFromHashTable(key = "notexists"))
})

test_that("HashTable: getKeyClass", {
  expect_no_error(hashtable$getKeyClass())
})

test_that("HashTable: removeFromHashTable", {
  expect_no_error(hashtable$removeFromHashTable(key = "obj1"))
  expect_no_error(hashtable$getFromHashTable(key = "notexists"))
})

test_that("HashTable: getId", {
  expect_no_error(hashtable$getId())
})

test_that("HashTable: getHashTable", {
  expect_no_error(hashtable$getHashTable())
})


test_that("HashTable: checkClassExist", {
  expect_no_error(hashtable$checkClassExist(.class = "notexists"))
})

#' Create Loss
#'
#' @param pred A \code{vector} of predictions.
#' @param target A \code{vector} of target values.
#'
createLoss = function (pred, target) {
  # Check if lengths of `pred` and `target` match
  if (length(pred) != length(target)) {
    stop("Error: The lengths of 'pred' and 'target' must be the same.")
  }

  idx = !is.na(pred)
  idx_na = is.na(pred)

  # loss
  if (any(idx)) {
    valid_loss = sum((target[idx] - pred[idx])^2)
  } else {
    valid_loss = 0
  }

  # If NA, then penalize by assigning maximum loss
  if (any(idx_na)) {
    na_loss =  sum((target[idx_na] - (1 - target[idx_na]))^2)
  } else {
    na_loss = 0
  }

  # Loss function
  total_loss = (valid_loss + na_loss) / length(target)
  return(total_loss)
}

test_that("createLoss works", {
  # Test 1: No error for valid inputs
  expect_no_error({
    pred <- c(0.1, 0.5, NA, 0.7)
    target <- c(0, 1, 1, 0)
    createLoss(pred, target)
  })

  # Test 2: Correct result for mixed valid and NA values
  pred <- c(0.2, NA, 0.8)
  target <- c(0, 1, 1)
  result <- createLoss(pred, target)
  # Calculate expected loss manually
  valid_loss <- sum((target[!is.na(pred)] - pred[!is.na(pred)])^2)
  na_loss <- sum((target[is.na(pred)] - (1 - target[is.na(pred)]))^2)
  expected <- (valid_loss + na_loss) / length(target)
  expect_equal(result, expected)

  # Test 3: Correct result when all values in `pred` are NA
  pred <- c(NA, NA, NA)
  target <- c(1, 0, 1)
  result <- createLoss(pred, target)
  # Expected loss is based only on NA penalties
  na_loss <- sum((target - (1 - target))^2)
  expect_equal(result, na_loss / length(target))

  # Test 4: No error for inputs without any NA
  expect_no_error({
    pred <- c(0.2, 0.6, 0.8)
    target <- c(0, 1, 1)
    result <- createLoss(pred, target)
    # Check if the loss matches the expected value
    expect_equal(result, sum((target - pred)^2) / length(target))
  })

  # Test 5: Error for mismatched lengths of `pred` and `target`
  expect_error({
    pred <- c(0.2, 0.6)
    target <- c(0, 1, 1)  # Different length
    createLoss(pred, target)
  })

})

#' @title TrainLayer Class
#'
#' @description
#' This class implements a traning layer. A [TrainLayer] object can only exist as a component of a [Training] object.
#'
#' A training layer is structured as followed:
#'
#' * [TrainData]: Data to be used to train the learner.
#' * [Lrner]: Includes a learning function and the package implementing the function.
#' * [Model]: The result of training the learner on the training data.
#' * [VarSel]: Includes a variable selection function and the package implementing the function.
#'
#' A training layer can train its learner on its training data and store the resulting model. See the public function \code{Layer$train()} below.
#'
#' A training layer can make predictions for a new layer passed as argument to its predict function. See the public function \code{Layer$predict()} below.
#'
#' @export
#' @importFrom R6 R6Class
#' @seealso [Training], [Lrner], [TrainData], [TestData] and [Model]
TrainLayer <- R6Class("TrainLayer",
                      inherit = HashTable,
                      public = list(
                        #' @description
                        #' constructor
                        #'
                        #' @param id `character`\cr
                        #' Training layer id.
                        #' @param training `Training`\cr
                        #'
                        initialize = function (id, training) {
                          super$initialize(id = id)
                          private$training = training
                          if ("Training" %in% class(training)) {
                            training$add2HashTable(key = id,
                                                      value = self,
                                                      .class = "TrainLayer")
                          } else {
                            stop("A TrainLayer can only belong to a Training.")
                          }
                          private$status = FALSE
                        },
                        #' @description
                        #' Printer
                        #' @param ... `any`
                        #'
                        print = function (...){
                          if (!private$status) {
                            status = "Not trained"
                          } else {
                            status = "Trained"
                          }
                          cat(sprintf("TrainLayer            : %s\n", private$id))
                          cat(sprintf("Status                : %s\n", status))
                          stored_obj = self$getKeyClass()
                          if (!nrow(stored_obj)) {
                            cat("Empty layer.\n")
                          } else {
                            cat(sprintf("Nb. of objects stored : %s\n", nrow(stored_obj)))
                            cat("-----------------------\n")
                            print(stored_obj)
                          }
                        },
                        #' @description
                        #' Getter of the current training object.
                        #'
                        #' @return
                        #' The current training object is returned.
                        #'
                        getTraining = function () {
                          return(private$training)
                        },
                        #' @description
                        #' Getter of the target object.
                        #' @export
                        getTargetObj = function () {
                          return(private$training$getTargetObj())
                        },
                        #' @description
                        #' Trains the current layer.
                        #'
                        #' @param ind_subset `vector` \cr
                        #' ID subset of individuals to be used for training.
                        #' @param use_var_sel `boolean` \cr
                        #' If TRUE, variable selection is performed before training.
                        #' @param verbose `boolean` \cr
                        #' Warning messages will be displayed if set to TRUE.
                        #'
                        #' @return
                        #' The current layer is returned with the resulting model.
                        #' @export
                        #'
                        train = function (ind_subset = NULL,
                                          use_var_sel = FALSE,
                                          verbose = TRUE) {
                          layer_kc = self$getKeyClass()
                          # Stop if either learner of data is missing on this layer.
                          if (!("Lrner" %in% layer_kc[ , "class"])){
                            stop(sprintf("No learner on layer %s.", self$getId()))
                          } else {
                            if (!("TrainData" %in% layer_kc[ , "class"])) {
                              stop(sprintf("No data on layer %s.", self$getId()))
                            }
                          }
                          # The learner is trained on the current dataset
                          lrner_key = layer_kc[layer_kc$class == "Lrner" , "key"]
                          lrner = self$getFromHashTable(key = lrner_key[1L])
                          if (use_var_sel & (!self$checkVarSelExist())) {
                            if (verbose) {
                              warning(sprintf("No var. sel. on layer %s.", self$getId()))
                            }
                            use_var_sel = FALSE
                          }
                          if (verbose) {
                            message(sprintf("Training of base model on layer %s started.\n", self$getId()))
                          }
                          model = lrner$train(ind_subset = ind_subset,
                                              use_var_sel = use_var_sel,
                                              verbose = verbose)
                          if (verbose) {
                            message(sprintf("Training of base model on layer %s done.\n", self$getId()))
                          }
                          # Updating the training status.
                          if (!private$status) {
                            # The training layer has not been trained before.
                            private$training$increaseNbTrainedLayer()
                            private$status = TRUE
                          } else {
                            # The training layer has been trained before.
                            private$status = TRUE
                          }
                          return(model)
                        },
                        #' @description
                        #' Variable selection on the current layer.
                        #'
                        #' @param ind_subset `vector` \cr
                        #' ID subset of individuals to be used for variable selection.
                        #' @param verbose `boolean` \cr
                        #' Warning messages will be displayed if set to TRUE.
                        #'
                        #' @return
                        #' The current layer is returned with the resulting model.
                        #' @export
                        #'
                        varSelection = function (ind_subset = NULL,
                                                 verbose = TRUE) {
                          layer_kc = self$getKeyClass()
                          # Stop if either selector or data is missing on this layer.
                          if (!("VarSel" %in% layer_kc[ , "class"])) {
                            if (verbose) {
                              warning(sprintf("No var. sel. method on layer %s.", self$getId()))
                            }
                            return(NULL)
                          } else {
                            if (!("TrainData" %in% layer_kc[ , "class"])) {
                              stop(sprintf("No data on layer %s.", self$getId()))
                            }
                          }
                          # The learner is trained on the current dataset
                          varsel_key = layer_kc[layer_kc$class == "VarSel" , "key"]
                          varsel = self$getFromHashTable(key = varsel_key[1L])
                          if (verbose) {
                            message(sprintf("Variable selection on layer %s started.\n", self$getId()))
                          }
                          selected = varsel$varSelection(ind_subset = ind_subset)
                          if (verbose) {
                            message(sprintf("Variable selection on layer %s done.\n", self$getId()))
                          }
                          return(selected)
                        },
                        #' @description
                        #' Predicts values for the new layer taking as argument.
                        #'
                        #' @param new_layer `TrainLayer` \cr
                        #' @param use_var_sel `boolean` \cr
                        #' If TRUE, selected variables available at each layer are used.
                        #' @param ind_subset `vector` \cr
                        #'
                        #' @return
                        #' A new [PredictLayer] object with the predicted data is returned.
                        #' @export
                        #'
                        predict = function (new_layer,
                                            use_var_sel,
                                            ind_subset = NULL) {
                          k = self$getId()
                          # Layer IDs must match together.
                          if (k == new_layer$getId()) {
                            m_layer = self$getModel()
                          } else {
                            stop("The new layer ID does not match with the current layer ID.")
                          }
                          testing_data = new_layer$getTestData()
                          # Predicting: Data and model exist on this layer.
                          model = self$getModel()
                          pred_data = model$predict(testing_data = testing_data,
                                                    use_var_sel,
                                                    ind_subset = ind_subset)
                          # Initialize a predicted layer to store predictions
                          pred_layer = PredictLayer$new(
                            id = private$id
                          )
                          pred_data$setPredictLayer(pred_layer)
                          return(pred_layer)
                        },
                        #' @description
                        #' Getter of the training dataset stored on the current layer.
                        #'
                        #' @return
                        #' The stored [TrainData] object is returned.
                        #' @export
                        #'
                        getTrainData = function () {
                          layer_kc = self$getKeyClass()
                          if ("TrainData" %in% layer_kc[ , "class"]) {
                            train_data_key = layer_kc[layer_kc$class == "TrainData" ,
                                                      "key"]
                            train_data = self$getFromHashTable(key = train_data_key[1L])
                          } else {
                            stop(sprintf("No train data on layer %s.", self$getId()))
                          }
                          return(train_data)
                        },
                        #' @description
                        #' Getter of target values from the current layer.
                        #'
                        #' @return
                        #' A \code{data.frame} containing individuals IDs and corresponding target values.
                        #' @export
                        #'
                        getTargetValues = function () {
                          layer_kc = self$getKeyClass()
                          # Stop if training data is missing on this layer.
                          if (("TrainData" %in% layer_kc[ , "class"])) {
                            # Searching for layer specific training dataset
                            train_data_key = layer_kc[layer_kc$class == "TrainData" ,
                                                      "key"]
                            train_data = self$getTrainData()
                            train_data_frame = train_data$getDataFrame()
                            target_data = train_data_frame[ , c(train_data$getIndCol(),
                                                                train_data$getTargetName())]
                            return(target_data)
                          } else {
                            stop(sprintf("No data on layer %s.", self$getId()))
                          }
                        },
                        #' @description
                        #' Getter of IDS from the current layer.
                        #'
                        #' @return
                        #' A \code{data.frame} containing individuals IDs values.
                        #' @export
                        #'
                        getIndIDs = function () {
                          layer_kc = self$getKeyClass()
                          # Stop if training data is missing on this layer.
                          if (("TrainData" %in% layer_kc[ , "class"])) {
                            # Searching for layer specific new dataset
                            data_key = layer_kc[layer_kc$class == "TrainData" ,
                                                "key"]
                            current_data = self$getTestData()
                          } else {
                            stop(sprintf("No data on layer %s.", self$getId()))
                          }
                          current_data_frame = current_data$getDataFrame()
                          ids_data = current_data_frame[ , current_data$getIndCol(), drop = FALSE]
                          return(ids_data)
                        },
                        #' @description
                        #' Getter of the new data.
                        #'
                        #' @return
                        #' The stored [TestData] object is returned.
                        # A TrainLayer plays the role of TestLayer when creating meta modality-specific prediction data.
                        getTestData = function () {
                          layer_kc = self$getKeyClass()
                          if (any(c("TestData", "TrainData") %in% layer_kc[ , "class"])) {
                            if ("TestData" %in% layer_kc[ , "class"]) {
                              # nocov start
                              # TODO: This part of the code will never be executed. Check it and maybe remove it
                              testing_data_key = layer_kc[layer_kc$class == "TestData" ,
                                                      "key"]
                              testing_data = self$getFromHashTable(key = testing_data_key[1L])
                              # nocov end
                            } else {
                              testing_data = self$getTrainData()
                            }
                          } else {
                            stop(sprintf("No new data on layer %s.", self$getId()))
                          }
                          return(testing_data)
                        },
                        #' @description
                        #' Getter of the learner.
                        #'
                        #' @return
                        #' The stored [Lrner] object is returned.
                        #' @export
                        getLrner = function () {
                          layer_kc = self$getKeyClass()
                          if (!("Lrner" %in% layer_kc[ , "class"])) {
                            stop(sprintf("No Lrner on layer %s.", self$getId()))
                          } else {
                            lrner_key = layer_kc[layer_kc$class == "Lrner" ,
                                                 "key"]
                            lrner = self$getFromHashTable(key = lrner_key[1L])
                          }
                          return(lrner)
                        },
                        #' @description
                        #' Getter of the variable selector.
                        #'
                        #' @return
                        #' The stored [VarSel] object is returned.
                        #' @export
                        getVarSel = function () {
                          layer_kc = self$getKeyClass()
                          if (!("VarSel" %in% layer_kc[ , "class"])) {
                            stop(sprintf("No VarSel on layer %s.", self$getId()))
                          } else {
                            varsel_key = layer_kc[layer_kc$class == "VarSel" ,
                                                  "key"]
                            varsel = self$getFromHashTable(key = varsel_key[1L])
                          }
                          return(varsel)
                        },
                        #' @description
                        #' Getter of the model.
                        #'
                        #' @return
                        #' The stored [Model] object is returned.
                        #' @export
                        #'
                        getModel = function () {
                          layer_kc = self$getKeyClass()
                          if (!("Model" %in% layer_kc[ , "class"])) {
                            stop(sprintf("No Model on layer %s.", self$getId()))
                          } else {
                            model_key = layer_kc[layer_kc$class == "Model" ,
                                                 "key"]
                            model = self$getFromHashTable(key = model_key[1L])
                          }
                          return(model)
                        },
                        #' @description
                        #' Check whether a learner has been already stored.
                        #'
                        #' @return
                        #' Boolean value
                        #'
                        checkLrnerExist = function () {
                          return(super$checkClassExist(.class = "Lrner"))
                        },
                        #' @description
                        #' Check whether a model has been already stored.
                        #'
                        #' @return
                        #' Boolean value
                        #'
                        checkModelExist = function () {
                          return(super$checkClassExist(.class = "Model"))
                        },
                        #' @description
                        #' Check whether a variable selection tool has been already stored.
                        #'
                        #' @return
                        #' Boolean value
                        #'
                        checkVarSelExist = function () {
                          return(super$checkClassExist(.class = "VarSel"))
                        },
                        #' @description
                        #' Check whether a training data has been already stored.
                        #'
                        #' @return
                        #' Boolean value
                        #'
                        checkTrainDataExist = function () {
                          return(super$checkClassExist(.class = "TrainData"))
                        },
                        #' @description
                        #' Generate summary.
                        #'
                        #' @export
                        #'
                        summary = function () {
                          cat(sprintf("   Layer %s\n", self$getId()))
                          cat("   ----------------\n")
                          if (!private$status) {
                            status = "Not trained"
                          } else {
                            status = "Trained"
                          }
                          cat(sprintf("   TrainLayer            : %s\n", private$id))
                          cat(sprintf("   Status                : %s\n", status))
                          stored_obj = self$getKeyClass()
                          if (!nrow(stored_obj)) {
                            cat("   Empty layer.\n")
                          } else {
                            cat(sprintf("   Nb. of objects stored : %s\n", nrow(stored_obj)))
                          }
                          cat("   ----------------\n")
                          layer_kc = self$getKeyClass()
                          cat(sprintf("   Object(s) on layer %s\n\n", self$getId()))
                          if (!nrow(layer_kc)) {
                            cat("      Empty layer\n")
                          }
                          for (k in layer_kc[layer_kc$class != "Model" , "key"]) {
                            cat("      ----------------\n")
                            current_obj = self$getFromHashTable(key = k)
                            current_obj$summary()
                            cat("      ----------------\n")
                            cat("\n")
                          }
                        }
                      ),
                      private = list(
                        training = NULL,
                        status = FALSE
                      ),
                      # TODO: define a deep_clone function for this class.
                      cloneable = FALSE
)

data("multi_omics")
test_that("TrainLayer: all tests", {
  # Training and layers
  training <- Training$new(id = "training",
                                ind_col = "IDS",
                                target = "disease",
                                target_df = multi_omics$training$target)
  # A TrainLayer can only belong to a Training
  expect_error({
    TrainLayer$new(id = "geneexpr", training = "not_a_training")
  })
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for training empty layer                           +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  expect_no_error({
    tl_ge <- TrainLayer$new(id = "geneexpr", training = training)
  })
  expect_no_error({
    tl_ge$summary()
  })
  # Must fail because of not existing learner
  expect_error({tl_ge$train()})
  expect_error({
    tl_ge$getLrner()
  })
  lrner_ge <- Lrner$new(id = "ranger",
                        package = "ranger",
                        lrn_fct = "ranger",
                        param_train_list = list(probability = TRUE,
                                                mtry = 2L,
                                                num.trees = 10L),
                        train_layer = tl_ge)
  expect_no_error({
    tl_ge$getLrner()
  })
  # Must fail because of not existing training data
  expect_error({tl_ge$train()})
  expect_error({tl_ge$getTargetValues()})
  # Must throw a warning message because of no variable selection tool
  expect_warning({tl_ge$varSelection()})
  expect_error({tl_ge$getVarSel()})
  expect_error({tl_ge$getPredictions()})
  varsel_ge <- VarSel$new(id = "varsel_geneexpr",
                          package = "Boruta",
                          varsel_fct = "Boruta",
                          varsel_param = list(num.trees = 50L,
                                              mtry = 3L),
                          train_layer = tl_ge)
  # Must fail because of not existing training data
  expect_error({tl_ge$varSelection()})
  expect_error({tl_ge$getTestData()})
  train_data_ge <- TrainData$new(id = "geneexpr",
                                 train_layer = tl_ge,
                                 data_frame = multi_omics$training$geneexpr)

  testing <- Testing$new(id = "testing", ind_col = "IDS")
  nl_ge <- TestLayer$new(id = "geneexpr", testing = testing)
  # Must fail because of not existing model
  expect_error({
    tl_ge$predict(nl_ge)
  })
  expect_no_error({
    tl_ge$summary()
  })
  tl_ge$train()
  expect_no_error({
    tl_ge$summary()
  })
  testing <- Testing$new(id = "testing", ind_col = "IDS")
  nl_ge <- TestLayer$new(id = "geneexpr_inconsistent", testing = testing)
  # Must fail because of inconsistent ID
  expect_error({
    tl_ge$predict(nl_ge)
  })
  testing <- Testing$new(id = "testing", ind_col = "IDS")
  nl_ge <- TestLayer$new(id = "geneexpr", testing = testing)
  # Must fail because of not existing new data
  expect_error({
    tl_ge$predict(nl_ge)
  })
  testing_data_ge <- TestData$new(id = "geneexpr",
                             new_layer = nl_ge,
                             data_frame = multi_omics$testing$geneexpr)
  expect_no_error({
    tl_ge$predict(nl_ge, use_var_sel = TRUE)
  })
  # expect_no_error({
  #   tl_ge$getPredictions()
  # })
})

data("multi_omics")
test_that("TrainLayer: all tests", {
  # Training and layers
  training <- Training$new(id = "training",
                                ind_col = "IDS",
                                target = "disease",
                                target_df = multi_omics$training$target)
  tl_meta <- TrainMetaLayer$new(id = "meta",
                                training = training)
  # A TrainLayer can only belong to a Training
  expect_no_error({
    print(tl_meta)
  })
  expect_no_error({
    tl_meta$summary()
  })
  expect_error({
    tl_meta$train()
  })
  expect_error({
    tl_meta$getLrner()
  })
  lrner_meta <- Lrner$new(id = "weighted",
                          lrn_fct = "weightedMeanLearner",
                          param_train_list = list(),
                          train_layer = tl_meta)
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for training empty meta layer                      +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  # Tests for training empty not meta layer                  +
  # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  tl_ge <- TrainLayer$new(id = "geneexpr", training = training)
  lrner_ge <- Lrner$new(id = "ranger",
                        package = "ranger",
                        lrn_fct = "ranger",
                        param_train_list = list(probability = TRUE,
                                     mtry = 2L,
                                     num.trees = 10L),
                        train_layer = tl_ge)
  train_data_ge <- TrainData$new(id = "geneexpr",
                                 train_layer = tl_ge,
                                 data_frame = multi_omics$training$geneexpr)
  expect_error({
    tl_meta$train()
  })
  expect_error({
    tl_meta$getTrainData()
  })
  # Predicting without model
  testing <- Testing$new(id = "testing", ind_col = "IDS")
  nl_ge <- TestLayer$new(id = "meta", testing = testing)
  # Must fail because of not existing model
  expect_error({
    tl_meta$predict(nl_ge)
  })
  disease <- training$getTargetValues()$disease
  expect_warning({
    skip_on_cran()
    training$train(resampling_method = "caret::createFolds",
                      resampling_arg = list(y = disease,
                                            k = 2L),
                      use_var_sel = FALSE)
    print(tl_meta)
    tl_meta$train()
    training$train(resampling_method = "caret::createFolds",
                      resampling_arg = list(y = disease,
                                            k = 2L),
                      use_var_sel = FALSE)
  })
  expect_no_error({
    tl_meta$getLrner()
  })
})

#' @title Target Class
#'
#' @description
#' This class implements the target object. A [Target] object can only
#' exist as a component of a [Training] object.
#'
#' @export
#'
#' @importFrom R6 R6Class
#' @seealso [TrainLayer], [Lrner], [Model], [TestData]
Target <- R6Class("Target",
                  inherit = Data,
                  public = list(
                    #' @description
                    #' Initialize a new object from the current class.
                    #'
                    #' @param id `character` \cr
                    #' The Object ID.
                    #' @param data_frame `data.frame`\cr
                    #' \code{data.frame} containing data.
                    #' @param training `Training` \cr
                    #' Training where to store the current object.
                    initialize = function (id,
                                           data_frame,
                                           training) {
                      if (!any(c("Training") %in% class(training))) {
                        # nocov start
                        stop("A Target can belong only to a Training object.\n")
                        # nocov end
                      }
                      ind_col = training$getIndCol()
                      target = training$getTarget()
                      if (!all(c(ind_col, target) %in% colnames(data_frame))) {
                        # nocov start
                        stop("Individual column ID or target variable not found in the provided data.frame.\n")
                        # nocov end
                      }
                      if (training$checkTargetExist()) {
                        # Remove TrainData if already existing
                        # nocov start
                        key_class = train_layer$getKeyClass()
                        key = key_class[key_class$class == "Target", "key"]
                        training$removeFromHashTable(key = key)
                        # nocov end
                      }
                      private$training = training
                      missing_target = is.na(data_frame[ , target])
                      if (any(missing_target)) {
                        # nocov start
                        data_frame = data_frame[!missing_target, ]
                        # nocov end
                      }
                      missing_id = is.na(data_frame[ , ind_col])
                      if (any(missing_id)) {
                        # nocov start
                        data_frame = data_frame[!missing_id, ]
                        # nocov end
                      }
                      super$initialize(id = id,
                                       ind_col = training$getIndCol(),
                                       data_frame = data_frame)
                      private$target = training$getTarget()
                      # Add object to ht
                      training$add2HashTable(key = private$id,
                                             value = self,
                                             .class = "Target")
                      if (any(missing_target)) {
                        # nocov start
                        warning(sprintf("%s individual(s) with missing target value(s) recognized and removed.\n",
                                        sum(missing_target)))
                        # nocov end
                      }
                      if (any(missing_id)) {
                        # nocov start
                        warning(sprintf("%s individual(s) with missing ID value(s) recognized and removed.\n",
                                        sum(missing_id)))
                        # nocov end
                      }
                    },
                    #' @description
                    #' Printer
                    #' @param ... `any`
                    #'
                    # nocov start
                    print = function (...) {
                      cat(sprintf("Training  : %s\n", private$training$getId()))
                      cat(sprintf("ind. id.  : %s\n", private$ind_col))
                      cat(sprintf("target    : %s\n", private$target))
                      cat(sprintf("n         : %s\n", nrow(private$data_frame)))
                      cat(sprintf("Missing   : %s\n", sum(!complete.cases(private$data_frame))))
                    },
                    # nocov end
                    #' @description
                    #' Summary
                    #' @param ... `any`
                    #'
                    # nocov start
                    summary = function (...) {
                      cat(sprintf("      Layer     : %s\n", private$training$getId()))
                      cat(sprintf("      Ind. id.  : %s\n", private$ind_col))
                      cat(sprintf("      Target    : %s\n", private$target))
                      cat(sprintf("      n         : %s\n", nrow(private$data_frame)))
                      cat(sprintf("      Missing   : %s\n", sum(!complete.cases(private$data_frame))))
                    },
                    # nocov end
                    #' @description
                    #' Getter of the current \code{data.frame} wihtout individual
                    #' ID nor target variables.
                    #'
                    #' @return
                    #' The \code{data.frame} without individual ID nor target variables is returned.
                    #'
                    # nocov start
                    getData = function () {
                      return(private$data_frame)
                    },
                    # # nocov end
                    #' @description
                    #' Getter of target values stored on the current training layer.
                    #'
                    #' @return
                    #' The observed target values stored on the current training layer are returned.
                    #' @export
                    #'
                    # nocov start
                    getTargetValues = function () {
                      return(private$data_frame[[private$target]])
                    },
                    # nocov end
                    #' @description
                    #' Getter of the target variable name.
                    #'
                    #' @export
                    #'
                    getTargetName = function () {
                      # nocov start
                      return(private$target)
                      # nocov end
                    },
                    #' @description
                    #' Getter of the current training object.
                    #'
                    #' @return
                    #' The training layer (from class [Training]) on which the current train data are stored
                    #' is returned.
                    #' @export
                    #'
                    getTraining = function () {
                      # nocov start
                      return(private$training)
                      # nocov end
                    },
                    # nocov end
                    #' @description
                    #' Getter of the current \code{data.frame} wihtout individual
                    #' ID nor target variables.
                    #' @param data_frame `data.frame` \cr
                    #' data.frame to be set.
                    #'
                    # nocov start
                    #' Title
                    #'
                    setData = function (data_frame) {
                      private$data_frame = data_frame
                      invisible(self)
                    }
                  ),
                  private = list(
                    target = character(0L),
                    training = NULL
                  ),
                  cloneable = TRUE
)

#' @title Testing Class
#'
#' @description
#' This is a primary classes of fuseMLR. An object from this class
#' is designed to contain multiple layers, but only one new meta layer.
#'
#'  A Testing object is structured as followed:
#' * [TestLayer]
#' * [TestMetaLayer]
#'
#' @export
#'
#' @importFrom R6 R6Class
#'
#' @seealso [TrainLayer]
Testing <- R6Class("Testing",
                 inherit = HashTable,
                 public = list(
                   #' @description
                   #' constructor
                   #'
                   #' @param id `character`\cr
                   #' Testing id.
                   #' @param ind_col `character`
                   #' Name of column of individuals IDS in testing data.frame.
                   #' @param verbose `boolean` \cr
                   #' Warning messages will be displayed if set to TRUE.
                   initialize = function (id, ind_col, verbose = TRUE) {
                     super$initialize(id = id)
                     private$ind_col = ind_col
                     layers = self$getKeyClass()
                     layers = layers[layers$class %in% "TestLayer", ]
                     nb_layers = nrow(layers)
                     # nocov start
                     if (nb_layers) {
                       layer_dims = NULL
                       for (k in layers$key) {
                         layer = self$getFromHashTable(key = k)
                         test_data = layer$getTestData()
                         layer_dim = ncol(test_data$getData())
                         layer_dims = c(layer_dims, layer_dim)
                       }
                       layer_dims = paste0(layer_dims, collapse = " | ")
                       cat(sprintf("p               : %s\n", layer_dims))
                     }
                     # nocov end
                   },
                   #' @description
                   #' Printer
                   #'
                   #' @param ... `any`
                   #'
                   print = function (...) {
                     nb_layers = length(private$hash_table)
                     cat(sprintf("Testing         : %s\n", private$id))
                     cat(sprintf("Number of layers: %s\n", nb_layers))
                     layers = self$getKeyClass()
                     layers = layers[layers$class %in% "TestLayer", ]
                     nb_layers = nrow(layers)
                     if (nb_layers) {
                       layer_ps = NULL
                       layer_ns = NULL
                       for (k in layers$key) {
                         layer = self$getFromHashTable(key = k)
                         test_data = layer$getTestData()
                         layer_p = ncol(test_data$getData())
                         layer_n = nrow(test_data$getData())
                         max_width <- max(nchar(layer_p), nchar(layer_n))
                         layer_ps = c(layer_ps, format(layer_p, width = max_width, justify = "left"))
                         layer_ns = c(layer_ns, format(layer_n, width = max_width, justify = "left"))
                       }
                       layer_ps = paste0(layer_ps, collapse = " | ")
                       layer_ns = paste0(layer_ns, collapse = " | ")
                       cat(sprintf("p               : %s\n", layer_ps))
                       cat(sprintf("n               : %s\n", layer_ns))
                     }
                   },
                   #' @description
                   #' Gather individual IDs from all layer.
                   #'
                   #' @return
                   #' A \code{data.frame} containing individuals IDs.
                   #' @export
                   #'
                   getIndIDs = function() {
                     layers = self$getKeyClass()
                     # This code accesses each layer (except the MetaLayer) level
                     # and get the target variable
                     layers = layers[layers$class %in% "TestLayer", ]
                     ids_data = NULL
                     current_data = NULL
                     for (k in layers$key) {
                       new_layer = self$getFromHashTable(key = k)
                       ids_data = as.data.frame(rbind(ids_data,
                                                      new_layer$getIndIDs()))
                     }
                     ids_data = ids_data[!duplicated(ids_data[ , 1L]), ,
                                         drop = FALSE]
                     return(ids_data)
                   },
                   #' @description
                   #' Getter of the meta layer.
                   #'
                   #' @return
                   #' Object from class [TestMetaLayer]
                   #' @export
                   #'
                   getTestMetaLayer = function () {
                     testing_layers = self$getKeyClass()
                     testing_meta_layer_key = testing_layers[testing_layers$class == "TestMetaLayer" ,
                                                     "key"]
                     testing_meta_layer = self$getFromHashTable(key = testing_meta_layer_key)
                     return(testing_meta_layer)
                   },
                   #' @description
                   #' Getter of the individual column name.
                   #' @export
                   getIndCol = function () {
                     return(private$ind_col)
                   },
                   #' @description
                   #' Getter of the verbose setting.
                   #' @export
                   getVerbose = function () {
                     return(private$verbose)
                   },
                   #' @description
                   #' Retrieve modality-specific prediction data.
                   #'
                   #' @return
                   #' A \code{list} containing all (base and meta) models.
                   #' @export
                   #'
                   # nocov start
                   getData = function() {
                     layers = self$getKeyClass()
                     layers = layers[layers$class %in% "TestLayer", ]
                     all_data = list()
                     for (k in layers$key) {
                       layer = self$getFromHashTable(key = k)
                       all_data[[layer$getId()]] = layer$getTestData()$getDataFrame()
                     }
                     return(all_data)
                   },
                   # nocov end
                   #' @description
                   #' UpSet plot to show an overview of the overlap of individuals across various layers.
                   #'
                   #' @param ... `any` \cr
                   #' Further parameters to be passed to the the \code{upset} function from package \code{UpSetR}.
                   #'
                   #' @export
                   #'
                   upset = function (...) {
                     layers = self$getKeyClass()
                     # This code accesses each layer (except TrainMetaLayer) level
                     # and get the individual IDs.
                     layers = layers[layers$class %in% "TestLayer", ]
                     if (!nrow(layers) | (nrow(layers) == 1L)) {
                       stop("No or only one available layer in this training object.")
                     }
                     ids_list = lapply(layers$key, function (k) {
                       layer = self$getFromHashTable(key = k)
                       return(layer$getIndIDs()[ , 1L])
                     })
                     param_upset = list(...)
                     from_list_ids = do.call(eval(parse(text = "UpSetR::fromList")),
                                             list(input = ids_list))
                     names(from_list_ids) = layers$key
                     param_upset$data = from_list_ids
                     print(do.call(eval(parse(text = "UpSetR::upset")),
                                   param_upset))
                     invisible(TRUE)
                   },
                   #' @description
                   #' Generate testing summary
                   #'
                   #' @export
                   #'
                   summary = function () {
                     cat(sprintf("Testing %s\n", self$getId()))
                     cat("----------------\n")
                     self$print()
                     cat("----------------\n")
                     cat("\n")
                     layers = self$getKeyClass()
                     for (k in layers$key) {
                       layer = self$getFromHashTable(key = k)
                       layer$summary()
                       cat("\n")
                     }
                   }
                 ),
                 private = list(
                   ind_col = character(0L),
                   verbose = TRUE
                 ),
                 cloneable = FALSE
)

data("multi_omics")
test_that("TestData: complement tests", {
  # Testing and layers with wrong ID column
  testing_wrong_id <- Testing$new(id = "testing", ind_col = "IDSS")
  nl_ge <- TestLayer$new(id = "geneexpr", testing = testing_wrong_id)
  expect_error({
    testing_data_ge <- TestData$new(id = "geneexpr",
                               new_layer = nl_ge,
                               data_frame = multi_omics$testing$geneexpr)
  })
  # Prepare testing and layers with correct ID column
  testing <- Testing$new(id = "testing", ind_col = "IDS")
  nl_ge <- TestLayer$new(id = "geneexpr", testing = testing)
  expect_error({
    testing_data_ge <- TestData$new(id = "geneexpr",
                               new_layer = "layer",
                               data_frame = multi_omics$testing$geneexpr)
  })
  expect_no_error({
    testing_data_ge <- TestData$new(id = "geneexpr",
                               new_layer = nl_ge,
                               data_frame = multi_omics$testing$geneexpr)
    testing_data_ge <- TestData$new(id = "geneexpr",
                               new_layer = nl_ge,
                               data_frame = multi_omics$testing$geneexpr)
  })
})

---
title: "How does fuseMLR work?"
author: "Cesaire Fouodo"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{How does fuseMLR work?}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  toc: true
  toc_depth: 2
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{css, echo=FALSE}
pre {
  max-height: 300px;
  overflow-y: auto;
}

pre[class] {
  max-height: 400px;
}
```
## A - Introduction

The R package **fuseMLR** offers a framework for late integrative predictive modeling using multi-omics data. 
This vignette serves as a user guide to use the package effectively.
Three key functionalities are provided: variable selection for modality-specific data, late integrative multi-omics training, and predictions for testing multi-omics datasets. 

## B - Data

```{r libraries, warning = FALSE}
library(fuseMLR)
```

The following example is based on multi-omics simulated data available in `fuseMLR`. Data have been simulated using the R package `InterSIM`, version 2.2.0. Two types of data were simulated: training and testing datasets. Each consists of four `data.frame`s—gene expression, protein expression, methylation data modalities, and targeted observations. Individuals are organized in rows, variables in columns, with an additional column for individual IDs. In total, $70$ individuals with $50$ (not necessarily overlapping) individuals pro layer have been simulated for training, and $23$ ($20$ per layer) for testing. Effects have been introduced for across data modalities by shifting the means by $0.5$ to create case-control study with $50$% prevalence. For illustration, the number of variables was kept smaller than what is typically expected in reality. The data simulation code is available [here](https://github.com/imbs-hl/fuseMLR/blob/master/test_code/build_data.R).

```{r data_exam, include=TRUE, eval=TRUE}
data("multi_omics")
# This list contains two lists of data: training and test.
# Each sublist contains three omics data as described above.
str(object = multi_omics, max.level = 2L)
```

## C - Training #

The training process is handled in **fuseMLR** by a class called **Training**. We will use ```training``` to refer to an instance from this class. Function ```createTraining``` is available to create an empty (without layers) ```training```.

### C.1 - Creating a training ##

```{r training, include=TRUE, eval=TRUE}
training <- createTraining(id = "training",
                           ind_col = "IDS",
                           target = "disease",
                           target_df = multi_omics$training$target,
                           verbose = TRUE)
print(training)
```

A ```training``` contains modality-specific training layers (instances of the **TrainLayer** class) and a meta-layer (instance of the **TrainMetaLayer** class). The modality-specific training layers encapsulate training data modalities, variable selection functions and learners (i.e. ```R``` functions implementing statistical predicting methods). The meta-layer encapsulates the meta-learner. The terms modality-specific and layer-specific are used interchangeably as synonyms in the following.

Three main functions are necessary to create the ```training```: ```createTraining()```, ```createTrainLayer()``` and ```createTrainMetaLayer()```. ```createTraining()``` creates an empty ```training```, on which modalitity-specific training layers are added using the function ```createTrainLayer()```. Function ```createTrainMetaLayer()``` is used to add meta-layer to ```training```. 

The following code adds a gene expression, a protein abundance, and a methylation layer to ```training```. For illustration purpose we use the same variable selection method `Boruta` and the same learner (`ranger`) for all layers. These  functions fulfill `fuseMLR` requirements in terms of arguments and outputs (see `createTrainLayer` documentation for details). We expect the variable selection function to accept two arguments: `x` (the predictor design matrix) and `y` (the response vector). The function must return a vector of selected variables. Methods that do not follow this format -- such as Vita (via `ranger`) or Lasso (via `glmnet`) -- require additional wrapping or a custom function to extract the selected variables, for example, based on a significance threshold. An exception, however, is made for the Boruta function, for which an internal adjustment is implemented; its use requires no further modifications. For learners, the arguments `x` and `y` are mandatory as well, and the resulting model must support the generic `predict` function. Predictions should be returned either as a vector or a list with a `predictions` field containing the predicted values (also a vector). If the function does not meet the required input and output criteria, users should follow the steps in [E-Interface and wrapping](#e---interface-and-wrapping) to create an interface or wrap their function for use with `fuseMLR`.


```{r geneexpr, include=TRUE, eval=TRUE}
# Create gene expression layer.
createTrainLayer(training = training,
                 train_layer_id = "geneexpr",
                 train_data = multi_omics$training$geneexpr,
                 varsel_package = "Boruta",
                 varsel_fct = "Boruta",
                 varsel_param = list(num.trees = 1000L,
                                     mtry = 3L,
                                     probability = TRUE,
                                     getImp = Boruta::getImpRfZ),
                 lrner_package = "ranger",
                 lrn_fct = "ranger",
                 param_train_list = list(probability = TRUE,
                                         mtry = 1L),
                 param_pred_list = list(),
                 na_action = "na.keep")
```

```{r proteinexpr, include=TRUE, eval=TRUE}
# Create gene protein abundance layer
createTrainLayer(training = training,
                 train_layer_id = "proteinexpr",
                 train_data = multi_omics$training$proteinexpr,
                 varsel_package = "Boruta",
                 varsel_fct = "Boruta",
                 varsel_param = list(num.trees = 1000L,
                                     mtry = 3L,
                                     probability = TRUE,
                                     getImp = Boruta::getImpRfZ),
                 lrner_package = "ranger",
                 lrn_fct = "ranger",
                 param_train_list = list(probability = TRUE,
                                         mtry = 1L),
                 param_pred_list = list(type = "response"),
                 na_action = "na.keep")
```

```{r methylation, include=TRUE, eval=TRUE}
# Create methylation layer
createTrainLayer(training = training,
                 train_layer_id = "methylation",
                 train_data = multi_omics$training$methylation,
                 varsel_package = "Boruta",
                 varsel_fct = "Boruta",
                 varsel_param = list(num.trees = 1000L,
                                     mtry = 3L,
                                     probability = TRUE,
                                     getImp = Boruta::getImpRfZ),
                 lrner_package = "ranger",
                 lrn_fct = "ranger",
                 param_train_list = list(probability = TRUE,
                                         mtry = 1L),
                 param_pred_list = list(),
                 na_action = "na.keep")
```

Also add a meta-layer. We use the weighted mean (internal function to `fuseMLR`) as meta-learner. Similarly to learners, a meta-learner should allow at least the arguments `x` and `y` to pass the design matrix of predictors and should return a model that allow to call the generic function `predict` to make predictions for a new dataset (see documentation of `createTrainMetaLayer` for details). If these criteria are not fulfilled, the explanation provided in [E-Interface and wrapping](#e---interface-and-wrapping) details how to map the `x` and `y` to the original argument names or to wrap the original function. In [Appendix](#appendix) we provide information about available meta-learners.

```{css, echo=FALSE}
.scroll-100 {
  max-height: 400px;
  overflow-y: auto;
  background-color: inherit;
}
```

We use the weighted mean as the meta-learner. Weighted learners allow the meta-model to be more robust against outliers or noisy data. By adjusting the weights, it can downplay the influence of less reliable models or predictions, thus making the system more stable in the face of unpredictable data. Theoretically, we do not expect this meta-learner to outperform all the modality-specific learners but rather to achieve a performance level between the worst and the best of the modality-specific learners. 

```{r training_meta_layers, include=TRUE, eval=TRUE, class.output="scroll-100"}
# Create meta layer with imputation of missing values.
createTrainMetaLayer(training = training,
                     meta_layer_id = "meta_layer",
                     lrner_package = NULL,
                     lrn_fct = "weightedMeanLearner",
                     param_train_list = list(),
                     param_pred_list = list(na_rm = TRUE),
                     na_action = "na.rm")
print(training)
```

Function ```upsetplot()``` is available to generate an upset of the training data, i.e. an overview how patients overlap across layers.

```{r upsetplot, include=TRUE, eval=TRUE, fig.width=5, fig.align='center'}
upsetplot(object = training, order.by = "freq")
```

### C.2 - Variable selection ##

Function ```varSelection()``` performs modality-specific variable selection. So, a user can opt to conduct variable selection separately without training.

```{r varsel, include=TRUE, eval=TRUE, warning=FALSE}
# Variable selection
set.seed(5467)
var_sel_res <- varSelection(training = training)
print(var_sel_res)
```

Let us display the training object again to see the update on variable level.

```{r training_varsel_disp, include=TRUE, eval=TRUE, warning=FALSE}
print(training)
```

For each layer, the variable selection results show the chosen variables.

### C.2 - Train ##

We use the function ```fusemlr()``` to train our models using the subset of selected variables. Here we set `use_var_sel = TRUE` to use variables obtained from the variable selection step.

```{r lrner_train, include=TRUE, eval=TRUE, message=TRUE}
set.seed(5462)
fusemlr(training = training,
        use_var_sel = TRUE)
```

The display of the training object now updates information about the trained layers.

```{r display_lrner_trained, include=TRUE, eval=TRUE, message=TRUE}
print(training)
```

We can also display a summary of `training` to see more details on layer levels. Information about the training data modality, the variable selection method and the learner stored at each layer are displayed. 

```{r training_summary, include=TRUE, eval=TRUE, message=TRUE}
summary(training)
```

We use `extractModel()` to retrieve the list of stored models and `extractData()` to retrieve training data.

```{r basic_lrnr, include=TRUE, eval=TRUE}
models_list <- extractModel(training = training)
str(object = models_list, max.level = 1L)
```

Three random forests and one weighted meta-model trained on each layer are returned. The smallest weight is assigned to protein abundance, while the highest is given to gene expression.

```{r basic_data, include=TRUE, eval=TRUE}
data_list <- extractData(object = training)
str(object = data_list, max.level = 1)
```

The three simulated training modalities and the meta-data are returned.

## D - Predicting #

In this section, we create a ```testing``` instance (from the *Testing* class) and make predictions for new data. This is done analogously to ```training```. Only the testing data modalities are required. Relevant functions are ```createTesting()``` and ```createTestLayer()```.

```{r testing, include=TRUE, eval=TRUE}
# Create testing for predictions
testing <- createTesting(id = "testing",
                         ind_col = "IDS")
```

```{r testing_ge, include=TRUE, eval=TRUE}
# Create gene expression layer
createTestLayer(testing = testing,
                test_layer_id = "geneexpr",
                test_data = multi_omics$testing$geneexpr)
```

```{r testing_pr, include=TRUE, eval=TRUE}
# Create gene protein abundance layer
createTestLayer(testing = testing,
                test_layer_id = "proteinexpr",
                test_data = multi_omics$testing$proteinexpr)
```

```{r testing_me, include=TRUE, eval=TRUE}
# Create methylation layer
createTestLayer(testing = testing,
                test_layer_id = "methylation",
                test_data = multi_omics$testing$methylation)
```

A summary of `testing`.

```{r testing_summary, include=TRUE, eval=TRUE, message=TRUE}
summary(testing)
```

A look on testing data.

```{r basic_test_data, include=TRUE, eval=TRUE}
data_list <- extractData(object = testing)
str(object = data_list, max.level = 1)
```

An upset plot to visualize patient overlap across testing layers.

```{r upsetplot_new, include=TRUE, eval=TRUE, fig.width=5, fig.align='center'}
upsetplot(object = testing, order.by = "freq")
```

Function ```predict()``` is available for predicting.

```{r new_pred, include=TRUE, eval=TRUE}
predictions <- predict(object = training, testing = testing)
print(predictions)
```

Prediction performances for layer-specific levels and the meta-layer are estimated. Brier Score (BS) is utilized to assess calibration performance and the Area Under the Curve (AUC) to evaluate classification accuracy.

```{r brier, include=TRUE, eval=TRUE, message = FALSE}
pred_values <- predictions$predicted_values
actual_pred <- merge(x = pred_values,
                     y = multi_omics$testing$target,
                     by = "IDS",
                     all.y = TRUE)
y <- as.numeric(actual_pred$disease == "1")

# On all patients
perf_bs <- sapply(X = actual_pred[ , 2L:5L], FUN = function (my_pred) {
  bs <- mean((y[complete.cases(my_pred)] - my_pred[complete.cases(my_pred)])^2)
  roc_obj <- pROC::roc(y[complete.cases(my_pred)], my_pred[complete.cases(my_pred)])
  auc <- pROC::auc(roc_obj)
  performances = rbind(bs, auc)
  return(performances)
})
rownames(perf_bs) <- c("BS", "AUC")
print(perf_bs)
```

As expected, the performance of the meta-learner in terms of Brier Score is not the best; it falls between the worst and best modality-specific performance measures. For AUC, the meta-learner performs as well as the best modality-specific learner. We observe that performance on protein abundance is the worst, consistent with its lowest assigned weight. There is a reversal in the weight order between the methylation layer and gene expression. However, the performance difference between these two modalities is relatively small, approximately $0.05$.

## E - Interface and wrapping

This section explains how to resolve argument discrepancies when the original function does not conform to the `fuseMLR` format introduced in [C.1 - Creating a training](#c.1---creating-a-training). These discrepancies can occur in input (argument names) or output formats of the user-provided functions.

At the input level, we distinguish common supervised learning arguments from method specific arguments. The common arguments are a matrix ```x``` of independent variables and  ```y``` representing a response variable. If the provided original function (variable selection or learning function) does not have these two arguments, then discrepancies must be resolved. Moreover, the ```predict``` function extending the generic `R` predict function must allow arguments ```object``` and ```data```. If this is not the case, discrepancies must be resolved.

The provided learner must return a model compatible with an extension of the generic `predict` function (e.g., `predict.glmnet` for `glmnet` models). The `predict` function should return either a vector of predictions or a `list` with a `predictions` field (a vector of predicted values). For binary classification (classes $0$ and $1$), learners can return a two-column `matrix` or `data.frame` of class probabilities, where the second column represents the probability of class $1$ (used by `fuseMLR`) and the first column that of $0$ (ignored by `fuseMLR`). For variable selection, the output should be a vector of selected variables. If these criteria are not met, discrepancies must be resolved.

We offer two main ways to resolve discrepancies: either using an interface or a wrapping of the original function.

### Interface ##

The interface approach maps the argument names of the original learning function using arguments of `createTrainLayer()`. In the example below, the gene expression layer is recreated using the `svm` function from the `e1071` package as the learner. A discrepancy arises because `predict.svm` uses `object` and `newdata` as argument names. We also provide a function using the argument `extract_pred_fct` of `createTrainLayer` to extract the predicted values. Similar arguments are also available for the `createTrainMetaLayer` function to generate meta-layer.

```{r interface, include=TRUE, eval=FALSE}
# Re-create the gene expression layer with support vector machine as learner.
createTrainLayer(training = training,
                 train_layer_id = "geneexpr",
                 train_data = multi_omics$training$geneexpr,
                 varsel_package = "Boruta",
                 varsel_fct = "Boruta",
                 varsel_param = list(num.trees = 1000L,
                                     mtry = 3L,
                                     probability = TRUE,
                                     getImp = Boruta::getImpRfZ),
                 lrner_package = "e1071",
                 lrn_fct = "svm",
                 param_train_list = list(type = 'C-classification',
                                         kernel = 'radial',
                                         probability = TRUE),
                 param_pred_list = list(probability = TRUE),
                 na_action = "na.rm",
                 x_lrn = "x",
                 y_lrn = "y",
                 object = "object",
                 data = "newdata", # Name discrepancy resolved.
                 extract_pred_fct = function (pred) { 
                   pred <- attr(pred, "probabilities")
                   return(pred[ , 1L])
                 }
)
# Variable selection
set.seed(5467)
var_sel_res <- varSelection(training = training)
set.seed(5462)
training <- fusemlr(training = training,
                    use_var_sel = TRUE)

print(training)
```

### Wrapping

In the wrapping approach we define a new function ```mylasso``` to run a Lasso regression from the ```glmnet``` package as the meta-leaner.

- Wrapping of glmnet.

```{r wrap_lasso, include=TRUE, eval=FALSE}
# We wrap the original functions
mylasso <- function (x, y,
                     nlambda = 25,
                     nfolds = 5) {
  # Perform cross-validation to find the optimal lambda
  cv_lasso <- glmnet::cv.glmnet(x = as.matrix(x), y = y,
                        family = "binomial",
                        type.measure = "deviance",
                        nfolds = nfolds)
  best_lambda <- cv_lasso$lambda.min
  lasso_best <- glmnet::glmnet(x = as.matrix(x), y = y,
                       family = "binomial",
                       alpha = 1,
                       lambda = best_lambda
  )
  lasso_model <- list(model = lasso_best)
  class(lasso_model) <- "mylasso"
  return(lasso_model)
}
```

- Extension of `predict`.

```{r wrap_predict, include=TRUE, eval=FALSE}
# We extend the generic predict function mylasso. 
predict.mylasso <- function (object, data) {
  glmnet_pred <- predict(object = object$model,
                         newx = as.matrix(data),
                         type = "response",
                         s = object$model$lambda)
  return(as.vector(glmnet_pred))
}

# Re-create the gene expression layer with support vector machine as learner.
createTrainMetaLayer(training = training,
                     meta_layer_id = "meta_layer",
                     lrner_package = NULL,
                     lrn_fct = "mylasso",
                     param_train_list = list(nlambda = 100L),
                     na_action = "na.impute")
set.seed(5462)
training <- fusemlr(training = training,
                    use_var_sel = TRUE)
print(training)
```

- Re-set the meta-layer with the wrapped learner and train.

```{r add_wrap, include=TRUE, eval=FALSE}
# Re-create the gene expression layer with support vector machine as learner.
createTrainMetaLayer(training = training,
                     meta_layer_id = "meta_layer",
                     lrner_package = NULL,
                     lrn_fct = "mylasso",
                     param_train_list = list(nlambda = 100L),
                     na_action = "na.impute")
set.seed(5462)
training <- fusemlr(training = training,
                    use_var_sel = TRUE)
print(training)
```

## Appendix

In addition to any pre-existing learner in R as a meta-learner, we have implemented the following ones.

```{r implemented_learners, include=TRUE, echo=FALSE}
# Load knitr package
library(knitr)

# Create a data frame
data <- data.frame(
  Leaner = c("weightedMeanLearner", "bestLayerLearner", "cobra"),
  Description = c("The weighted mean meta learner. It uses modality-specific predictions to estimate the weights of the modality-specific models", "The best layer-specific model is used as meta model.", "cobra implements the COBRA (COmBined Regression Alternative), an aggregation method for combining predictions from multiple individual learners")
)

# Generate the table
kable(data, caption = "")
```

## References

- Fouodo, C. J. K., Bleskina, M. & Szymczak, S. fuseMLR: an R package for integrative prediction modeling of multi-omics data. BMC Bioinformatics 26, 221 (2025). https://doi.org/10.1186/s12859-025-06248-4

- Wright MN, Ziegler A.. Ranger: a fast implementation of random forests for high dimensional data in C ++ and R. J Stat Softw 2017;77:1–17, [https://doi.org/10.18637/jss.v077.i01](https://doi.org/10.18637/jss.v077.i01).

- Tay JK, Narasimhan B, Hastie T.. (2023). Elastic Net Regularization Paths for All Generalized Linear Models. Journal of Statistical Software, 106(1), 1-31, [https://doi.org/10.18637/jss.v106.i01](https://doi.org/10.18637/jss.v106.i01). 

- Kursa M, Rudnicki W.. Feature selection with the Boruta package. J Stat Softw 2010;36:1–13, [https://doi.org/10.18637/jss.v036.i11](https://doi.org/10.18637/jss.v036.i11).

- Janitza S, Celik E, Boulesteix A-L.. A computationally fast variable importance test for random forests for high-dimensional data. Adv Data Anal Classif 2016, in press, [https://doi.org/10.1007/s11634-016-0276-4](https://doi.org/10.1007/s11634-016-0276-4)

- Biau, G., Fischer, A., Guedj, B., & Malley, J. D. (2014). COBRA: A combined regression strategy. The Journal of Multivariate Analysis 46:18-28, [https://doi.org/10.1016/j.jmva.2015.04.007](https://doi.org/10.1016/j.jmva.2015.04.007).


&copy; 2024 Institute of Medical Biometry and Statistics (IMBS). All rights reserved.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Training.R
\name{Training}
\alias{Training}
\title{Training Class}
\description{
This is a primary classes of fuseMLR. An object from this class
is designed to contain multiple training layers, but only one meta training layer.

The Training class is structured as followed:
\itemize{
\item \link{TrainLayer}: Specific layer containing:
\itemize{
\item \link{Lrner}: Specific learner. This must be set by the user.
\item \link{TrainData}: Specific training dataset. This must be set up by the user.
\item \link{Model}: Specific model. This is set up by training the learner on the training data.
}
\item \link{TrainMetaLayer}: Basically a \link{TrainLayer}, but with some specific properties.
\itemize{
\item \link{Lrner}: This is the meta learner, it must be set up by the user.
\item \link{TrainData}: Specific modality-specific prediction data. This is set up internally after cross-validation.
\item \link{Model}: Specific meta model. This is set up by training the learner on the training data.
}
}

Use the function \code{train} for training and \code{predict} for predicting.
}
\seealso{
\link{TrainLayer}

\link{Testing} and \link{Predicting}
}
\section{Super class}{
\code{\link[fuseMLR:HashTable]{fuseMLR::HashTable}} -> \code{Training}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Training-new}{\code{Training$new()}}
\item \href{#method-Training-print}{\code{Training$print()}}
\item \href{#method-Training-trainLayer}{\code{Training$trainLayer()}}
\item \href{#method-Training-predictLayer}{\code{Training$predictLayer()}}
\item \href{#method-Training-createMetaTrainData}{\code{Training$createMetaTrainData()}}
\item \href{#method-Training-train}{\code{Training$train()}}
\item \href{#method-Training-predict}{\code{Training$predict()}}
\item \href{#method-Training-varSelection}{\code{Training$varSelection()}}
\item \href{#method-Training-getTargetValues}{\code{Training$getTargetValues()}}
\item \href{#method-Training-getIndIDs}{\code{Training$getIndIDs()}}
\item \href{#method-Training-getLayer}{\code{Training$getLayer()}}
\item \href{#method-Training-getTrainMetaLayer}{\code{Training$getTrainMetaLayer()}}
\item \href{#method-Training-getModel}{\code{Training$getModel()}}
\item \href{#method-Training-getData}{\code{Training$getData()}}
\item \href{#method-Training-removeLayer}{\code{Training$removeLayer()}}
\item \href{#method-Training-removeTrainMetaLayer}{\code{Training$removeTrainMetaLayer()}}
\item \href{#method-Training-getIndCol}{\code{Training$getIndCol()}}
\item \href{#method-Training-getTarget}{\code{Training$getTarget()}}
\item \href{#method-Training-getVerbose}{\code{Training$getVerbose()}}
\item \href{#method-Training-getUseVarSel}{\code{Training$getUseVarSel()}}
\item \href{#method-Training-getVarSelDone}{\code{Training$getVarSelDone()}}
\item \href{#method-Training-increaseNbTrainedLayer}{\code{Training$increaseNbTrainedLayer()}}
\item \href{#method-Training-checkTargetExist}{\code{Training$checkTargetExist()}}
\item \href{#method-Training-getTargetObj}{\code{Training$getTargetObj()}}
\item \href{#method-Training-getProblemTyp}{\code{Training$getProblemTyp()}}
\item \href{#method-Training-setImpute}{\code{Training$setImpute()}}
\item \href{#method-Training-testOverlap}{\code{Training$testOverlap()}}
\item \href{#method-Training-upset}{\code{Training$upset()}}
\item \href{#method-Training-summary}{\code{Training$summary()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="add2HashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-add2HashTable'><code>fuseMLR::HashTable$add2HashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="checkClassExist"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-checkClassExist'><code>fuseMLR::HashTable$checkClassExist()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getFromHashTable'><code>fuseMLR::HashTable$getFromHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getHashTable'><code>fuseMLR::HashTable$getHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getId"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getId'><code>fuseMLR::HashTable$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getKeyClass"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getKeyClass'><code>fuseMLR::HashTable$getKeyClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="removeFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-removeFromHashTable'><code>fuseMLR::HashTable$removeFromHashTable()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-new"></a>}}
\if{latex}{\out{\hypertarget{method-Training-new}{}}}
\subsection{Method \code{new()}}{
constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$new(
  id,
  ind_col,
  target,
  target_df,
  problem_type = "classification",
  verbose = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr}

\item{\code{ind_col}}{\code{character} \cr
Name of column of individuals IDS.}

\item{\code{target}}{\code{character} \cr
Name of the target variable.}

\item{\code{target_df}}{\code{data.frame} \cr
Data frame with two columns: individual IDs and response variable values.}

\item{\code{problem_type}}{\code{character} \cr
Either "classification" or "regression".}

\item{\code{verbose}}{\code{boolean} \cr
Warning messages will be displayed if set to TRUE.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-print"></a>}}
\if{latex}{\out{\hypertarget{method-Training-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-trainLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Training-trainLayer}{}}}
\subsection{Method \code{trainLayer()}}{
Train each layer of the current Training.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$trainLayer(ind_subset = NULL, use_var_sel = FALSE, verbose = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ind_subset}}{\code{character}\cr
Subset of individuals IDs to be used for training.}

\item{\code{use_var_sel}}{\code{boolean} \cr
If TRUE, selected variables available at each layer are used.}

\item{\code{verbose}}{\code{boolean} \cr
Warning messages will be displayed if set to TRUE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Returns the object itself, with a model for each layer.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-predictLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Training-predictLayer}{}}}
\subsection{Method \code{predictLayer()}}{
Predicts values given new data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$predictLayer(testing, ind_subset = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{testing}}{\code{TestData} \cr
Object of class \link{TestData}.}

\item{\code{ind_subset}}{\code{vector} \cr
Subset of individuals IDs to be used for training.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new \link{Training} with predicted values for each layer.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-createMetaTrainData"></a>}}
\if{latex}{\out{\hypertarget{method-Training-createMetaTrainData}{}}}
\subsection{Method \code{createMetaTrainData()}}{
Creates a meta training dataset and assigns it to the meta layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$createMetaTrainData(
  resampling_method,
  resampling_arg,
  use_var_sel,
  impute = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{resampling_method}}{\code{function} \cr
Function for internal validation.}

\item{\code{resampling_arg}}{\code{list} \cr
List of arguments to be passed to the function.}

\item{\code{use_var_sel}}{\code{boolean} \cr
If TRUE, selected variables available at each layer are used.}

\item{\code{impute}}{\code{boolean} \cr
If TRUE, mode or median based imputation is performed on the modality-specific predictions.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The current object is returned, with a meta training dataset assigned to the meta layer.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-train"></a>}}
\if{latex}{\out{\hypertarget{method-Training-train}{}}}
\subsection{Method \code{train()}}{
Trains the current object. All leaners and the meta learner are trained.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$train(
  ind_subset = NULL,
  use_var_sel = FALSE,
  resampling_method = NULL,
  resampling_arg = list(),
  seed = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ind_subset}}{\code{vector} \cr
ID subset to be used for training.}

\item{\code{use_var_sel}}{\code{boolean} \cr
If TRUE, variable selection is performed before training.}

\item{\code{resampling_method}}{\code{function} \cr
Function for internal validation. If not specify, the \code{resampling} function from the package \code{caret} is used for a 10-folds cross-validation.}

\item{\code{resampling_arg}}{\code{list} \cr
List of arguments to be passed to the function.}

\item{\code{seed}}{\code{integer} \cr
Random seed. Default is NULL, which generates the seed from \code{R}.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The current object is returned, with each learner trained on each layer.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-predict"></a>}}
\if{latex}{\out{\hypertarget{method-Training-predict}{}}}
\subsection{Method \code{predict()}}{
Compute predictions for a testing object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$predict(testing, ind_subset = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{testing}}{\code{Testing} \cr
A new testing object to be predicted.}

\item{\code{ind_subset}}{\code{vector} \cr
Vector of IDs to be predicted.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The predicted object. All layers and the meta layer are predicted. This is the final predicted object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-varSelection"></a>}}
\if{latex}{\out{\hypertarget{method-Training-varSelection}{}}}
\subsection{Method \code{varSelection()}}{
Variable selection on the current training object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$varSelection(ind_subset = NULL, verbose = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ind_subset}}{\code{vector} \cr
ID subset of individuals to be used for variable selection.}

\item{\code{verbose}}{\code{boolean} \cr
Warning messages will be displayed if set to TRUE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The current layer is returned with the resulting model.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getTargetValues"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getTargetValues}{}}}
\subsection{Method \code{getTargetValues()}}{
Gather target values from all layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getTargetValues()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{data.frame} containing individuals IDs and corresponding target values.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getIndIDs"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getIndIDs}{}}}
\subsection{Method \code{getIndIDs()}}{
Gather individual IDs from all layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getIndIDs()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{data.frame} containing individuals IDs.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getLayer}{}}}
\subsection{Method \code{getLayer()}}{
Get a layer of a given ID.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getLayer(id)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
The ID of the layer to be returned.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The \link{TrainLayer} object is returned for the given ID.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getTrainMetaLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getTrainMetaLayer}{}}}
\subsection{Method \code{getTrainMetaLayer()}}{
Getter of the meta layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getTrainMetaLayer()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Object from class \link{TrainMetaLayer}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getModel"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getModel}{}}}
\subsection{Method \code{getModel()}}{
Retrieve models from all layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getModel()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{list} containing all (base and meta) models.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getData"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getData}{}}}
\subsection{Method \code{getData()}}{
Retrieve modality-specific predictions.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{list} containing all (base and meta) models.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-removeLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Training-removeLayer}{}}}
\subsection{Method \code{removeLayer()}}{
Remove a layer of a given ID.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$removeLayer(id)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
The ID of the layer to be removed.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The \link{TrainLayer} object is returned for the given ID.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-removeTrainMetaLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Training-removeTrainMetaLayer}{}}}
\subsection{Method \code{removeTrainMetaLayer()}}{
Remove the meta layer from the current \link{Training} object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$removeTrainMetaLayer()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getIndCol"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getIndCol}{}}}
\subsection{Method \code{getIndCol()}}{
Getter of the individual column name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getIndCol()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getTarget"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getTarget}{}}}
\subsection{Method \code{getTarget()}}{
Getter of the target variable name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getTarget()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getVerbose"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getVerbose}{}}}
\subsection{Method \code{getVerbose()}}{
Getter of the verbose setting.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getVerbose()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getUseVarSel"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getUseVarSel}{}}}
\subsection{Method \code{getUseVarSel()}}{
Getter of the use_var_sel field.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getUseVarSel()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getVarSelDone"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getVarSelDone}{}}}
\subsection{Method \code{getVarSelDone()}}{
Getter of the use_var_sel field.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getVarSelDone()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-increaseNbTrainedLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Training-increaseNbTrainedLayer}{}}}
\subsection{Method \code{increaseNbTrainedLayer()}}{
Increase the number of trained layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$increaseNbTrainedLayer()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-checkTargetExist"></a>}}
\if{latex}{\out{\hypertarget{method-Training-checkTargetExist}{}}}
\subsection{Method \code{checkTargetExist()}}{
Check whether a target object has already been stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$checkTargetExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getTargetObj"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getTargetObj}{}}}
\subsection{Method \code{getTargetObj()}}{
Getter of the target object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getTargetObj()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-getProblemTyp"></a>}}
\if{latex}{\out{\hypertarget{method-Training-getProblemTyp}{}}}
\subsection{Method \code{getProblemTyp()}}{
Getter of the problem type.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$getProblemTyp()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-setImpute"></a>}}
\if{latex}{\out{\hypertarget{method-Training-setImpute}{}}}
\subsection{Method \code{setImpute()}}{
Set imputation action na.action.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$setImpute(impute)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{impute}}{\code{character} \cr
How to handle missing values.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-testOverlap"></a>}}
\if{latex}{\out{\hypertarget{method-Training-testOverlap}{}}}
\subsection{Method \code{testOverlap()}}{
Test that individuals overlap over layers.
At least five individuals must overlapped.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$testOverlap()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-upset"></a>}}
\if{latex}{\out{\hypertarget{method-Training-upset}{}}}
\subsection{Method \code{upset()}}{
UpSet plot to show an overview of the overlap of individuals across various layers.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$upset(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any} \cr
Further parameters to be passed to the \code{upset} function from package \code{UpSetR}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Training-summary"></a>}}
\if{latex}{\out{\hypertarget{method-Training-summary}{}}}
\subsection{Method \code{summary()}}{
Generate training summary
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Training$summary()}\if{html}{\out{</div>}}
}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testingFunctions.R
\name{createTestLayer}
\alias{createTestLayer}
\title{createTestLayer}
\usage{
createTestLayer(testing, test_layer_id, test_data)
}
\arguments{
\item{testing}{\code{Testing} \cr
Testing object where the created layer will be stored.}

\item{test_layer_id}{\code{character} \cr
ID of the testing layer to be created.}

\item{test_data}{\code{data.frame} \cr
Data modality to be stored in \link{TestData}.}
}
\value{
The updated \link{Testing} object (with the new layer) is returned.
}
\description{
Creates and stores a \link{TestLayer} on the \link{Testing} object passed as argument.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createLoss.R
\name{createLoss}
\alias{createLoss}
\title{Create Loss}
\usage{
createLoss(pred, target)
}
\arguments{
\item{pred}{A \code{vector} of predictions.}

\item{target}{A \code{vector} of target values.}
}
\description{
Create Loss
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainingFunctions.R
\name{extractData}
\alias{extractData}
\title{extractData}
\usage{
extractData(object)
}
\arguments{
\item{object}{\verb{Training or Testing} \cr
The object of interest.}
}
\value{
A list of data is returned.
}
\description{
Extracts data stored on each layers; base data and modality-specific predictions (for Training) are extracted.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HashTable.R
\name{HashTable}
\alias{HashTable}
\title{Class HashTable}
\description{
Hashtable to contain object modalities. Storage objects like \link{Training} and \link{TrainLayer} are extensions of this class.
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-HashTable-new}{\code{HashTable$new()}}
\item \href{#method-HashTable-print}{\code{HashTable$print()}}
\item \href{#method-HashTable-add2HashTable}{\code{HashTable$add2HashTable()}}
\item \href{#method-HashTable-getFromHashTable}{\code{HashTable$getFromHashTable()}}
\item \href{#method-HashTable-getKeyClass}{\code{HashTable$getKeyClass()}}
\item \href{#method-HashTable-removeFromHashTable}{\code{HashTable$removeFromHashTable()}}
\item \href{#method-HashTable-getId}{\code{HashTable$getId()}}
\item \href{#method-HashTable-getHashTable}{\code{HashTable$getHashTable()}}
\item \href{#method-HashTable-checkClassExist}{\code{HashTable$checkClassExist()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HashTable-new"></a>}}
\if{latex}{\out{\hypertarget{method-HashTable-new}{}}}
\subsection{Method \code{new()}}{
Initialize a default parameters list.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HashTable$new(id)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
ID of the hash table. It must be unique.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HashTable-print"></a>}}
\if{latex}{\out{\hypertarget{method-HashTable-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HashTable$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HashTable-add2HashTable"></a>}}
\if{latex}{\out{\hypertarget{method-HashTable-add2HashTable}{}}}
\subsection{Method \code{add2HashTable()}}{
Function to add a key-value pair to the hash table.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HashTable$add2HashTable(key, value, .class)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{\code{character} \cr
The key to be added.}

\item{\code{value}}{\code{object} \cr
Object to be added.}

\item{\code{.class}}{\code{character} \cr
Class of the object to be added.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HashTable-getFromHashTable"></a>}}
\if{latex}{\out{\hypertarget{method-HashTable-getFromHashTable}{}}}
\subsection{Method \code{getFromHashTable()}}{
Getter of the object which the key passed as argument.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HashTable$getFromHashTable(key)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{\code{character} \cr
Key of the required object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HashTable-getKeyClass"></a>}}
\if{latex}{\out{\hypertarget{method-HashTable-getKeyClass}{}}}
\subsection{Method \code{getKeyClass()}}{
Getter of the \code{data.frame} that stores all key class pairs.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HashTable$getKeyClass()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
\link{data.frame}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HashTable-removeFromHashTable"></a>}}
\if{latex}{\out{\hypertarget{method-HashTable-removeFromHashTable}{}}}
\subsection{Method \code{removeFromHashTable()}}{
Remove the object with the corresponding key from the hashtable.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HashTable$removeFromHashTable(key)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{Key of the object to be removed.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HashTable-getId"></a>}}
\if{latex}{\out{\hypertarget{method-HashTable-getId}{}}}
\subsection{Method \code{getId()}}{
Getter of the current object ID.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HashTable$getId()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HashTable-getHashTable"></a>}}
\if{latex}{\out{\hypertarget{method-HashTable-getHashTable}{}}}
\subsection{Method \code{getHashTable()}}{
Getter of the current hashtable.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HashTable$getHashTable()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-HashTable-checkClassExist"></a>}}
\if{latex}{\out{\hypertarget{method-HashTable-checkClassExist}{}}}
\subsection{Method \code{checkClassExist()}}{
Check whether object from a class has already been stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{HashTable$checkClassExist(.class)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{.class}}{\code{character} \cr}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Boolean value
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createCobraPred.R
\name{createCobraPred}
\alias{createCobraPred}
\title{Create COBRA Predictions}
\usage{
createCobraPred(
  train,
  test,
  n_train,
  n_test,
  nlearners,
  eps,
  alpha,
  train_target
)
}
\arguments{
\item{train}{A \code{matrix} representing the training data. Rows represent observations,
and columns contain predictions from individual learners for these observations.
In cases where a prediction is unavailable for a specific observation, \code{NA} is used.}

\item{test}{A \code{matrix} representing the test data. Rows represent observations,
and columns contain predictions from individual learners for these observations.
In cases where a prediction is unavailable for a specific observation, \code{NA} is used.}

\item{n_train}{An \code{integer} specifying the number of training observations.}

\item{n_test}{An \code{integer} specifying the number of test observations.}

\item{nlearners}{An \code{integer} representing the number of learners.}

\item{eps}{A \code{numeric} value representing the threshold for proximity between two predictions.}

\item{alpha}{A value that determines the optimal number of learners in the neighborhood (only for alpha optimization).}

\item{train_target}{A \code{vector} containing the target values for the training dataset}
}
\description{
The \code{createCobraPred} function calculates predictions by averaging the target
values of all the nearest candidates in the training dataset. Only the
training points that are within the specified proximity (\code{eps}) to the test
point are used to determine the prediction. If no suitable training points
are found, the function returns \code{NA} as the prediction.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Target.R
\name{Target}
\alias{Target}
\title{Target Class}
\description{
This class implements the target object. A \link{Target} object can only
exist as a component of a \link{Training} object.
}
\seealso{
\link{TrainLayer}, \link{Lrner}, \link{Model}, \link{TestData}
}
\section{Super class}{
\code{\link[fuseMLR:Data]{fuseMLR::Data}} -> \code{Target}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Target-new}{\code{Target$new()}}
\item \href{#method-Target-print}{\code{Target$print()}}
\item \href{#method-Target-summary}{\code{Target$summary()}}
\item \href{#method-Target-getData}{\code{Target$getData()}}
\item \href{#method-Target-getTargetValues}{\code{Target$getTargetValues()}}
\item \href{#method-Target-getTargetName}{\code{Target$getTargetName()}}
\item \href{#method-Target-getTraining}{\code{Target$getTraining()}}
\item \href{#method-Target-setData}{\code{Target$setData()}}
\item \href{#method-Target-clone}{\code{Target$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getCompleteData"><a href='../../fuseMLR/html/Data.html#method-Data-getCompleteData'><code>fuseMLR::Data$getCompleteData()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getDataFrame"><a href='../../fuseMLR/html/Data.html#method-Data-getDataFrame'><code>fuseMLR::Data$getDataFrame()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getId"><a href='../../fuseMLR/html/Data.html#method-Data-getId'><code>fuseMLR::Data$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getIndCol"><a href='../../fuseMLR/html/Data.html#method-Data-getIndCol'><code>fuseMLR::Data$getIndCol()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getIndSubset"><a href='../../fuseMLR/html/Data.html#method-Data-getIndSubset'><code>fuseMLR::Data$getIndSubset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getSetDiff"><a href='../../fuseMLR/html/Data.html#method-Data-getSetDiff'><code>fuseMLR::Data$getSetDiff()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getVarSubset"><a href='../../fuseMLR/html/Data.html#method-Data-getVarSubset'><code>fuseMLR::Data$getVarSubset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="impute"><a href='../../fuseMLR/html/Data.html#method-Data-impute'><code>fuseMLR::Data$impute()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="setDataFrame"><a href='../../fuseMLR/html/Data.html#method-Data-setDataFrame'><code>fuseMLR::Data$setDataFrame()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Target-new"></a>}}
\if{latex}{\out{\hypertarget{method-Target-new}{}}}
\subsection{Method \code{new()}}{
Initialize a new object from the current class.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Target$new(id, data_frame, training)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
The Object ID.}

\item{\code{data_frame}}{\code{data.frame}\cr
\code{data.frame} containing data.}

\item{\code{training}}{\code{Training} \cr
Training where to store the current object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Target-print"></a>}}
\if{latex}{\out{\hypertarget{method-Target-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Target$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Target-summary"></a>}}
\if{latex}{\out{\hypertarget{method-Target-summary}{}}}
\subsection{Method \code{summary()}}{
Summary
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Target$summary(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Target-getData"></a>}}
\if{latex}{\out{\hypertarget{method-Target-getData}{}}}
\subsection{Method \code{getData()}}{
Getter of the current \code{data.frame} wihtout individual
ID nor target variables.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Target$getData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The \code{data.frame} without individual ID nor target variables is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Target-getTargetValues"></a>}}
\if{latex}{\out{\hypertarget{method-Target-getTargetValues}{}}}
\subsection{Method \code{getTargetValues()}}{
Getter of target values stored on the current training layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Target$getTargetValues()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The observed target values stored on the current training layer are returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Target-getTargetName"></a>}}
\if{latex}{\out{\hypertarget{method-Target-getTargetName}{}}}
\subsection{Method \code{getTargetName()}}{
Getter of the target variable name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Target$getTargetName()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Target-getTraining"></a>}}
\if{latex}{\out{\hypertarget{method-Target-getTraining}{}}}
\subsection{Method \code{getTraining()}}{
Getter of the current training object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Target$getTraining()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The training layer (from class \link{Training}) on which the current train data are stored
is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Target-setData"></a>}}
\if{latex}{\out{\hypertarget{method-Target-setData}{}}}
\subsection{Method \code{setData()}}{
Getter of the current \code{data.frame} wihtout individual
ID nor target variables.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Target$setData(data_frame)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{data_frame}}{\code{data.frame} \cr
data.frame to be set.

Title}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Target-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Target-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Target$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictData.R
\name{PredictData}
\alias{PredictData}
\title{PredictData Class}
\description{
This class implements \link{PredictData} object to be predicted.
A \link{PredictData} object can only exist as a component of a \link{PredictLayer} or a \link{PredictMetaLayer} object.
}
\seealso{
\link{TrainData}, \link{TestData}
}
\section{Super class}{
\code{\link[fuseMLR:Data]{fuseMLR::Data}} -> \code{PredictData}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-PredictData-new}{\code{PredictData$new()}}
\item \href{#method-PredictData-print}{\code{PredictData$print()}}
\item \href{#method-PredictData-getPredictData}{\code{PredictData$getPredictData()}}
\item \href{#method-PredictData-getPredictLayer}{\code{PredictData$getPredictLayer()}}
\item \href{#method-PredictData-setPredictLayer}{\code{PredictData$setPredictLayer()}}
\item \href{#method-PredictData-clone}{\code{PredictData$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getCompleteData"><a href='../../fuseMLR/html/Data.html#method-Data-getCompleteData'><code>fuseMLR::Data$getCompleteData()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getData"><a href='../../fuseMLR/html/Data.html#method-Data-getData'><code>fuseMLR::Data$getData()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getDataFrame"><a href='../../fuseMLR/html/Data.html#method-Data-getDataFrame'><code>fuseMLR::Data$getDataFrame()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getId"><a href='../../fuseMLR/html/Data.html#method-Data-getId'><code>fuseMLR::Data$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getIndCol"><a href='../../fuseMLR/html/Data.html#method-Data-getIndCol'><code>fuseMLR::Data$getIndCol()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getIndSubset"><a href='../../fuseMLR/html/Data.html#method-Data-getIndSubset'><code>fuseMLR::Data$getIndSubset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getSetDiff"><a href='../../fuseMLR/html/Data.html#method-Data-getSetDiff'><code>fuseMLR::Data$getSetDiff()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getVarSubset"><a href='../../fuseMLR/html/Data.html#method-Data-getVarSubset'><code>fuseMLR::Data$getVarSubset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="impute"><a href='../../fuseMLR/html/Data.html#method-Data-impute'><code>fuseMLR::Data$impute()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="setDataFrame"><a href='../../fuseMLR/html/Data.html#method-Data-setDataFrame'><code>fuseMLR::Data$setDataFrame()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictData-new"></a>}}
\if{latex}{\out{\hypertarget{method-PredictData-new}{}}}
\subsection{Method \code{new()}}{
Initialize a new object from the current class.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictData$new(id, ind_col, data_frame)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
Object ID.}

\item{\code{ind_col}}{\code{character}\cr
Column name containing individual IDs.}

\item{\code{data_frame}}{\code{data.frame}\cr
\code{data.frame} containing data.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictData-print"></a>}}
\if{latex}{\out{\hypertarget{method-PredictData-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictData$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictData-getPredictData"></a>}}
\if{latex}{\out{\hypertarget{method-PredictData-getPredictData}{}}}
\subsection{Method \code{getPredictData()}}{
Getter of the current predicted \code{data.frame} wihtout individual
ID variable.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictData$getPredictData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The \code{data.frame} without individual ID nor target variables is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictData-getPredictLayer"></a>}}
\if{latex}{\out{\hypertarget{method-PredictData-getPredictLayer}{}}}
\subsection{Method \code{getPredictLayer()}}{
Getter of the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictData$getPredictLayer()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The layer (from class \link{PredictLayer}) on which the current train data are stored
is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictData-setPredictLayer"></a>}}
\if{latex}{\out{\hypertarget{method-PredictData-setPredictLayer}{}}}
\subsection{Method \code{setPredictLayer()}}{
Assigns a predicted layer to the predicted data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictData$setPredictLayer(predict_layer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{predict_layer}}{\code{PredictLayer(1)} \cr}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The current object
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictData-clone"></a>}}
\if{latex}{\out{\hypertarget{method-PredictData-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictData$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.weightedMeanLearner.R
\name{predict.weightedMeanLearner}
\alias{predict.weightedMeanLearner}
\title{Weighted mean prediction.}
\usage{
\method{predict}{weightedMeanLearner}(object, data, na_rm = FALSE, ...)
}
\arguments{
\item{object}{\code{weightedMeanLearner(1)} \cr
An object from class \link{weightedMeanLearner}}

\item{data}{\code{data.frame} \cr
\code{data.frame} to be predicted.}

\item{na_rm}{\code{boolean} \cr
Removes NAs when TRUE.}

\item{...}{\code{any} \cr
Further arguments.}
}
\value{
Predicted target values are returned.
}
\description{
Predict function for models from class \code{weightedMeanLearner}.
}
\examples{
set.seed(20240625)
x <- data.frame(x1 = rnorm(50L))
y <- sample(x = 0:1, size = 50L, replace = TRUE)
my_model <- weightedMeanLearner(x = x, y = y)
x_new <- data.frame(x1 = rnorm(10L))
my_predictions <- predict(object = my_model, data = x_new)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Lrner.R
\name{Lrner}
\alias{Lrner}
\title{Lrner Class}
\description{
This class implements a learner. A \link{Lrner} object can only exist as a component of a
\link{TrainLayer} or a \link{TrainMetaLayer} object.
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Lrner-new}{\code{Lrner$new()}}
\item \href{#method-Lrner-print}{\code{Lrner$print()}}
\item \href{#method-Lrner-summary}{\code{Lrner$summary()}}
\item \href{#method-Lrner-interface}{\code{Lrner$interface()}}
\item \href{#method-Lrner-train}{\code{Lrner$train()}}
\item \href{#method-Lrner-getTrainLayer}{\code{Lrner$getTrainLayer()}}
\item \href{#method-Lrner-getNaRm}{\code{Lrner$getNaRm()}}
\item \href{#method-Lrner-getNaAction}{\code{Lrner$getNaAction()}}
\item \href{#method-Lrner-getId}{\code{Lrner$getId()}}
\item \href{#method-Lrner-getPackage}{\code{Lrner$getPackage()}}
\item \href{#method-Lrner-getIndSubset}{\code{Lrner$getIndSubset()}}
\item \href{#method-Lrner-getVarSubset}{\code{Lrner$getVarSubset()}}
\item \href{#method-Lrner-getParamPred}{\code{Lrner$getParamPred()}}
\item \href{#method-Lrner-getParamInterface}{\code{Lrner$getParamInterface()}}
\item \href{#method-Lrner-getExtractPred}{\code{Lrner$getExtractPred()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-new"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-new}{}}}
\subsection{Method \code{new()}}{
Initialize a default parameters list.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$new(
  id,
  package = NULL,
  lrn_fct,
  param_train_list,
  param_pred_list = list(),
  train_layer,
  na_action = "na.rm"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
Learner ID.}

\item{\code{package}}{\code{character} \cr
Package that implements the learn function. If NULL, the}

\item{\code{lrn_fct}}{\code{character} \cr
learn function is called from the current environment.}

\item{\code{param_train_list}}{\code{list} \cr
List of parameter for training.}

\item{\code{param_pred_list}}{\code{list} \cr
List of parameter for testing.
Learn parameters.}

\item{\code{train_layer}}{\code{TrainLayer} \cr
Layer on which the learner is stored.}

\item{\code{na_action}}{\code{character} \cr
Handling of missing values. Set to "na.keep" to keep missing values, "na.rm" to remove individuals with missing values or "na.impute" (only applicable on meta-data) to impute missing values in meta-data. Only median and mode based imputations are actually handled. With the "na.keep" option, ensure that the provided learner can handle missing values.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-print"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-summary"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-summary}{}}}
\subsection{Method \code{summary()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$summary(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-interface"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-interface}{}}}
\subsection{Method \code{interface()}}{
Learner and prediction parameter interface. Use this function
to provide how the following parameters are named in the learning
function (\code{lrn_fct}) you provided when creating the learner, or in the predicting function.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$interface(
  x = "x",
  y = "y",
  object = "object",
  data = "data",
  extract_pred_fct = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{\code{character} \cr
Name of the argument to pass the matrix of independent variables in the original learning function.}

\item{\code{y}}{\code{character} \cr
Name of the argument to pass the response variable in the original learning function.}

\item{\code{object}}{\code{character} \cr
Name of the argument to pass the model in the original predicting function.}

\item{\code{data}}{\code{character} \cr
Name of the argument to pass new data in the original predicting function.}

\item{\code{extract_pred_fct}}{\code{character} or \code{function} \cr
If the predict function that is called for the model does not return a vector, then
use this argument to specify a (or a name of a) function that can be used to extract vector of predictions.
Default value is NULL, if predictions are in a vector.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-train"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-train}{}}}
\subsection{Method \code{train()}}{
Tains the current learner (from class \link{Lrner}) on the current training data (from class \link{TrainData}).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$train(ind_subset = NULL, use_var_sel = FALSE, verbose = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ind_subset}}{\code{vector} \cr
Individual ID subset on which the training will be performed.}

\item{\code{use_var_sel}}{\code{boolean} \cr
If TRUE, variable selection is performed before training.}

\item{\code{verbose}}{\code{boolean} \cr
Warning messages will be displayed if set to TRUE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The resulting model, from class \link{Model}, is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getTrainLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getTrainLayer}{}}}
\subsection{Method \code{getTrainLayer()}}{
The current layer is returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getTrainLayer()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
\link{TrainLayer} object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getNaRm"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getNaRm}{}}}
\subsection{Method \code{getNaRm()}}{
The current layer is returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getNaRm()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getNaAction"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getNaAction}{}}}
\subsection{Method \code{getNaAction()}}{
The current layer is returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getNaAction()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getId"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getId}{}}}
\subsection{Method \code{getId()}}{
Getter of the current learner ID.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getId()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The current learner ID.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getPackage"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getPackage}{}}}
\subsection{Method \code{getPackage()}}{
Getter of the learner package implementing the learn function.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getPackage()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The name of the package implementing the learn function.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getIndSubset"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getIndSubset}{}}}
\subsection{Method \code{getIndSubset()}}{
Getter of the learner package implementing the learn function.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getIndSubset()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The name of the package implementing the learn function.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getVarSubset"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getVarSubset}{}}}
\subsection{Method \code{getVarSubset()}}{
Getter of the variable subset used for training.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getVarSubset()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The list of variables used for training is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getParamPred"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getParamPred}{}}}
\subsection{Method \code{getParamPred()}}{
Getter predicting parameter list.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getParamPred()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The list of predicting parameters.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getParamInterface"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getParamInterface}{}}}
\subsection{Method \code{getParamInterface()}}{
The current parameter interface is returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getParamInterface()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A data.frame of interface.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Lrner-getExtractPred"></a>}}
\if{latex}{\out{\hypertarget{method-Lrner-getExtractPred}{}}}
\subsection{Method \code{getExtractPred()}}{
The function to extract predicted values is returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Lrner$getExtractPred()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A data.frame of interface.
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.cobra.R
\name{predict.cobra}
\alias{predict.cobra}
\title{Predict Using COBRA object}
\usage{
\method{predict}{cobra}(object, data, ...)
}
\arguments{
\item{object}{An object of class "cobra" created by the \code{cobra} function.}

\item{data}{A \code{data.frame} of new data, where rows are observations and
columns are predictions from individual learners. Use \code{NA} for missing predictions.}

\item{...}{Additional arguments (currently not used).}
}
\value{
A vector of predictions for the new data.
}
\description{
#' The \code{predict.cobra} function makes predictions on new data using a trained COBRA object.
}
\examples{
# Example usage
set.seed(123)
x_train <- data.frame(a = rnorm(10L), b = rnorm(10L))
y_train <- sample(0L:1L, size = 10L, replace = TRUE)

# Train the model with epsilon optimization
cobra_model <- cobra(x = x_train, y = y_train, tune = "epsilon")

# Make predictions on new data
set.seed(156)
x_new <- data.frame(a = rnorm(5L), b = rnorm(5L))
prediction <- predict(object = cobra_model, data = x_new)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/VarSel.R
\name{VarSel}
\alias{VarSel}
\title{Varsel Class}
\description{
This class implements a learner. A \link{VarSel} object can only exist as a component of a
\link{TrainLayer} or a \link{TrainMetaLayer} object.
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-VarSel-new}{\code{VarSel$new()}}
\item \href{#method-VarSel-print}{\code{VarSel$print()}}
\item \href{#method-VarSel-summary}{\code{VarSel$summary()}}
\item \href{#method-VarSel-interface}{\code{VarSel$interface()}}
\item \href{#method-VarSel-varSelection}{\code{VarSel$varSelection()}}
\item \href{#method-VarSel-getTrainLayer}{\code{VarSel$getTrainLayer()}}
\item \href{#method-VarSel-getId}{\code{VarSel$getId()}}
\item \href{#method-VarSel-getPackage}{\code{VarSel$getPackage()}}
\item \href{#method-VarSel-getVarSubSet}{\code{VarSel$getVarSubSet()}}
\item \href{#method-VarSel-getParamInterface}{\code{VarSel$getParamInterface()}}
\item \href{#method-VarSel-getNaAction}{\code{VarSel$getNaAction()}}
\item \href{#method-VarSel-getExtractVar}{\code{VarSel$getExtractVar()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-new"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-new}{}}}
\subsection{Method \code{new()}}{
Variable selection parameter list.

Learner ID.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$new(
  id,
  package = NULL,
  varsel_fct,
  varsel_param,
  train_layer,
  na_action = "na.rm"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
Package that implements the variable selection function.
If NULL, the variable selection  function is called from
the current environment.}

\item{\code{package}}{\code{character} \cr
Variable selection function name. Note: Variable selection functions, except \code{Boruta}, must return a vector of selected variables.}

\item{\code{varsel_fct}}{\code{character} \cr
Variable selection parameters.}

\item{\code{varsel_param}}{\code{list} \cr
Layer on which the learner is stored.}

\item{\code{train_layer}}{\code{TrainLayer} \cr
The training layer where to store the learner.}

\item{\code{na_action}}{\code{character} \cr
Handling of missing values in meta-data. Set to "na.keep" to keep missing values, "na.rm" to remove individuals with missing values or "na.impute" (only applicable on meta-data) to impute missing values in meta-data. Only median and mode based imputations are actually handled. With the "na.keep" option, ensure that the provided learner can handle missing values.
If \code{TRUE}, the individuals with missing predictor values will be removed from the training dataset.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-print"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-summary"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-summary}{}}}
\subsection{Method \code{summary()}}{
Summary
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$summary(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-interface"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-interface}{}}}
\subsection{Method \code{interface()}}{
Learner and prediction parameter interface. Use this function
to provide how the following parameters are named in the learning
function (\code{lrn_fct}) you provided when creating the learner, or in the predicting function.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$interface(
  x = "x",
  y = "y",
  object = "object",
  data = "data",
  extract_var_fct = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{\code{string} \cr
Name of the argument to pass the matrix of independent variables in the original learning function.}

\item{\code{y}}{\code{string} \cr
Name of the argument to pass the response variable in the original learning function.}

\item{\code{object}}{\code{string} \cr
Name of the argument to pass the model in the original predicting function.}

\item{\code{data}}{\code{character} \cr
Name of the argument to pass new data in the original predicting function.}

\item{\code{extract_var_fct}}{\code{character} or \code{function} \cr
If the variable selection function that is called does not return a vector, then
use this argument to specify a (or a name of a) function that can be used to extract vector of selected variables.
Default value is NULL, if selected variables are in a vector.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-varSelection"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-varSelection}{}}}
\subsection{Method \code{varSelection()}}{
Tains the current learner (from class \link{Lrner}) on the current training data (from class \link{TrainData}).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$varSelection(ind_subset = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ind_subset}}{\code{vector} \cr
Individual ID subset on which the training will be performed.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The resulting model, from class \link{Model}, is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-getTrainLayer"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-getTrainLayer}{}}}
\subsection{Method \code{getTrainLayer()}}{
The current layer is returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$getTrainLayer()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
\link{TrainLayer} object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-getId"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-getId}{}}}
\subsection{Method \code{getId()}}{
Getter of the current learner ID.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$getId()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The current learner ID.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-getPackage"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-getPackage}{}}}
\subsection{Method \code{getPackage()}}{
Getter of the variable selection package implementing the variable selection function.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$getPackage()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The name of the package implementing the variable selection function.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-getVarSubSet"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-getVarSubSet}{}}}
\subsection{Method \code{getVarSubSet()}}{
Getter of the list of selected variables.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$getVarSubSet()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
List of selected variables..
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-getParamInterface"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-getParamInterface}{}}}
\subsection{Method \code{getParamInterface()}}{
The current parameter interface is returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$getParamInterface()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A data.frame of interface.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-getNaAction"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-getNaAction}{}}}
\subsection{Method \code{getNaAction()}}{
The current layer is returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$getNaAction()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-VarSel-getExtractVar"></a>}}
\if{latex}{\out{\hypertarget{method-VarSel-getExtractVar}{}}}
\subsection{Method \code{getExtractVar()}}{
The function to extract selected variables is returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VarSel$getExtractVar()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A data.frame of interface.
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainingFunctions.R
\name{createTraining}
\alias{createTraining}
\title{createTraining}
\usage{
createTraining(
  id,
  target_df,
  ind_col,
  target,
  problem_type = "classification",
  verbose = TRUE
)
}
\arguments{
\item{id}{\code{character} \cr
Training's ID.}

\item{target_df}{\code{data.frame} \cr
Observed target values. A data frame with two columns: individual IDs and response variable values.}

\item{ind_col}{\code{character} \cr
Name of column of individuals IDs.}

\item{target}{\code{character} \cr
Name of the target variable.}

\item{problem_type}{\code{character} \cr
Either "classification" or "regression".}

\item{verbose}{\code{boolean} \cr
Warning and processing information (including those of cross-validation) will be displayed if set to TRUE.}
}
\value{
The created \link{Training} object is returned.
}
\description{
Creates a \link{Training} object. A training object is designed to encapsulate training layers
and training meta-layer. Functions \link{createTrainLayer} and \link{createTrainMetaLayer} are available
to add the training layer and the training meta-layer to a training object.
}
\seealso{
\link{createTrainLayer}, \link{createTrainMetaLayer} and \link{fusemlr}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TestData.R
\name{TestData}
\alias{TestData}
\title{TestData Class}
\description{
This class implements \link{TestData} object to be predicted.
A \link{TestData} object can only exist as a component of a \link{TestLayer} or a \link{TestMetaLayer} object.
}
\seealso{
\link{TrainData}
}
\section{Super class}{
\code{\link[fuseMLR:Data]{fuseMLR::Data}} -> \code{TestData}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-TestData-new}{\code{TestData$new()}}
\item \href{#method-TestData-print}{\code{TestData$print()}}
\item \href{#method-TestData-getData}{\code{TestData$getData()}}
\item \href{#method-TestData-getTestLayer}{\code{TestData$getTestLayer()}}
\item \href{#method-TestData-clone}{\code{TestData$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getCompleteData"><a href='../../fuseMLR/html/Data.html#method-Data-getCompleteData'><code>fuseMLR::Data$getCompleteData()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getDataFrame"><a href='../../fuseMLR/html/Data.html#method-Data-getDataFrame'><code>fuseMLR::Data$getDataFrame()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getId"><a href='../../fuseMLR/html/Data.html#method-Data-getId'><code>fuseMLR::Data$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getIndCol"><a href='../../fuseMLR/html/Data.html#method-Data-getIndCol'><code>fuseMLR::Data$getIndCol()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getIndSubset"><a href='../../fuseMLR/html/Data.html#method-Data-getIndSubset'><code>fuseMLR::Data$getIndSubset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getSetDiff"><a href='../../fuseMLR/html/Data.html#method-Data-getSetDiff'><code>fuseMLR::Data$getSetDiff()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="getVarSubset"><a href='../../fuseMLR/html/Data.html#method-Data-getVarSubset'><code>fuseMLR::Data$getVarSubset()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="impute"><a href='../../fuseMLR/html/Data.html#method-Data-impute'><code>fuseMLR::Data$impute()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="Data" data-id="setDataFrame"><a href='../../fuseMLR/html/Data.html#method-Data-setDataFrame'><code>fuseMLR::Data$setDataFrame()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestData-new"></a>}}
\if{latex}{\out{\hypertarget{method-TestData-new}{}}}
\subsection{Method \code{new()}}{
Initialize a new object from the current class.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestData$new(id, data_frame, new_layer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
Object ID.}

\item{\code{data_frame}}{\code{data.frame}\cr
\code{data.frame} containing data.}

\item{\code{new_layer}}{\code{TestLayer} \cr
Layer where to store the current object.}

\item{\code{ind_col}}{\code{character}\cr
Column name containing individual IDs.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestData-print"></a>}}
\if{latex}{\out{\hypertarget{method-TestData-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestData$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestData-getData"></a>}}
\if{latex}{\out{\hypertarget{method-TestData-getData}{}}}
\subsection{Method \code{getData()}}{
Getter of the current \code{data.frame} wihtout individual
ID variable.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestData$getData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The \code{data.frame} without individual ID nor target variables is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestData-getTestLayer"></a>}}
\if{latex}{\out{\hypertarget{method-TestData-getTestLayer}{}}}
\subsection{Method \code{getTestLayer()}}{
Getter of the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestData$getTestLayer()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The layer (from class \link{TestLayer}) on which the current train data are stored
is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestData-clone"></a>}}
\if{latex}{\out{\hypertarget{method-TestData-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestData$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weightedMeanLearner.R
\name{weightedMeanLearner}
\alias{weightedMeanLearner}
\title{The weighted mean meta-learner}
\usage{
weightedMeanLearner(x, y, weighted = TRUE, perf = NULL, na_rm = FALSE)
}
\arguments{
\item{x}{\code{data.frame} \cr
Modality-specific predictions. Each column of the \code{data.frame} content the predictions
a specific learner.}

\item{y}{\code{vector} \cr
True target values. If classification, either binary or two level factor variable.}

\item{weighted}{\code{boolean} \cr
If TRUE, a weighted sum is computed. As default, weights are estimated based on Brier Score
for classification setting and mean squared error for regression. Otherwise, use argument
\code{perf} below to specify the function to use estimate learner performance.}

\item{perf}{\code{function} \cr
Function to compute layer-specific performance of learners. If NULL, the Brier Score (classification) or a mean squared error (regression) is used by default as performance measure.
Otherwise, the performance function must accept two parameters: \code{observed} (observed values) and \code{predicted} (predicted values).}

\item{na_rm}{\code{boolean} \cr
Should missing values be removed when computing the weights?}
}
\value{
Object of class \code{weightedMeanLearner} with the vector of estimated weights pro layer.\cr
}
\description{
Modality-specific learner are assessed and weighted based on their predictions. This function is intended to be (internally) used as meta-learner in fuseMLR.
}
\examples{
set.seed(20240624L)
x = data.frame(x1 = runif(n = 50L, min = 0, max = 1),
               x2 = runif(n = 50L, min = 0, max = 1))
y = sample(x = 0L:1L, size = 50L, replace = TRUE)
my_model = weightedMeanLearner(x = x, y = y)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TestMetaLayer.R
\name{TestMetaLayer}
\alias{TestMetaLayer}
\title{TestMetaLayer Class}
\description{
This class implement a predicted meta layer. A \link{TestMetaLayer} can only exist as unique element of a \link{Training} object.

A predicted meta layer can only contain a \link{TestData} object.
}
\section{Super class}{
\code{\link[fuseMLR:HashTable]{fuseMLR::HashTable}} -> \code{TestMetaLayer}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-TestMetaLayer-new}{\code{TestMetaLayer$new()}}
\item \href{#method-TestMetaLayer-print}{\code{TestMetaLayer$print()}}
\item \href{#method-TestMetaLayer-getTesting}{\code{TestMetaLayer$getTesting()}}
\item \href{#method-TestMetaLayer-getTestData}{\code{TestMetaLayer$getTestData()}}
\item \href{#method-TestMetaLayer-openAccess}{\code{TestMetaLayer$openAccess()}}
\item \href{#method-TestMetaLayer-closeAccess}{\code{TestMetaLayer$closeAccess()}}
\item \href{#method-TestMetaLayer-getAccess}{\code{TestMetaLayer$getAccess()}}
\item \href{#method-TestMetaLayer-setTestData}{\code{TestMetaLayer$setTestData()}}
\item \href{#method-TestMetaLayer-checkTestDataExist}{\code{TestMetaLayer$checkTestDataExist()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="add2HashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-add2HashTable'><code>fuseMLR::HashTable$add2HashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="checkClassExist"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-checkClassExist'><code>fuseMLR::HashTable$checkClassExist()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getFromHashTable'><code>fuseMLR::HashTable$getFromHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getHashTable'><code>fuseMLR::HashTable$getHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getId"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getId'><code>fuseMLR::HashTable$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getKeyClass"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getKeyClass'><code>fuseMLR::HashTable$getKeyClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="removeFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-removeFromHashTable'><code>fuseMLR::HashTable$removeFromHashTable()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestMetaLayer-new"></a>}}
\if{latex}{\out{\hypertarget{method-TestMetaLayer-new}{}}}
\subsection{Method \code{new()}}{
constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestMetaLayer$new(id, testing)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character}\cr
Testing meta-layer id.}

\item{\code{testing}}{\code{Testing}\cr}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestMetaLayer-print"></a>}}
\if{latex}{\out{\hypertarget{method-TestMetaLayer-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestMetaLayer$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestMetaLayer-getTesting"></a>}}
\if{latex}{\out{\hypertarget{method-TestMetaLayer-getTesting}{}}}
\subsection{Method \code{getTesting()}}{
Getter of the current testing object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestMetaLayer$getTesting()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The current testing object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestMetaLayer-getTestData"></a>}}
\if{latex}{\out{\hypertarget{method-TestMetaLayer-getTestData}{}}}
\subsection{Method \code{getTestData()}}{
Getter of the training dataset stored on the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestMetaLayer$getTestData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{TestData} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestMetaLayer-openAccess"></a>}}
\if{latex}{\out{\hypertarget{method-TestMetaLayer-openAccess}{}}}
\subsection{Method \code{openAccess()}}{
Open access to the meta layer. A meta learner is only
modifiable if the access is opened.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestMetaLayer$openAccess()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestMetaLayer-closeAccess"></a>}}
\if{latex}{\out{\hypertarget{method-TestMetaLayer-closeAccess}{}}}
\subsection{Method \code{closeAccess()}}{
Close access to the meta layer to avoid accidental
modification.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestMetaLayer$closeAccess()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestMetaLayer-getAccess"></a>}}
\if{latex}{\out{\hypertarget{method-TestMetaLayer-getAccess}{}}}
\subsection{Method \code{getAccess()}}{
Getter of the current access to the meta layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestMetaLayer$getAccess()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestMetaLayer-setTestData"></a>}}
\if{latex}{\out{\hypertarget{method-TestMetaLayer-setTestData}{}}}
\subsection{Method \code{setTestData()}}{
Create and set an \link{TestData} object to the current
new meta learner.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestMetaLayer$setTestData(id, ind_col, data_frame)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character(1)} \cr
ID of the \link{TestData} object to be instanciated.}

\item{\code{ind_col}}{\code{character(1)} \cr
Name of individual column IDs.}

\item{\code{data_frame}}{\code{data.frame(1)} \cr
\code{data.frame} of layer specific predictions.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestMetaLayer-checkTestDataExist"></a>}}
\if{latex}{\out{\hypertarget{method-TestMetaLayer-checkTestDataExist}{}}}
\subsection{Method \code{checkTestDataExist()}}{
Check whether a new data has been already stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestMetaLayer$checkTestDataExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainingFunctions.R
\name{extractModel}
\alias{extractModel}
\title{extractModel}
\usage{
extractModel(training)
}
\arguments{
\item{training}{\code{Training} \cr
The \link{Training} object of interest.}
}
\value{
A list of models is returned.
}
\description{
Extracts models stored on each layers; base and meta models are extracted.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictLayer.R
\name{PredictLayer}
\alias{PredictLayer}
\title{PredictLayer Class}
\description{
This class implements a layer. A \link{PredictLayer} object can only exist as a component of a \link{Predicting} object.

A predicted layer can only contain \link{PredictData}.
}
\seealso{
\link{Training}, \link{Lrner}, \link{TrainData}, \link{TestData} and \link{Model}
}
\section{Super class}{
\code{\link[fuseMLR:HashTable]{fuseMLR::HashTable}} -> \code{PredictLayer}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-PredictLayer-new}{\code{PredictLayer$new()}}
\item \href{#method-PredictLayer-print}{\code{PredictLayer$print()}}
\item \href{#method-PredictLayer-getPredicting}{\code{PredictLayer$getPredicting()}}
\item \href{#method-PredictLayer-getIndIDs}{\code{PredictLayer$getIndIDs()}}
\item \href{#method-PredictLayer-getPredictData}{\code{PredictLayer$getPredictData()}}
\item \href{#method-PredictLayer-setPredicting}{\code{PredictLayer$setPredicting()}}
\item \href{#method-PredictLayer-summary}{\code{PredictLayer$summary()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="add2HashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-add2HashTable'><code>fuseMLR::HashTable$add2HashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="checkClassExist"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-checkClassExist'><code>fuseMLR::HashTable$checkClassExist()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getFromHashTable'><code>fuseMLR::HashTable$getFromHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getHashTable'><code>fuseMLR::HashTable$getHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getId"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getId'><code>fuseMLR::HashTable$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getKeyClass"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getKeyClass'><code>fuseMLR::HashTable$getKeyClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="removeFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-removeFromHashTable'><code>fuseMLR::HashTable$removeFromHashTable()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictLayer-new"></a>}}
\if{latex}{\out{\hypertarget{method-PredictLayer-new}{}}}
\subsection{Method \code{new()}}{
constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictLayer$new(id)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
The layer ID.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictLayer-print"></a>}}
\if{latex}{\out{\hypertarget{method-PredictLayer-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictLayer$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictLayer-getPredicting"></a>}}
\if{latex}{\out{\hypertarget{method-PredictLayer-getPredicting}{}}}
\subsection{Method \code{getPredicting()}}{
Getter of the current predicting object
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictLayer$getPredicting()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The current predicting object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictLayer-getIndIDs"></a>}}
\if{latex}{\out{\hypertarget{method-PredictLayer-getIndIDs}{}}}
\subsection{Method \code{getIndIDs()}}{
Getter of IDS from the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictLayer$getIndIDs()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{data.frame} containing individuals IDs values.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictLayer-getPredictData"></a>}}
\if{latex}{\out{\hypertarget{method-PredictLayer-getPredictData}{}}}
\subsection{Method \code{getPredictData()}}{
Getter of the predicted data stored on the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictLayer$getPredictData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{PredictData} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictLayer-setPredicting"></a>}}
\if{latex}{\out{\hypertarget{method-PredictLayer-setPredicting}{}}}
\subsection{Method \code{setPredicting()}}{
Assigns a predicting object to the predicted layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictLayer$setPredicting(predicting)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{predicting}}{\code{Predicting} \cr}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The current object
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictLayer-summary"></a>}}
\if{latex}{\out{\hypertarget{method-PredictLayer-summary}{}}}
\subsection{Method \code{summary()}}{
Generate summary.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictLayer$summary()}\if{html}{\out{</div>}}
}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TrainMetaLayer.R
\name{TrainMetaLayer}
\alias{TrainMetaLayer}
\title{TrainMetaLayer Class}
\description{
This class implement a meta meta layer. A \link{TrainMetaLayer} can only exist as unique element of a \link{Training} object.

A layer is structured as followed:
\itemize{
\item \link{Lrner}: It is set by the user to be trained on the meta training data.
\item \link{TrainData}: It are modality-specific prediction data, automatically created by the internal cross validation.
\item \link{Model}: The meta model, result of training the learner on the training data, and therefore, not to be set by the user.
\item \link{TestData}: The meta new data to be predicted, consisting in predictions obtained from each layer.
}

A meta layer can train its meta learner on the meta training data and store the resulting meta model.
The meta layer can predict values given a new meta layer.
}
\section{Super class}{
\code{\link[fuseMLR:HashTable]{fuseMLR::HashTable}} -> \code{TrainMetaLayer}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-TrainMetaLayer-new}{\code{TrainMetaLayer$new()}}
\item \href{#method-TrainMetaLayer-print}{\code{TrainMetaLayer$print()}}
\item \href{#method-TrainMetaLayer-getTraining}{\code{TrainMetaLayer$getTraining()}}
\item \href{#method-TrainMetaLayer-getTargetObj}{\code{TrainMetaLayer$getTargetObj()}}
\item \href{#method-TrainMetaLayer-train}{\code{TrainMetaLayer$train()}}
\item \href{#method-TrainMetaLayer-predict}{\code{TrainMetaLayer$predict()}}
\item \href{#method-TrainMetaLayer-impute}{\code{TrainMetaLayer$impute()}}
\item \href{#method-TrainMetaLayer-getTrainData}{\code{TrainMetaLayer$getTrainData()}}
\item \href{#method-TrainMetaLayer-getLrner}{\code{TrainMetaLayer$getLrner()}}
\item \href{#method-TrainMetaLayer-getModel}{\code{TrainMetaLayer$getModel()}}
\item \href{#method-TrainMetaLayer-openAccess}{\code{TrainMetaLayer$openAccess()}}
\item \href{#method-TrainMetaLayer-closeAccess}{\code{TrainMetaLayer$closeAccess()}}
\item \href{#method-TrainMetaLayer-getAccess}{\code{TrainMetaLayer$getAccess()}}
\item \href{#method-TrainMetaLayer-setTrainData}{\code{TrainMetaLayer$setTrainData()}}
\item \href{#method-TrainMetaLayer-checkLrnerExist}{\code{TrainMetaLayer$checkLrnerExist()}}
\item \href{#method-TrainMetaLayer-checkModelExist}{\code{TrainMetaLayer$checkModelExist()}}
\item \href{#method-TrainMetaLayer-checkTrainDataExist}{\code{TrainMetaLayer$checkTrainDataExist()}}
\item \href{#method-TrainMetaLayer-set2NotTrained}{\code{TrainMetaLayer$set2NotTrained()}}
\item \href{#method-TrainMetaLayer-summary}{\code{TrainMetaLayer$summary()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="add2HashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-add2HashTable'><code>fuseMLR::HashTable$add2HashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="checkClassExist"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-checkClassExist'><code>fuseMLR::HashTable$checkClassExist()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getFromHashTable'><code>fuseMLR::HashTable$getFromHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getHashTable'><code>fuseMLR::HashTable$getHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getId"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getId'><code>fuseMLR::HashTable$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getKeyClass"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getKeyClass'><code>fuseMLR::HashTable$getKeyClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="removeFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-removeFromHashTable'><code>fuseMLR::HashTable$removeFromHashTable()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-new"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-new}{}}}
\subsection{Method \code{new()}}{
constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$new(id, training)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character}\cr
Id of training meta-layer.}

\item{\code{training}}{\code{Training}\cr}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-print"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-getTraining"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-getTraining}{}}}
\subsection{Method \code{getTraining()}}{
Getter of the current training object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$getTraining()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The current training object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-getTargetObj"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-getTargetObj}{}}}
\subsection{Method \code{getTargetObj()}}{
Getter of the target object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$getTargetObj()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-train"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-train}{}}}
\subsection{Method \code{train()}}{
Trains the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$train(ind_subset = NULL, verbose = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ind_subset}}{\code{vector} \cr
ID subset of individuals to be used for training.}

\item{\code{verbose}}{\code{boolean} \cr
Warning messages will be displayed if set to TRUE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The current layer is returned with the resulting model.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-predict"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-predict}{}}}
\subsection{Method \code{predict()}}{
Predicts values for the new layer taking as argument.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$predict(new_layer, ind_subset = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{new_layer}}{\code{TrainLayer} \cr
A trained TrainLayer object.}

\item{\code{ind_subset}}{\code{vector} \cr
Index subset.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new object with the predicted values is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-impute"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-impute}{}}}
\subsection{Method \code{impute()}}{
Imputes missing values in modality-specific predictions.
Only mode and median based imputations are actually supported.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$impute(impute_fct = NULL, impute_param = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{impute_fct}}{\code{character} \cr
An imputation function to use instead of median or mode imputation.
This parameter is actually not used.
This corresponds to median or mode based imputation.}

\item{\code{impute_param}}{\code{list} \cr
The list of parameters to call the imputation function. Not yet implemented!}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new object with the predicted values is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-getTrainData"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-getTrainData}{}}}
\subsection{Method \code{getTrainData()}}{
Getter of the training dataset stored on the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$getTrainData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{TrainData} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-getLrner"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-getLrner}{}}}
\subsection{Method \code{getLrner()}}{
Getter of the learner.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$getLrner()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{Lrner} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-getModel"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-getModel}{}}}
\subsection{Method \code{getModel()}}{
Getter of the model.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$getModel()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{Model} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-openAccess"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-openAccess}{}}}
\subsection{Method \code{openAccess()}}{
Open access to the meta layer. A meta learner is only
modifiable if the access is opened.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$openAccess()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-closeAccess"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-closeAccess}{}}}
\subsection{Method \code{closeAccess()}}{
Close access to the meta layer to avoid accidental
modification.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$closeAccess()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-getAccess"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-getAccess}{}}}
\subsection{Method \code{getAccess()}}{
Getter of the current access to the meta layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$getAccess()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-setTrainData"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-setTrainData}{}}}
\subsection{Method \code{setTrainData()}}{
Create and set an \link{TrainData} object to the current
meta learner.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$setTrainData(id, ind_col, data_frame)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
ID of the \link{TrainData} object to be instanciated.}

\item{\code{ind_col}}{\code{character} \cr
Name of individual column IDs.}

\item{\code{data_frame}}{\code{data.frame} \cr
\code{data.frame} of layer specific predictions.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-checkLrnerExist"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-checkLrnerExist}{}}}
\subsection{Method \code{checkLrnerExist()}}{
Check whether a training data has been already stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$checkLrnerExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-checkModelExist"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-checkModelExist}{}}}
\subsection{Method \code{checkModelExist()}}{
Check whether a model has been already stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$checkModelExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-checkTrainDataExist"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-checkTrainDataExist}{}}}
\subsection{Method \code{checkTrainDataExist()}}{
Check whether a training data has been already stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$checkTrainDataExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-set2NotTrained"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-set2NotTrained}{}}}
\subsection{Method \code{set2NotTrained()}}{
Only usefull to reset status FALSE after cross validation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$set2NotTrained()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainMetaLayer-summary"></a>}}
\if{latex}{\out{\hypertarget{method-TrainMetaLayer-summary}{}}}
\subsection{Method \code{summary()}}{
Generate summary.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainMetaLayer$summary()}\if{html}{\out{</div>}}
}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createDif.R
\name{createDif}
\alias{createDif}
\title{Create Difference}
\usage{
createDif(x)
}
\arguments{
\item{x}{Predictions vector}
}
\description{
The \code{createDif} function computes the difference between the maximum and minimum predictions in a dataset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Data.R
\name{Data}
\alias{Data}
\title{Abstract class Data}
\description{
As abstract, a \link{Data} object cannot be stored on any layer. Instead, extended
\link{TrainData} or \link{TestData} objects can be stored on a layer.
}
\seealso{
\link{TrainData} and \link{TestData}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Data-new}{\code{Data$new()}}
\item \href{#method-Data-print}{\code{Data$print()}}
\item \href{#method-Data-getIndSubset}{\code{Data$getIndSubset()}}
\item \href{#method-Data-impute}{\code{Data$impute()}}
\item \href{#method-Data-getVarSubset}{\code{Data$getVarSubset()}}
\item \href{#method-Data-getSetDiff}{\code{Data$getSetDiff()}}
\item \href{#method-Data-getDataFrame}{\code{Data$getDataFrame()}}
\item \href{#method-Data-setDataFrame}{\code{Data$setDataFrame()}}
\item \href{#method-Data-getCompleteData}{\code{Data$getCompleteData()}}
\item \href{#method-Data-getId}{\code{Data$getId()}}
\item \href{#method-Data-getData}{\code{Data$getData()}}
\item \href{#method-Data-getIndCol}{\code{Data$getIndCol()}}
\item \href{#method-Data-clone}{\code{Data$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-new"></a>}}
\if{latex}{\out{\hypertarget{method-Data-new}{}}}
\subsection{Method \code{new()}}{
Constructor of class Data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$new(id, ind_col, data_frame)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
Object ID.}

\item{\code{ind_col}}{\code{character} \cr
Column name containing individual IDs.}

\item{\code{data_frame}}{\code{data.frame} \cr
\code{data.frame} containing data.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-print"></a>}}
\if{latex}{\out{\hypertarget{method-Data-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-getIndSubset"></a>}}
\if{latex}{\out{\hypertarget{method-Data-getIndSubset}{}}}
\subsection{Method \code{getIndSubset()}}{
Retrieve a data subset for a given variable name and values, a data subset.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$getIndSubset(var_name, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{var_name}}{\code{character} \cr
Variable name of interest.}

\item{\code{value}}{\code{vector} \cr
Values of interest.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The data subset is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-impute"></a>}}
\if{latex}{\out{\hypertarget{method-Data-impute}{}}}
\subsection{Method \code{impute()}}{
Imputes missing values in modality-specific predictions.
Only mode and median based imputations are actually supported.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$impute(impute_fct, impute_param, target_name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{impute_fct}}{\code{character} \cr
An imputation function to use instead of median or mode imputation. Not yet implemented!}

\item{\code{impute_param}}{\code{list} \cr}

\item{\code{target_name}}{\code{character} \cr
Name of the target variable.
The list of parameters to call the imputation function.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new object with the predicted values is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-getVarSubset"></a>}}
\if{latex}{\out{\hypertarget{method-Data-getVarSubset}{}}}
\subsection{Method \code{getVarSubset()}}{
Retrieve a subset of variables from data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$getVarSubset(var_name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{var_name}}{\code{character} \cr
Variable names of interest.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The data subset is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-getSetDiff"></a>}}
\if{latex}{\out{\hypertarget{method-Data-getSetDiff}{}}}
\subsection{Method \code{getSetDiff()}}{
For the given variable name, non existing values in the
current dataset are returned.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$getSetDiff(var_name, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{var_name}}{\code{character} \cr
Variable name of interest.}

\item{\code{value}}{\code{vector} \cr
Values of interest.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The subset difference is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-getDataFrame"></a>}}
\if{latex}{\out{\hypertarget{method-Data-getDataFrame}{}}}
\subsection{Method \code{getDataFrame()}}{
Getter of the \code{data.frame}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$getDataFrame()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The \code{data.frame} of the current object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-setDataFrame"></a>}}
\if{latex}{\out{\hypertarget{method-Data-setDataFrame}{}}}
\subsection{Method \code{setDataFrame()}}{
Set a new \code{data.frame} to the current object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$setDataFrame(data_frame)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{data_frame}}{\code{data.frame}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The current object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-getCompleteData"></a>}}
\if{latex}{\out{\hypertarget{method-Data-getCompleteData}{}}}
\subsection{Method \code{getCompleteData()}}{
Getter of the complete dataset without missing values.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$getCompleteData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The complete dataset is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-getId"></a>}}
\if{latex}{\out{\hypertarget{method-Data-getId}{}}}
\subsection{Method \code{getId()}}{
Getter of the current object ID.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$getId()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The current object ID is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-getData"></a>}}
\if{latex}{\out{\hypertarget{method-Data-getData}{}}}
\subsection{Method \code{getData()}}{
Getter of the current Data. This function is re-implemented
by \link{TrainData} and \link{TestData}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$getData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Do not use on this class.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-getIndCol"></a>}}
\if{latex}{\out{\hypertarget{method-Data-getIndCol}{}}}
\subsection{Method \code{getIndCol()}}{
Getter of the individual column variable.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$getIndCol()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Data-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Data-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Data$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cobra.R
\name{cobra}
\alias{cobra}
\title{Cobra Meta Learner}
\usage{
cobra(x, y, tune = "epsilon", k_folds = NULL, eps = NULL)
}
\arguments{
\item{x}{\code{data.frame} \cr
A training data, where rows are observations and
columns are predictions from individual learners. Use \code{NA} for missing predictions.}

\item{y}{\code{vector} \cr
A vector containing the training targets. This can be a binary or two-level factor variable.}

\item{tune}{\code{character} \cr
A character value specifying the tuning mode:
\itemize{
\item \code{"alpha_epsilon"}: Tunes both \code{alpha} (number of learners) and \code{epsilon} (proximity threshold) via cross-validation.
\item \code{"epsilon"}: Tunes \code{epsilon} only via cross-validation.
\item \code{"user"}: No tuning; the user provides an optimal \code{epsilon}.
#' The default value is \code{epsilon}.
}}

\item{k_folds}{\code{integer} \cr
Number of folds for cross-validation when \code{tune = "alpha_epsilon"} or \code{"epsilon"}.
Default is \code{10}.}

\item{eps}{\code{numeric} \cr
A numeric value for the proximity threshold, used only when \code{tune = "user"}.
Defaults to \code{0.1}.}
}
\value{
An object of class \code{cobra} containing the training data, target values, and chosen parameters.
}
\description{
The function \code{cobra} implements the COBRA (COmBined Regression Alternative),
an aggregation method for combining predictions from multiple individual learners.
This method aims to tune key parameters for achieving optimal predictions
by averaging the target values of similar candidates in the training dataset's predictions.
Only the training points that are sufficiently similar to the test point
(based on the proximity threshold \code{epsilon}) are used for prediction.
If no suitable training points are found, the function returns \code{NA}.
}
\examples{
# Example usage
set.seed(123)
x_train <- data.frame(a = runif(10L), b = runif(10L))
y_train <- sample(0L:1L, size = 10L, replace = TRUE)

# Train the model with epsilon optimization
cobra_model <- cobra(x = x_train, y = y_train, tune = "epsilon", k_folds = 2)

# Make predictions on new data
set.seed(156)
x_new <- data.frame(a = runif(5L), b = runif(5L))
prediction <- predict(object = cobra_model, data = x_new)

}
\references{
Biau, G., Fischer, A., Guedj, B., & Malley, J. D. (2014). COBRA: A combined regression strategy. The Journal of Multivariate Analysis 46:18-28
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Predicting.R
\name{Predicting}
\alias{Predicting}
\title{Predicting Class}
\description{
This class is designed for predictions.

The Predicting is structured as followed:
\itemize{
\item \link{PredictLayer}: Exists for each modality.
\itemize{
\item \link{PredictData}: Related class for modality-specific predictions.
}
\item \link{PredictMetaLayer}: Related class for meta predictions.
\itemize{
\item \link{PredictData}: Specific to the meta layer, it is set up internally after cross-validation.
}
}

Use the function \code{train} for training and \code{predict} for predicting.

TODO: Do not export me.
}
\seealso{
\link{TrainLayer}
}
\section{Super class}{
\code{\link[fuseMLR:HashTable]{fuseMLR::HashTable}} -> \code{Predicting}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Predicting-new}{\code{Predicting$new()}}
\item \href{#method-Predicting-print}{\code{Predicting$print()}}
\item \href{#method-Predicting-createMetaTestData}{\code{Predicting$createMetaTestData()}}
\item \href{#method-Predicting-getIndIDs}{\code{Predicting$getIndIDs()}}
\item \href{#method-Predicting-getPredictMetaLayer}{\code{Predicting$getPredictMetaLayer()}}
\item \href{#method-Predicting-getIndCol}{\code{Predicting$getIndCol()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="add2HashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-add2HashTable'><code>fuseMLR::HashTable$add2HashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="checkClassExist"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-checkClassExist'><code>fuseMLR::HashTable$checkClassExist()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getFromHashTable'><code>fuseMLR::HashTable$getFromHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getHashTable'><code>fuseMLR::HashTable$getHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getId"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getId'><code>fuseMLR::HashTable$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getKeyClass"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getKeyClass'><code>fuseMLR::HashTable$getKeyClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="removeFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-removeFromHashTable'><code>fuseMLR::HashTable$removeFromHashTable()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Predicting-new"></a>}}
\if{latex}{\out{\hypertarget{method-Predicting-new}{}}}
\subsection{Method \code{new()}}{
constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Predicting$new(id, ind_col)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character}\cr
Predicting id.}

\item{\code{ind_col}}{\code{character}
Name of column of individuals IDS}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Predicting-print"></a>}}
\if{latex}{\out{\hypertarget{method-Predicting-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Predicting$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Predicting-createMetaTestData"></a>}}
\if{latex}{\out{\hypertarget{method-Predicting-createMetaTestData}{}}}
\subsection{Method \code{createMetaTestData()}}{
Creates a new modality-specific predictions dataset based on layer predictions.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Predicting$createMetaTestData(meta_layer_id)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{meta_layer_id}}{(\code{character(1)}) \cr
ID of the meta layer where the testing meta data will be stored.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A \link{TestData} is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Predicting-getIndIDs"></a>}}
\if{latex}{\out{\hypertarget{method-Predicting-getIndIDs}{}}}
\subsection{Method \code{getIndIDs()}}{
Gather individual IDs from all layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Predicting$getIndIDs()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{data.frame} containing individuals IDs.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Predicting-getPredictMetaLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Predicting-getPredictMetaLayer}{}}}
\subsection{Method \code{getPredictMetaLayer()}}{
Getter of the meta layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Predicting$getPredictMetaLayer()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Object from class \link{PredictMetaLayer}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Predicting-getIndCol"></a>}}
\if{latex}{\out{\hypertarget{method-Predicting-getIndCol}{}}}
\subsection{Method \code{getIndCol()}}{
Getter of the individual column name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Predicting$getIndCol()}\if{html}{\out{</div>}}
}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Testing.R
\name{Testing}
\alias{Testing}
\title{Testing Class}
\description{
This is a primary classes of fuseMLR. An object from this class
is designed to contain multiple layers, but only one new meta layer.

A Testing object is structured as followed:
\itemize{
\item \link{TestLayer}
\item \link{TestMetaLayer}
}
}
\seealso{
\link{TrainLayer}
}
\section{Super class}{
\code{\link[fuseMLR:HashTable]{fuseMLR::HashTable}} -> \code{Testing}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Testing-new}{\code{Testing$new()}}
\item \href{#method-Testing-print}{\code{Testing$print()}}
\item \href{#method-Testing-getIndIDs}{\code{Testing$getIndIDs()}}
\item \href{#method-Testing-getTestMetaLayer}{\code{Testing$getTestMetaLayer()}}
\item \href{#method-Testing-getIndCol}{\code{Testing$getIndCol()}}
\item \href{#method-Testing-getVerbose}{\code{Testing$getVerbose()}}
\item \href{#method-Testing-getData}{\code{Testing$getData()}}
\item \href{#method-Testing-upset}{\code{Testing$upset()}}
\item \href{#method-Testing-summary}{\code{Testing$summary()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="add2HashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-add2HashTable'><code>fuseMLR::HashTable$add2HashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="checkClassExist"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-checkClassExist'><code>fuseMLR::HashTable$checkClassExist()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getFromHashTable'><code>fuseMLR::HashTable$getFromHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getHashTable'><code>fuseMLR::HashTable$getHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getId"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getId'><code>fuseMLR::HashTable$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getKeyClass"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getKeyClass'><code>fuseMLR::HashTable$getKeyClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="removeFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-removeFromHashTable'><code>fuseMLR::HashTable$removeFromHashTable()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Testing-new"></a>}}
\if{latex}{\out{\hypertarget{method-Testing-new}{}}}
\subsection{Method \code{new()}}{
constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Testing$new(id, ind_col, verbose = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character}\cr
Testing id.}

\item{\code{ind_col}}{\code{character}
Name of column of individuals IDS in testing data.frame.}

\item{\code{verbose}}{\code{boolean} \cr
Warning messages will be displayed if set to TRUE.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Testing-print"></a>}}
\if{latex}{\out{\hypertarget{method-Testing-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Testing$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Testing-getIndIDs"></a>}}
\if{latex}{\out{\hypertarget{method-Testing-getIndIDs}{}}}
\subsection{Method \code{getIndIDs()}}{
Gather individual IDs from all layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Testing$getIndIDs()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{data.frame} containing individuals IDs.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Testing-getTestMetaLayer"></a>}}
\if{latex}{\out{\hypertarget{method-Testing-getTestMetaLayer}{}}}
\subsection{Method \code{getTestMetaLayer()}}{
Getter of the meta layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Testing$getTestMetaLayer()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Object from class \link{TestMetaLayer}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Testing-getIndCol"></a>}}
\if{latex}{\out{\hypertarget{method-Testing-getIndCol}{}}}
\subsection{Method \code{getIndCol()}}{
Getter of the individual column name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Testing$getIndCol()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Testing-getVerbose"></a>}}
\if{latex}{\out{\hypertarget{method-Testing-getVerbose}{}}}
\subsection{Method \code{getVerbose()}}{
Getter of the verbose setting.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Testing$getVerbose()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Testing-getData"></a>}}
\if{latex}{\out{\hypertarget{method-Testing-getData}{}}}
\subsection{Method \code{getData()}}{
Retrieve modality-specific prediction data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Testing$getData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{list} containing all (base and meta) models.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Testing-upset"></a>}}
\if{latex}{\out{\hypertarget{method-Testing-upset}{}}}
\subsection{Method \code{upset()}}{
UpSet plot to show an overview of the overlap of individuals across various layers.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Testing$upset(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any} \cr
Further parameters to be passed to the the \code{upset} function from package \code{UpSetR}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Testing-summary"></a>}}
\if{latex}{\out{\hypertarget{method-Testing-summary}{}}}
\subsection{Method \code{summary()}}{
Generate testing summary
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Testing$summary()}\if{html}{\out{</div>}}
}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testingFunctions.R
\name{summary.Testing}
\alias{summary.Testing}
\title{Testing object Summaries}
\usage{
\method{summary}{Testing}(object, ...)
}
\arguments{
\item{object}{\code{Testing} \cr
The \link{Testing} object of interest.}

\item{...}{\code{any} \cr
Further arguments.}
}
\description{
Summaries a \code{fuseMLR} \link{Testing} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.bestLayerLearner.R
\name{predict.bestLayerLearner}
\alias{predict.bestLayerLearner}
\title{Best specific Learner prediction.}
\usage{
\method{predict}{bestLayerLearner}(object, data, ...)
}
\arguments{
\item{object}{\code{bestLayerLearner} \cr
An object from class \link{bestLayerLearner}}

\item{data}{\code{data.frame} \cr
New data to predicted.}

\item{...}{\code{any}\cr
Further arguments passed to or from other methods.}
}
\value{
Predicted target values are returned.
}
\description{
Predict function for models from class \code{bestLayerLearner}.
}
\examples{
set.seed(20240625)
x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
y <- sample(x = 0:1, size = 50L, replace = TRUE)
my_model <- bestLayerLearner(x = x, y = y)
x_new <- data.frame(x1 = rnorm(10L))
my_predictions <- predict(object = my_model, data = x_new)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Model.R
\name{Model}
\alias{Model}
\title{Model Class}
\description{
This class implements a model. A \link{Model} object can only exist as element of a
\link{TrainLayer} or a \link{TrainMetaLayer} object. A \link{Model} object is
automatically created by fitting a learner on a training data.

A \link{Model} object can compute predictions for a \link{TestData} object. See the \code{predict} function below.
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Model-new}{\code{Model$new()}}
\item \href{#method-Model-print}{\code{Model$print()}}
\item \href{#method-Model-summary}{\code{Model$summary()}}
\item \href{#method-Model-getBaseModel}{\code{Model$getBaseModel()}}
\item \href{#method-Model-getTrainData}{\code{Model$getTrainData()}}
\item \href{#method-Model-getTrainLabel}{\code{Model$getTrainLabel()}}
\item \href{#method-Model-getLrner}{\code{Model$getLrner()}}
\item \href{#method-Model-setId}{\code{Model$setId()}}
\item \href{#method-Model-predict}{\code{Model$predict()}}
\item \href{#method-Model-clone}{\code{Model$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-new"></a>}}
\if{latex}{\out{\hypertarget{method-Model-new}{}}}
\subsection{Method \code{new()}}{
Constructor of Model class.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$new(lrner, train_data, base_model, train_layer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{lrner}}{\code{Lrner} \cr
The learner.}

\item{\code{train_data}}{\code{TrainData(1)} \cr
Training data.}

\item{\code{base_model}}{\code{object} \cr
Base model as returned by the original learn function.}

\item{\code{train_layer}}{\code{TrainLayer} \cr
The current training layer on which the model is stored.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
An object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-print"></a>}}
\if{latex}{\out{\hypertarget{method-Model-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-summary"></a>}}
\if{latex}{\out{\hypertarget{method-Model-summary}{}}}
\subsection{Method \code{summary()}}{
Summary
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$summary(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-getBaseModel"></a>}}
\if{latex}{\out{\hypertarget{method-Model-getBaseModel}{}}}
\subsection{Method \code{getBaseModel()}}{
Getter of the base model
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$getBaseModel()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-getTrainData"></a>}}
\if{latex}{\out{\hypertarget{method-Model-getTrainData}{}}}
\subsection{Method \code{getTrainData()}}{
Getter of the traning data
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$getTrainData()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-getTrainLabel"></a>}}
\if{latex}{\out{\hypertarget{method-Model-getTrainLabel}{}}}
\subsection{Method \code{getTrainLabel()}}{
Getter of the individual ID column in the training data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$getTrainLabel()}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-getLrner"></a>}}
\if{latex}{\out{\hypertarget{method-Model-getLrner}{}}}
\subsection{Method \code{getLrner()}}{
Getter of the learner use to fit the model.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$getLrner()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-setId"></a>}}
\if{latex}{\out{\hypertarget{method-Model-setId}{}}}
\subsection{Method \code{setId()}}{
Setter of the model ID.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$setId(id)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character} \cr
ID value}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-predict"></a>}}
\if{latex}{\out{\hypertarget{method-Model-predict}{}}}
\subsection{Method \code{predict()}}{
Predict target values for the new data
(from class \link{TestData}) taken as into.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$predict(testing_data, use_var_sel, ind_subset = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{testing_data}}{\code{TestData} \cr
An object from class \link{TestData}.}

\item{\code{use_var_sel}}{\code{boolean} \cr
If TRUE, selected variables available at each layer are used.}

\item{\code{ind_subset}}{\code{vector} \cr
Subset of individual IDs to be predicted.}

\item{\code{...}}{Further parameters to be passed to the basic predict function.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The predicted object are returned. The predicted object must be either a vector or a list
containing a field predictions with predictions.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Model-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Model-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Model$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainingFunctions.R
\name{createTrainMetaLayer}
\alias{createTrainMetaLayer}
\title{createTrainMetaLayer}
\usage{
createTrainMetaLayer(
  training,
  meta_layer_id,
  lrner_package = NULL,
  lrn_fct,
  param_train_list = list(),
  param_pred_list = list(),
  na_action = "na.impute",
  x_lrn = "x",
  y_lrn = "y",
  object = "object",
  data = "data",
  extract_pred_fct = NULL
)
}
\arguments{
\item{training}{\code{Training} \cr
Training object for storing the created meta-layer.}

\item{meta_layer_id}{\code{character} \cr
ID of the layer to be created.}

\item{lrner_package}{\code{character} \cr
Package name containing the variable selection algorithm function.
Defaults to \code{NULL} if the function exists in the current working environment.}

\item{lrn_fct}{\code{character} \cr
Name of the learning function. The function must accept at least two
parameters: \code{x} (predictors) and \code{y} (response values), and return a model.
If not, use the interface parameters \code{x_lrn} and \code{y_lrn} below to map these
argument names to the original arguments in your function. The returned model
must support the generic \code{predict} function (with arguments \code{object} and \code{data})
to make predictions for new data, and the predictions should be a vector or
a \code{list} containing a vector called \code{predictions} with the predicted values.
If the arguments \code{object} and \code{data} are named differently in your predict
function, use the interface parameters \code{object} and \code{data} below to specify
the original names. See the details below about meta-learners.}

\item{param_train_list}{\code{character} \cr
List of arguments to be passed to \code{lrn_fct}.}

\item{param_pred_list}{\code{list} \cr
List of arguments to be passed to \code{predict} when computing predictions.}

\item{na_action}{\code{character}\cr
Handling of missing values in modality-specific predictions during training.
Set to \code{"na.keep"} to keep missing values, \code{"na.rm"} to remove individuals
with missing values or \code{"na.impute"} to impute missing values in modality-specific
predictions. Only median and mode based imputations are actually handled.
With the \code{"na.keep"} option, ensure that the provided meta-learner can handle missing values.}

\item{x_lrn}{\code{character} \cr
If the argument name used by the provided original function to pass the matrix
of independent variables is not \code{x}, use this argument to specify the name used
in the function.}

\item{y_lrn}{\code{character} \cr
If the argument name used by the provided original function to pass the target
variable is not \code{y}, use this argument to specify the name used in the function.}

\item{object}{\code{character} \cr
The generic function \code{predict} uses a parameter \code{object} to pass a model.
If the corresponding argument is named differently in your predict function, specify the name.}

\item{data}{\code{character} \cr
The generic function \code{predict} uses a parameter \code{data} to pass new data.
If the corresponding argument is named differently in your predict function, specify the name.}

\item{extract_pred_fct}{\verb{character or function} \cr
If the predict function that is called for the model does not return a vector, then
use this argument to specify a (or a name of a) function that can be used to extract vector of predictions.
Defaults to NULL, if predictions are a vector.}
}
\value{
The updated \link{Training} object (with the new layer) is returned.
}
\description{
Creates and store a \link{TrainMetaLayer} on the \link{Training} object passed as argument.
The meta-layer encapsulates the meta-learner and the fold predictions (internally created) of the layer-specific
base models.
}
\details{
Internal meta-learners are available in the package.

The \link{cobra} meta-learner implements the COBRA (COmBined Regression Alternative),
an aggregation method for combining predictions from multiple individual learners (Biau et al. 2014).
This method aims to tune key parameters for achieving optimal predictions
by averaging the target values of similar candidates in the training dataset's predictions.
Only the training points that are sufficiently similar to the test point
(based on the proximity threshold \code{epsilon}) are used for prediction.
If no suitable training points are found, the function returns \code{NA}.

The \link{weightedMeanLearner} evaluates the prediction performance of modality-specific
learners and uses these estimates to weight the base models, aggregating their
predictions accordingly.

The \link{bestLayerLearner} evaluates the prediction performance of modality-specific
learners and returns predictions made by the best learner as the meta-prediction.

Beyond the internal meta-learners, any other learning algorithm can be used.
}
\references{
Fouodo, C. J. K., Bleskina, M. & Szymczak, S. fuseMLR: an R package for integrative prediction modeling of multi-omics data. BMC Bioinformatics 26, 221 (2025). https://doi.org/10.1186/s12859-025-06248-4 \cr
Biau, G., Fischer, A., Guedj, B., & Malley, J. D. (2014). COBRA: A combined regression strategy. The Journal of Multivariate Analysis 46:18-28
}
\seealso{
\link{createTrainLayer}, \link{varSelection}, and \link{fusemlr}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TrainLayer.R
\name{TrainLayer}
\alias{TrainLayer}
\title{TrainLayer Class}
\description{
This class implements a traning layer. A \link{TrainLayer} object can only exist as a component of a \link{Training} object.

A training layer is structured as followed:
\itemize{
\item \link{TrainData}: Data to be used to train the learner.
\item \link{Lrner}: Includes a learning function and the package implementing the function.
\item \link{Model}: The result of training the learner on the training data.
\item \link{VarSel}: Includes a variable selection function and the package implementing the function.
}

A training layer can train its learner on its training data and store the resulting model. See the public function \code{Layer$train()} below.

A training layer can make predictions for a new layer passed as argument to its predict function. See the public function \code{Layer$predict()} below.
}
\seealso{
\link{Training}, \link{Lrner}, \link{TrainData}, \link{TestData} and \link{Model}
}
\section{Super class}{
\code{\link[fuseMLR:HashTable]{fuseMLR::HashTable}} -> \code{TrainLayer}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-TrainLayer-new}{\code{TrainLayer$new()}}
\item \href{#method-TrainLayer-print}{\code{TrainLayer$print()}}
\item \href{#method-TrainLayer-getTraining}{\code{TrainLayer$getTraining()}}
\item \href{#method-TrainLayer-getTargetObj}{\code{TrainLayer$getTargetObj()}}
\item \href{#method-TrainLayer-train}{\code{TrainLayer$train()}}
\item \href{#method-TrainLayer-varSelection}{\code{TrainLayer$varSelection()}}
\item \href{#method-TrainLayer-predict}{\code{TrainLayer$predict()}}
\item \href{#method-TrainLayer-getTrainData}{\code{TrainLayer$getTrainData()}}
\item \href{#method-TrainLayer-getTargetValues}{\code{TrainLayer$getTargetValues()}}
\item \href{#method-TrainLayer-getIndIDs}{\code{TrainLayer$getIndIDs()}}
\item \href{#method-TrainLayer-getTestData}{\code{TrainLayer$getTestData()}}
\item \href{#method-TrainLayer-getLrner}{\code{TrainLayer$getLrner()}}
\item \href{#method-TrainLayer-getVarSel}{\code{TrainLayer$getVarSel()}}
\item \href{#method-TrainLayer-getModel}{\code{TrainLayer$getModel()}}
\item \href{#method-TrainLayer-checkLrnerExist}{\code{TrainLayer$checkLrnerExist()}}
\item \href{#method-TrainLayer-checkModelExist}{\code{TrainLayer$checkModelExist()}}
\item \href{#method-TrainLayer-checkVarSelExist}{\code{TrainLayer$checkVarSelExist()}}
\item \href{#method-TrainLayer-checkTrainDataExist}{\code{TrainLayer$checkTrainDataExist()}}
\item \href{#method-TrainLayer-summary}{\code{TrainLayer$summary()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="add2HashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-add2HashTable'><code>fuseMLR::HashTable$add2HashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="checkClassExist"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-checkClassExist'><code>fuseMLR::HashTable$checkClassExist()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getFromHashTable'><code>fuseMLR::HashTable$getFromHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getHashTable'><code>fuseMLR::HashTable$getHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getId"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getId'><code>fuseMLR::HashTable$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getKeyClass"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getKeyClass'><code>fuseMLR::HashTable$getKeyClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="removeFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-removeFromHashTable'><code>fuseMLR::HashTable$removeFromHashTable()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-new"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-new}{}}}
\subsection{Method \code{new()}}{
constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$new(id, training)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character}\cr
Training layer id.}

\item{\code{training}}{\code{Training}\cr}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-print"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-getTraining"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-getTraining}{}}}
\subsection{Method \code{getTraining()}}{
Getter of the current training object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$getTraining()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The current training object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-getTargetObj"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-getTargetObj}{}}}
\subsection{Method \code{getTargetObj()}}{
Getter of the target object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$getTargetObj()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-train"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-train}{}}}
\subsection{Method \code{train()}}{
Trains the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$train(ind_subset = NULL, use_var_sel = FALSE, verbose = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ind_subset}}{\code{vector} \cr
ID subset of individuals to be used for training.}

\item{\code{use_var_sel}}{\code{boolean} \cr
If TRUE, variable selection is performed before training.}

\item{\code{verbose}}{\code{boolean} \cr
Warning messages will be displayed if set to TRUE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The current layer is returned with the resulting model.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-varSelection"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-varSelection}{}}}
\subsection{Method \code{varSelection()}}{
Variable selection on the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$varSelection(ind_subset = NULL, verbose = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ind_subset}}{\code{vector} \cr
ID subset of individuals to be used for variable selection.}

\item{\code{verbose}}{\code{boolean} \cr
Warning messages will be displayed if set to TRUE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The current layer is returned with the resulting model.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-predict"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-predict}{}}}
\subsection{Method \code{predict()}}{
Predicts values for the new layer taking as argument.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$predict(new_layer, use_var_sel, ind_subset = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{new_layer}}{\code{TrainLayer} \cr}

\item{\code{use_var_sel}}{\code{boolean} \cr
If TRUE, selected variables available at each layer are used.}

\item{\code{ind_subset}}{\code{vector} \cr}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new \link{PredictLayer} object with the predicted data is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-getTrainData"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-getTrainData}{}}}
\subsection{Method \code{getTrainData()}}{
Getter of the training dataset stored on the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$getTrainData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{TrainData} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-getTargetValues"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-getTargetValues}{}}}
\subsection{Method \code{getTargetValues()}}{
Getter of target values from the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$getTargetValues()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{data.frame} containing individuals IDs and corresponding target values.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-getIndIDs"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-getIndIDs}{}}}
\subsection{Method \code{getIndIDs()}}{
Getter of IDS from the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$getIndIDs()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{data.frame} containing individuals IDs values.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-getTestData"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-getTestData}{}}}
\subsection{Method \code{getTestData()}}{
Getter of the new data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$getTestData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{TestData} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-getLrner"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-getLrner}{}}}
\subsection{Method \code{getLrner()}}{
Getter of the learner.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$getLrner()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{Lrner} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-getVarSel"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-getVarSel}{}}}
\subsection{Method \code{getVarSel()}}{
Getter of the variable selector.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$getVarSel()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{VarSel} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-getModel"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-getModel}{}}}
\subsection{Method \code{getModel()}}{
Getter of the model.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$getModel()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{Model} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-checkLrnerExist"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-checkLrnerExist}{}}}
\subsection{Method \code{checkLrnerExist()}}{
Check whether a learner has been already stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$checkLrnerExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-checkModelExist"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-checkModelExist}{}}}
\subsection{Method \code{checkModelExist()}}{
Check whether a model has been already stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$checkModelExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-checkVarSelExist"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-checkVarSelExist}{}}}
\subsection{Method \code{checkVarSelExist()}}{
Check whether a variable selection tool has been already stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$checkVarSelExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-checkTrainDataExist"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-checkTrainDataExist}{}}}
\subsection{Method \code{checkTrainDataExist()}}{
Check whether a training data has been already stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$checkTrainDataExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TrainLayer-summary"></a>}}
\if{latex}{\out{\hypertarget{method-TrainLayer-summary}{}}}
\subsection{Method \code{summary()}}{
Generate summary.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TrainLayer$summary()}\if{html}{\out{</div>}}
}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bestLayerLearner.R
\name{bestLayerLearner}
\alias{bestLayerLearner}
\title{The best layer-specific model is used as meta model.}
\usage{
bestLayerLearner(x, y, perf = NULL)
}
\arguments{
\item{x}{\code{data.frame} \cr
\code{data.frame} of predictors.}

\item{y}{\code{vector} \cr
True target observations. Either binary or two level factor variable.}

\item{perf}{\code{function} \cr
Function to compute layer-specific performance of learners. If NULL, the Brier Score (classification) or a mean squared error (regression) is used by default as performance measure.
Otherwise, the performance function must accept two parameters: \code{observed} (observed values) and \code{predicted} (predicted values).}
}
\value{
A model object of class \code{weightedMeanLeaner}.
}
\description{
The meta learner is the best layer-specific learner. This function is intended to be (internally) used as meta-learner in fuseMLR.
}
\examples{
set.seed(20240624L)
x = data.frame(x1 = runif(n = 50L, min = 0, max = 1))
y = sample(x = 0L:1L, size = 50L, replace = TRUE)
my_best_model = bestLayerLearner(x = x, y = y)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainingFunctions.R
\name{fusemlr}
\alias{fusemlr}
\title{fusemlr}
\usage{
fusemlr(
  training,
  ind_subset = NULL,
  use_var_sel = FALSE,
  resampling_method = NULL,
  resampling_arg = list(),
  seed = NULL
)
}
\arguments{
\item{training}{\code{Training} \cr
Training object for storing training layers.}

\item{ind_subset}{\code{vector} \cr
ID subset to be used for training.}

\item{use_var_sel}{\code{boolean} \cr
If \code{TRUE} and no variable selection has been performed for the provide training object,
then a variable selection will proceed the training.
Otherwise, if variable selection has been previously performed, the selected variables will be used for training.}

\item{resampling_method}{\code{function} \cr
Function for internal validation. If not specify, the \code{resampling} function
from the package \code{caret} is used for a 10-folds cross-validation.}

\item{resampling_arg}{\code{list} \cr
List of arguments to be passed to the function.}

\item{seed}{\code{integer} \cr
Random seed used for resampling. Default is NULL, which generates the seed from \code{R}.}
}
\value{
The current object is returned, with each learner trained on each layer.
}
\description{
Trains the \link{Training} object passed as argument. A training object must contain
the training layers and a training meta-layer. A training layer encapsulates
data modalities, a variable selection method and a learner. Use the function
\link{createTraining} to create a training object, \link{createTrainLayer} to add training
layers to the created training object, and \link{createTrainMetaLayer} to add a meta-layer
with the corresponding meta-learner to the training object. The function \code{fusemlr}
is designed to train all training layers and the meta-learner. After training
the layer-specific base models and the meta-model will be stored in the training
object which can be used for predictions.
}
\references{
Fouodo C.J.K, Bleskina M. and Szymczak S. (2024). fuseMLR: An R package for integrative prediction modeling of multi-omics data, paper submitted.
}
\seealso{
\link{createTrainLayer}, \link{createTrainMetaLayer}, \link{extractModel} and \link{extractData}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TestLayer.R
\name{TestLayer}
\alias{TestLayer}
\title{TestLayer Class}
\description{
This class implements a layer. A \link{TestLayer} object can only exist as a component of a \link{Predicting} object.

A predicted layer can only contain \link{TestData}.
}
\seealso{
\link{Training}, \link{Lrner}, \link{TrainData}, \link{TestData} and \link{Model}
}
\section{Super class}{
\code{\link[fuseMLR:HashTable]{fuseMLR::HashTable}} -> \code{TestLayer}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-TestLayer-new}{\code{TestLayer$new()}}
\item \href{#method-TestLayer-print}{\code{TestLayer$print()}}
\item \href{#method-TestLayer-getTesting}{\code{TestLayer$getTesting()}}
\item \href{#method-TestLayer-getIndIDs}{\code{TestLayer$getIndIDs()}}
\item \href{#method-TestLayer-getTestData}{\code{TestLayer$getTestData()}}
\item \href{#method-TestLayer-checkTestDataExist}{\code{TestLayer$checkTestDataExist()}}
\item \href{#method-TestLayer-summary}{\code{TestLayer$summary()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="add2HashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-add2HashTable'><code>fuseMLR::HashTable$add2HashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="checkClassExist"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-checkClassExist'><code>fuseMLR::HashTable$checkClassExist()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getFromHashTable'><code>fuseMLR::HashTable$getFromHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getHashTable'><code>fuseMLR::HashTable$getHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getId"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getId'><code>fuseMLR::HashTable$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getKeyClass"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getKeyClass'><code>fuseMLR::HashTable$getKeyClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="removeFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-removeFromHashTable'><code>fuseMLR::HashTable$removeFromHashTable()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestLayer-new"></a>}}
\if{latex}{\out{\hypertarget{method-TestLayer-new}{}}}
\subsection{Method \code{new()}}{
constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestLayer$new(id, testing)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character}\cr
Testing layer id.}

\item{\code{testing}}{\code{Testing}\cr}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestLayer-print"></a>}}
\if{latex}{\out{\hypertarget{method-TestLayer-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestLayer$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestLayer-getTesting"></a>}}
\if{latex}{\out{\hypertarget{method-TestLayer-getTesting}{}}}
\subsection{Method \code{getTesting()}}{
Getter of the current Testing object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestLayer$getTesting()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The current Testing object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestLayer-getIndIDs"></a>}}
\if{latex}{\out{\hypertarget{method-TestLayer-getIndIDs}{}}}
\subsection{Method \code{getIndIDs()}}{
Getter of IDS from the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestLayer$getIndIDs()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{data.frame} containing individuals IDs values.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestLayer-getTestData"></a>}}
\if{latex}{\out{\hypertarget{method-TestLayer-getTestData}{}}}
\subsection{Method \code{getTestData()}}{
Getter of the predicted data stored on the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestLayer$getTestData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{TestData} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestLayer-checkTestDataExist"></a>}}
\if{latex}{\out{\hypertarget{method-TestLayer-checkTestDataExist}{}}}
\subsection{Method \code{checkTestDataExist()}}{
Check whether a new data has been already stored.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestLayer$checkTestDataExist()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Boolean value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TestLayer-summary"></a>}}
\if{latex}{\out{\hypertarget{method-TestLayer-summary}{}}}
\subsection{Method \code{summary()}}{
Generate summary.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TestLayer$summary()}\if{html}{\out{</div>}}
}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainingFunctions.R
\name{upsetplot}
\alias{upsetplot}
\title{upsetplot}
\usage{
upsetplot(object, ...)
}
\arguments{
\item{object}{\verb{Training or Testing} \cr
Training or testing object for each the upset plot will be created.}

\item{...}{\code{any} \cr
Further arguments to be passed to the \code{upset} function from package \code{UpSetR}.}
}
\description{
An upset plot of overlapping individuals.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createWeights.R
\name{createWeights}
\alias{createWeights}
\title{Create weights for COBRA Predictions}
\usage{
createWeights(train, test, n_train, n_test, nlearners, eps, alpha)
}
\arguments{
\item{train}{A \code{matrix} representing the training data. Rows represent observations,
and columns contain predictions from individual learners for these observations.
In cases where a prediction is unavailable for a specific observation, \code{NA} is used.}

\item{test}{A \code{matrix} representing the test data. Rows represent observations,
and columns contain predictions from individual learners for these observations.
In cases where a prediction is unavailable for a specific observation, \code{NA} is used.}

\item{n_train}{An \code{integer} specifying the number of training observations.}

\item{n_test}{An \code{integer} specifying the number of test observations.}

\item{nlearners}{An \code{integer} representing the number of learners.}

\item{eps}{A \code{numeric} value representing the threshold for proximity between two predictions.}

\item{alpha}{A value that determines the optimal number of learners in the neighborhood (only for alpha optimization).}
}
\description{
The \code{createWeights} function is used to calculate weights for predictions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainingFunctions.R
\name{varSelection}
\alias{varSelection}
\title{varSelection}
\usage{
varSelection(training, ind_subset = NULL)
}
\arguments{
\item{training}{\code{Training} \cr
Training object for storing the created layer.}

\item{ind_subset}{\code{vector} \cr
ID subset of individuals to be used for variable selection.}
}
\value{
A \code{data.frame} with two columns: layer and selected variables.
}
\description{
Variable selection on the training object passed as argument.
}
\references{
Fouodo, C. J. K., Bleskina, M. & Szymczak, S. fuseMLR: an R package for integrative prediction modeling of multi-omics data. BMC Bioinformatics 26, 221 (2025). https://doi.org/10.1186/s12859-025-06248-4 \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainingFunctions.R
\name{createTrainLayer}
\alias{createTrainLayer}
\title{createTrainLayer}
\usage{
createTrainLayer(
  training,
  train_layer_id,
  train_data,
  varsel_package = NULL,
  varsel_fct = NULL,
  varsel_param = list(),
  lrner_package = NULL,
  lrn_fct,
  param_train_list = list(),
  param_pred_list = list(),
  na_action = "na.rm",
  x_varsel = "x",
  y_varsel = "y",
  x_lrn = "x",
  y_lrn = "y",
  object = "object",
  data = "data",
  extract_pred_fct = NULL,
  extract_var_fct = NULL
)
}
\arguments{
\item{training}{\code{Training} \cr
Training object for storing the created layer.}

\item{train_layer_id}{\code{character} \cr
ID of the \link{TrainLayer} to be created.}

\item{train_data}{\code{data.frame} \cr
Data modality to be stored on the layer.}

\item{varsel_package}{\code{character} \cr
Package name containing the variable selection algorithm function.
Defaults to \code{NULL} if the function exists in the current working environment.\cr}

\item{varsel_fct}{\code{character} \cr
Variable selection function name. Default value is \code{NULL} for no variable selection.
If specified, the function must accept at least two parameters: \code{x} (predictors)
and \code{y} (response values), and return a vector of selected variables.
Alternatively, use the interface parameters \code{x_varsel} and \code{y_varsel} to map
the original argument names, and \code{extract_var_fct} to specify how to extract
the vector of selected variables. An exception is made for the \code{Boruta} function,
which includes an internal adjustment and requires no additional modifications.}

\item{varsel_param}{\code{list} \cr
List of arguments to be passed to \code{varsel_fct}.}

\item{lrner_package}{\code{character} \cr
Name of the package containing the learning algorithm function. Defaults to
\code{NULL} if the function is available in the current working environment.}

\item{lrn_fct}{\code{character} \cr
Name of the learning function. The function must accept at least two parameters:
\code{x} (predictors) and \code{y} (response values) and return a model. Alternatively,
use the interface parameters \code{x_lrn} and \code{y_lrn} to map these names to the
original arguments in your function.
The returned model must support the generic \code{predict} function (with arguments
\code{object} and \code{data}) to generate predictions for new data. Predictions should
be either a vector or a \code{list} containing a vector named \code{predictions} with
the predicted values.\cr
If the arguments \code{object} and \code{data} have different names in your \code{predict}
function, use the interface parameters below to map them to the original names.
Additionally, if predictions are stored as a \code{matrix} or \code{data.frame}
(e.g., predicted probabilities for dichotomous classification), only the second
column (assumed to be class \code{1} probabilities) will be used.
If the predicted values are not returned in one of the formats mentioned above,
use the \code{extract_pred_fct} argument below to specify how to extract the predicted
values from the prediction object.}

\item{param_train_list}{\code{character} \cr
List of arguments to be passed to \code{lrn_fct}.}

\item{param_pred_list}{\code{character} \cr
List of arguments to be passed to \code{predict} when generating predictions.}

\item{na_action}{\code{character}\cr
Handling of missing values in data during training. Set to \code{"na.keep"} to retain
missing values, or \code{"na.rm"} to remove instances with missing values.}

\item{x_varsel}{\code{character} \cr
If the name of the argument used by the provided original variable selection function to pass
the matrix of independent variable is not \code{x}, use this argument to specify how it is called in the provided function.}

\item{y_varsel}{\code{character} \cr
If the name of the argument used by the provided original variable selection function to pass
the target variable is not \code{y}, use this argument to specify how it is called in the provided function.}

\item{x_lrn}{\code{character} \cr
If the name of the argument used by the provided original learning function to pass
the matrix of independent variable is not \code{x}, use this argument to specify how it is called in the provided function.}

\item{y_lrn}{\code{character} \cr
If the name of the argument used by the provided original learning function to pass
the target variable is not \code{y}, use this argument to specify how it is called in the provided function.}

\item{object}{\code{character} \cr
The generic function \code{predict} uses the parameter \code{object} to pass a model.
If the corresponding argument is named differently in your \code{predict} function, specify its name.}

\item{data}{\code{character} \cr
The generic function \code{predict} uses a parameter \code{data} to pass new data.
If the corresponding argument is named differently in your predict function, specify the name.}

\item{extract_pred_fct}{\verb{character or function} \cr
If the \code{predict} function called for the model does not return a vector,
use this argument to specify a function (or the name of a function) to extract
the vector of predictions. The default value is \code{NULL} if predictions are returned as a vector.}

\item{extract_var_fct}{\verb{character or function} \cr
If the variable selection function does not return a vector, use this argument
to specify a function (or the name of a function) to extract the vector of selected variables.}
}
\value{
The updated \link{Training} object (with the new layer) is returned.
}
\description{
Creates and stores a \link{TrainLayer} on the \link{Training} object passed as argument.
The main components of a training layer are training data modality, a variable
selection methods, and a modality-specific learner.
}
\references{
Fouodo, C. J. K., Bleskina, M. & Szymczak, S. fuseMLR: an R package for integrative prediction modeling of multi-omics data. BMC Bioinformatics 26, 221 (2025). https://doi.org/10.1186/s12859-025-06248-4
}
\seealso{
\link{createTrainMetaLayer} and \link{fusemlr}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trainingFunctions.R
\name{predict.Training}
\alias{predict.Training}
\title{predict.Training}
\usage{
\method{predict}{Training}(object, testing, ind_subset = NULL, ...)
}
\arguments{
\item{object}{\code{Training} \cr
A trained Training object to be used to compute predictions.}

\item{testing}{\code{Testing} \cr
A new testing object to be predicted.}

\item{ind_subset}{\code{vector} \cr
Vector of IDs to be predicted.}

\item{...}{\code{any} \cr
Further arguments passed to or from other methods.}
}
\value{
The final predicted object. All layers and the meta layer are predicted.
}
\description{
Computes predictions for the \link{Testing} object passed as argument.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multi_omics.R
\docType{data}
\name{multi_omics}
\alias{multi_omics}
\title{Simulated multiomics data for 70 training participants and 23 testing participants,
each with an effect size of 20 on each layer. Each layer includes 50 participants for
training and 20 for testing. Participants do not perfectly overlap across layers.
The simulation is based on the R package \code{interSIM}.}
\format{
A list with training and testing data contaning methylation, gene expressions and protein expressions data.
}
\usage{
data(multi_omics)
}
\description{
The dataset is a list containing training and testing data,
called \code{training} and \code{testing} respectively. Each data is a list
containing the following multi_omics at each layer.
}
\details{
\itemize{
\item \code{methylation}: A \code{data.frame} containing the simulated methylation dataset.
\item \code{genexpr}    : A \code{data.frame} containing the gene expression dataset.
\item \code{proteinexpr}: A \code{data.frame} containing the protein expression dataset.
\item \code{target}: A \code{data.frame} with two columns, containing patient IDs and values of target variable.
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testingFunctions.R
\name{createTesting}
\alias{createTesting}
\title{createTesting}
\usage{
createTesting(id, ind_col, verbose = TRUE)
}
\arguments{
\item{id}{\code{character} \cr
Testing id.}

\item{ind_col}{\code{character} \cr
Name of column of individuals IDs in testing \code{data.frame}.}

\item{verbose}{\code{boolean} \cr
Warning messages will be displayed if set to \code{TRUE}.}
}
\value{
A \link{Testing} object.
}
\description{
Creates a \link{Testing} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PredictMetaLayer.R
\name{PredictMetaLayer}
\alias{PredictMetaLayer}
\title{PredictMetaLayer Class}
\description{
This class implement a predicted meta layer. A \link{PredictMetaLayer} can only exist as unique element of a \link{Training} object.

A predicted meta layer can only contain a \link{PredictData} object.
}
\section{Super class}{
\code{\link[fuseMLR:HashTable]{fuseMLR::HashTable}} -> \code{PredictMetaLayer}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-PredictMetaLayer-new}{\code{PredictMetaLayer$new()}}
\item \href{#method-PredictMetaLayer-print}{\code{PredictMetaLayer$print()}}
\item \href{#method-PredictMetaLayer-getPredicting}{\code{PredictMetaLayer$getPredicting()}}
\item \href{#method-PredictMetaLayer-getIndIDs}{\code{PredictMetaLayer$getIndIDs()}}
\item \href{#method-PredictMetaLayer-getPredictData}{\code{PredictMetaLayer$getPredictData()}}
\item \href{#method-PredictMetaLayer-openAccess}{\code{PredictMetaLayer$openAccess()}}
\item \href{#method-PredictMetaLayer-closeAccess}{\code{PredictMetaLayer$closeAccess()}}
\item \href{#method-PredictMetaLayer-getAccess}{\code{PredictMetaLayer$getAccess()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="add2HashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-add2HashTable'><code>fuseMLR::HashTable$add2HashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="checkClassExist"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-checkClassExist'><code>fuseMLR::HashTable$checkClassExist()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getFromHashTable'><code>fuseMLR::HashTable$getFromHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getHashTable'><code>fuseMLR::HashTable$getHashTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getId"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getId'><code>fuseMLR::HashTable$getId()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="getKeyClass"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-getKeyClass'><code>fuseMLR::HashTable$getKeyClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="fuseMLR" data-topic="HashTable" data-id="removeFromHashTable"><a href='../../fuseMLR/html/HashTable.html#method-HashTable-removeFromHashTable'><code>fuseMLR::HashTable$removeFromHashTable()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictMetaLayer-new"></a>}}
\if{latex}{\out{\hypertarget{method-PredictMetaLayer-new}{}}}
\subsection{Method \code{new()}}{
constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictMetaLayer$new(id, predicting)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{\code{character}\cr}

\item{\code{predicting}}{\code{Predicting}\cr}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictMetaLayer-print"></a>}}
\if{latex}{\out{\hypertarget{method-PredictMetaLayer-print}{}}}
\subsection{Method \code{print()}}{
Printer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictMetaLayer$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{\code{any}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictMetaLayer-getPredicting"></a>}}
\if{latex}{\out{\hypertarget{method-PredictMetaLayer-getPredicting}{}}}
\subsection{Method \code{getPredicting()}}{
Getter of the current predicting object
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictMetaLayer$getPredicting()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The current predicting object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictMetaLayer-getIndIDs"></a>}}
\if{latex}{\out{\hypertarget{method-PredictMetaLayer-getIndIDs}{}}}
\subsection{Method \code{getIndIDs()}}{
Getter of IDS from the current layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictMetaLayer$getIndIDs()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A \code{data.frame} containing individuals IDs values.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictMetaLayer-getPredictData"></a>}}
\if{latex}{\out{\hypertarget{method-PredictMetaLayer-getPredictData}{}}}
\subsection{Method \code{getPredictData()}}{
Getter of the predicted data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictMetaLayer$getPredictData()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The stored \link{PredictData} object is returned.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictMetaLayer-openAccess"></a>}}
\if{latex}{\out{\hypertarget{method-PredictMetaLayer-openAccess}{}}}
\subsection{Method \code{openAccess()}}{
Open access to the meta layer. A meta learner is only
modifiable if the access is opened.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictMetaLayer$openAccess()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictMetaLayer-closeAccess"></a>}}
\if{latex}{\out{\hypertarget{method-PredictMetaLayer-closeAccess}{}}}
\subsection{Method \code{closeAccess()}}{
Close access to the meta layer to avoid accidental
modification.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictMetaLayer$closeAccess()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-PredictMetaLayer-getAccess"></a>}}
\if{latex}{\out{\hypertarget{method-PredictMetaLayer-getAccess}{}}}
\subsection{Method \code{getAccess()}}{
Getter of the current access to the meta layer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{PredictMetaLayer$getAccess()}\if{html}{\out{</div>}}
}

}
}
